// File           : FrameworkHelper.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2006
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Win32;
using System.Globalization;

namespace Environment
{
    /// <summary>
    /// FrameworkHelper is a class that helps with .NET Framework and .NET Framework Language Pack installation information
    /// </summary>
    public class FrameworkHelper
    {
        /// <summary>
        /// GetFrameworkPath gets the path of the .NET Framework
        /// </summary>
        /// <returns>The path of the .NET Framework</returns>
        public static string GetFrameworkPath()
        {
            string sourcesKeyName = @"Software\Microsoft\.NETFramework";
            RegistryKey frameworkKey = Registry.LocalMachine.OpenSubKey(sourcesKeyName);

            if (frameworkKey == null)
                return null;

            object rootValue = frameworkKey.GetValue(@"InstallRoot");
            if (rootValue == null)
                return null;

            return rootValue.ToString();
        }
        /// <summary>
        /// GetLanguagePacksInstalled gets an array of LCIDs of language packs installed
        /// </summary>
        /// <returns>An array of LCIDs of language packs installed</returns>
        public static int[] GetLanguagePacksInstalled()
        {
            string frameworkVersion = System.Environment.Version.Major.ToString() + "." + System.Environment.Version.Minor.ToString() +
                "." + System.Environment.Version.Build.ToString();
            string sourcesKeyName = @"Software\Microsoft\NET Framework Setup\NDP\v" + frameworkVersion;
            RegistryKey frameworkKey = Registry.LocalMachine.OpenSubKey(sourcesKeyName);

            if (frameworkKey == null)
                return new int[] { };

            string[] subKeyNames = frameworkKey.GetSubKeyNames();
            if (subKeyNames.GetLength(0) == 1)
                // The list of installed .NET Framework Language Packs
                // contains a single entry. This entry is always 1033 ("en-US")
                // which is the fallback assembly and not a language pack per se.
                return new int[] { };

            int[] languagePackLCIDs = new int[subKeyNames.GetLength(0) - 1];
            int languagePackNumber = 0;
            foreach (string subKeyName in subKeyNames)
            {
                int LCID = Int32.Parse(subKeyName);
                // ignore LCID 1033 ("en-US") because it is the fallback
                // language and not a language pack per se
                if (LCID != 1033)
                {
                    languagePackLCIDs[languagePackNumber++] = LCID;
                }
            }

            return languagePackLCIDs;
        }
        /// <summary>
        /// IsLanguagePackInstalled returns true if the .NET Framework Language Pack for the given culture is installed
        /// </summary>
        /// <param name="cultureName">The name of the culture to test the installation of a .NET Framework Language Pack for</param>
        /// <returns>True if the .NET Framework Language Pack for the given culture is installed</returns>
        /// <remarks>The cultureName must represent a specific culture and not a neutral culture.</remarks>
        public static bool IsLanguagePackInstalled(string cultureName)
        {
            CultureInfo cultureInfo = new CultureInfo(cultureName);
            return IsLanguagePackInstalled(cultureInfo.LCID);
        }
        /// <summary>
        /// IsLanguagePackInstalled returns true if the .NET Framework Language Pack for the given culture is installed
        /// </summary>
        /// <param name="LCID">The LCID of the culture to test the installation of a .NET Framework Language Pack for</param>
        /// <returns>True if the .NET Framework Language Pack for the given culture is installed</returns>
        /// <remarks>The LCID must represent a specific culture and not a neutral culture.</remarks>
        public static bool IsLanguagePackInstalled(int LCID)
        {
            string frameworkVersion = System.Environment.Version.Major.ToString() + "." + System.Environment.Version.Minor.ToString() +
                "." + System.Environment.Version.Build.ToString();
            string sourcesKeyName = @"Software\Microsoft\NET Framework Setup\NDP\v"
                + frameworkVersion + @"\" + LCID.ToString();
            RegistryKey frameworkKey = Registry.LocalMachine.OpenSubKey(sourcesKeyName);

            if (frameworkKey == null)
                return false;

            object rootValue = frameworkKey.GetValue("Install");
            if (rootValue == null)
                return false;

            return rootValue.ToString() == "1";
        }
    }
}
