// File           : ConvertResX.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2006
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Resources;
using System.Xml;
using System.Reflection;
using System.Globalization;
using System.ComponentModel.Design;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.Serialization.Formatters.Binary;
using System.ComponentModel;
using System.Collections;
using ConvertResX; // used for ConvertResXResources strongly typed resource class

namespace Internationalization.Tools
{
    /// <summary>
    /// ConvertResX converts .NET Framework 2.0 .resx files to .NET Framework 1.1 .resx files
    /// </summary>
    /// <remarks>Backup your .resx files before using this utility.
    /// Known limitations:-
    ///     At present this class does not convert Audio and File resources.</remarks>
    public class ConvertResX
    {
        private const string version = "1.0.5000.0";

        /// <summary>
        /// Convert converts a .NET Framework 2.0 .resx file to a .NET Framework 1.1 .resx file
        /// </summary>
        /// <param name="inputResX">The .NET Framework 2.0 .resx file to convert to .NET Framework 1.1</param>
        /// <param name="outputResX">The output filename of the .NET Framework 1.1 .resx file.
        /// If this file exists it is overwritten.</param>
        public static void Convert(string inputResX, string outputResX)
        {
            XmlTextWriter writer = new XmlTextWriter(outputResX, Encoding.UTF8);
            writer.Formatting = Formatting.Indented;
            writer.Indentation = 2;
            writer.IndentChar = ' ';
            try
            {
                writer.WriteStartDocument();
                writer.WriteStartElement("root");
                WriteComments(writer);
                WriteSchema(writer);
                WriteHeaderElements(writer);

                using (ResXResourceReader reader = new ResXResourceReader(inputResX))
                {
                    reader.BasePath = Path.GetFullPath(inputResX);
                    reader.UseResXDataNodes = true;
                    reader.BasePath = Path.GetDirectoryName(inputResX);
                    // TODO: Support ResXFileRefs
                    IDictionaryEnumerator enumerator = reader.GetEnumerator();
                    while (enumerator.MoveNext())
                    {
                        writer.WriteStartElement("data");
                        writer.WriteAttributeString("name", enumerator.Key.ToString());
                        ResXDataNode dataNode = (ResXDataNode)enumerator.Value;
                        object value = dataNode.GetValue((ITypeResolutionService) null);
                        if (value.GetType() != typeof(string))
                        {
                            writer.WriteAttributeString("type", ChangeTypeVersion(value.GetType()));
                            // TODO: Support Audio
                            if (value.GetType() == typeof(Bitmap) || value.GetType() == typeof(Icon))
                                writer.WriteAttributeString("mimetype", "application/x-microsoft.net.object.bytearray.base64");
                        }
                        writer.WriteStartElement("value");
                        if (value.GetType() == typeof(string) && enumerator.Key.ToString().EndsWith(".Type"))
                            writer.WriteValue(ChangeTypeVersion(value.ToString()));
                        else
                            WriteValue(writer, value);

                        writer.WriteEndElement();

                        if (!String.IsNullOrEmpty(dataNode.Comment))
                            // write the comment
                            writer.WriteElementString("Comment", dataNode.Comment);

                        writer.WriteEndElement();
                    }
                }
                writer.WriteEndElement();
                writer.WriteEndDocument();
            }
            finally
            {
                writer.Close();
            }
        }
        private static void WriteSchema(XmlTextWriter writer)
        {
            /* Write the following schema:-
<xsd:schema id="root" xmlns="" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:msdata="urn:schemas-microsoft-com:xml-msdata">
  <xsd:element name="root" msdata:IsDataSet="true">
    <xsd:complexType>
      <xsd:choice maxOccurs="unbounded">
        <xsd:element name="data">
          <xsd:complexType>
            <xsd:sequence>
              <xsd:element name="value" type="xsd:string" minOccurs="0" msdata:Ordinal="1" />
              <xsd:element name="comment" type="xsd:string" minOccurs="0" msdata:Ordinal="2" />
            </xsd:sequence>
            <xsd:attribute name="name" type="xsd:string" msdata:Ordinal="1" />
            <xsd:attribute name="type" type="xsd:string" msdata:Ordinal="3" />
            <xsd:attribute name="mimetype" type="xsd:string" msdata:Ordinal="4" />
          </xsd:complexType>
        </xsd:element>
        <xsd:element name="resheader">
          <xsd:complexType>
            <xsd:sequence>
              <xsd:element name="value" type="xsd:string" minOccurs="0" msdata:Ordinal="1" />
            </xsd:sequence>
            <xsd:attribute name="name" type="xsd:string" use="required" />
          </xsd:complexType>
        </xsd:element>
      </xsd:choice>
    </xsd:complexType>
  </xsd:element>
</xsd:schema>
             */

            // <xsd:schema id="root" xmlns="" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:msdata="urn:schemas-microsoft-com:xml-msdata">
            writer.WriteStartElement("xsd:schema");
            writer.WriteAttributeString("id", "root");
            writer.WriteAttributeString("xmlns", "");
            writer.WriteAttributeString("xmlns:xsd", "http://www.w3.org/2001/XMLSchema");
            writer.WriteAttributeString("xmlns:msdata", "urn:schemas-microsoft-com:xml-msdata");

            // <xsd:element name="root" msdata:IsDataSet="true">
            writer.WriteStartElement("xsd:element");
            writer.WriteAttributeString("name", "root");
            writer.WriteAttributeString("msdata:IsDataSet", "true");

            // <xsd:complexType>
            writer.WriteStartElement("xsd:complexType");

            // <xsd:choice maxOccurs="unbounded">
            writer.WriteStartElement("xsd:choice");
            writer.WriteAttributeString("maxOccurs", "unbounded");

            // <xsd:element name="data">
            writer.WriteStartElement("xsd:element");
            writer.WriteAttributeString("name", "data");

            // <xsd:complexType>
            writer.WriteStartElement("xsd:complexType");

            // <xsd:sequence>
            writer.WriteStartElement("xsd:sequence");

            // <xsd:element name="value" type="xsd:string" minOccurs="0" msdata:Ordinal="1" />
            writer.WriteStartElement("xsd:element");
            writer.WriteAttributeString("name", "value");
            writer.WriteAttributeString("type", "xsd:string");
            writer.WriteAttributeString("minOccurs", "0");
            writer.WriteAttributeString("msdata:Ordinal", "1");
            writer.WriteEndElement();


            // <xsd:element name="comment" type="xsd:string" minOccurs="0" msdata:Ordinal="2" />
            writer.WriteStartElement("xsd:element");
            writer.WriteAttributeString("name", "comment");
            writer.WriteAttributeString("type", "xsd:string");
            writer.WriteAttributeString("minOccurs", "0");
            writer.WriteAttributeString("msdata:Ordinal", "2");
            writer.WriteEndElement();

            // end "xsd:sequence"
            writer.WriteEndElement();

            // <xsd:attribute name="name" type="xsd:string" msdata:Ordinal="1" />
            writer.WriteStartElement("xsd:attribute");
            writer.WriteAttributeString("name", "name");
            writer.WriteAttributeString("type", "xsd:string");
            writer.WriteAttributeString("msdata:Ordinal", "1");
            writer.WriteEndElement();

            // <xsd:attribute name="type" type="xsd:string" msdata:Ordinal="3" />
            writer.WriteStartElement("xsd:attribute");
            writer.WriteAttributeString("name", "type");
            writer.WriteAttributeString("type", "xsd:string");
            writer.WriteAttributeString("msdata:Ordinal", "3");
            writer.WriteEndElement();

            // <xsd:attribute name="mimetype" type="xsd:string" msdata:Ordinal="4" />
            writer.WriteStartElement("xsd:attribute");
            writer.WriteAttributeString("name", "mimetype");
            writer.WriteAttributeString("type", "xsd:string");
            writer.WriteAttributeString("msdata:Ordinal", "4");
            writer.WriteEndElement();

            // end "xsd:complexType"
            writer.WriteEndElement();

            // end "xsd:element"
            writer.WriteEndElement();

            // <xsd:element name="resheader">
            writer.WriteStartElement("xsd:element");
            writer.WriteAttributeString("name", "resheader");

            // <xsd:complexType>
            writer.WriteStartElement("xsd:complexType");

            // <xsd:sequence>
            writer.WriteStartElement("xsd:sequence");

            // <xsd:element name="value" type="xsd:string" minOccurs="0" msdata:Ordinal="1" />
            writer.WriteStartElement("xsd:element");
            writer.WriteAttributeString("name", "value");
            writer.WriteAttributeString("type", "xsd:string");
            writer.WriteAttributeString("minOccurs", "0");
            writer.WriteAttributeString("msdata:Ordinal", "1");
            writer.WriteEndElement();

            // end "xsd:sequence"
            writer.WriteEndElement();

            // <xsd:attribute name="name" type="xsd:string" use="required" />
            writer.WriteStartElement("xsd:attribute");
            writer.WriteAttributeString("name", "name");
            writer.WriteAttributeString("type", "xsd:string");
            writer.WriteAttributeString("use", "required");
            writer.WriteEndElement();

            // end "xsd:complexType"
            writer.WriteEndElement();

            // end "xsd:element"
            writer.WriteEndElement();

            // end "xsd:choice"
            writer.WriteEndElement();

            // end "xsd:complexType"
            writer.WriteEndElement();

            // end "xsd:element"
            writer.WriteEndElement();

            // end "xsd:schema"
            writer.WriteEndElement();
        }
        private static void WriteHeaderElements(XmlTextWriter writer)
        {
            writer.WriteStartElement("resheader");
            writer.WriteAttributeString("name", "resmimetype");
            writer.WriteStartElement("value");
            writer.WriteValue("text/microsoft-resx");
            writer.WriteEndElement();
            writer.WriteEndElement();

            writer.WriteStartElement("resheader");
            writer.WriteAttributeString("name", "version");
            writer.WriteStartElement("value");
            writer.WriteValue("1.3");
            writer.WriteEndElement();
            writer.WriteEndElement();

            writer.WriteStartElement("resheader");
            writer.WriteAttributeString("name", "reader");
            writer.WriteStartElement("value");
            writer.WriteValue("System.Resources.ResXResourceReader, System.Windows.Forms, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            writer.WriteEndElement();
            writer.WriteEndElement();

            writer.WriteStartElement("resheader");
            writer.WriteAttributeString("name", "writer");
            writer.WriteStartElement("value");
            writer.WriteValue("System.Resources.ResXResourceWriter, System.Windows.Forms, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            writer.WriteEndElement();
            writer.WriteEndElement();
        }

        private static void WriteComments(XmlTextWriter writer)
        {
            writer.WriteComment(ConvertResXResources.ResX1Comments);
        }
        private static void WriteBytes(XmlTextWriter writer, Type type, object value)
        {
            TypeConverter typeConverter = TypeDescriptor.GetConverter(type);
            byte[] buffer = (byte[])typeConverter.ConvertTo(value, typeof(Byte[]));
            writer.WriteValue(ToBase64WrappedString(buffer));
        }
        private static string ToBase64WrappedString(byte[] data)
        {
            string base64String = System.Convert.ToBase64String(data, 0, data.Length, Base64FormattingOptions.None);
            if (base64String.Length <= 80)
            {
                return base64String;
            }
            StringBuilder stringBuilder = new StringBuilder(base64String.Length + ((base64String.Length / 80) * 3));
            int stringBuilderStringLength = 0;
            while (stringBuilderStringLength < (base64String.Length - 80))
            {
                stringBuilder.Append("\r\n");
                stringBuilder.Append("        ");
                stringBuilder.Append(base64String, stringBuilderStringLength, 80);
                stringBuilderStringLength += 80;
            }
            stringBuilder.Append("\r\n");
            stringBuilder.Append("        ");
            stringBuilder.Append(base64String, stringBuilderStringLength, base64String.Length - stringBuilderStringLength);
            stringBuilder.Append("\r\n");
            return stringBuilder.ToString();
        }
        private static void WriteValue(XmlTextWriter writer, object value)
        {
            string valueString = value.ToString();
            if (value is Bitmap)
            {
                WriteBytes(writer, typeof(Bitmap), (Bitmap)value);
            }
            else if (value is Icon)
            {
                WriteBytes(writer, typeof(Icon), (Icon)value);
            }
            else if (valueString.StartsWith("{") && valueString.EndsWith("}") && valueString.IndexOf("=") > -1)
            {
                // The value is from a struct or class. The representation needs to be changed to a stream of values
                // For example, Point objects get represented as "{X=248,Y=72}" when they should be "248, 72"
                string[] paramsAndValues = valueString.Substring(1, valueString.Length - 2).Split(',');
                StringBuilder valueBuilder = new StringBuilder();
                int paramNumber = 1;
                foreach (string paramAndValue in paramsAndValues)
                {
                    int equalsIndex = paramAndValue.IndexOf("=");
                    valueBuilder.Append(paramAndValue.Substring(equalsIndex + 1));
                    if (paramNumber != paramsAndValues.GetLength(0))
                        // this is not the last parameter
                        valueBuilder.Append(", ");

                    paramNumber++;
                }
                writer.WriteValue(valueBuilder.ToString());
            }
            else
                writer.WriteValue(valueString);
        }

        private static string ChangeTypeVersion(string typeString)
        {
            // Change a string like this:-
            // "System.Windows.Forms.GroupBox, System.Windows.Forms, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"
            // to this:-
            // "System.Windows.Forms.GroupBox, System.Windows.Forms, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"
            string[] typeParts = typeString.Split(',');
            StringBuilder builder = new StringBuilder();
            int typePartNumber = 1;
            foreach (string typePart in typeParts)
            {
                string typePartValue;
                if (typePart.StartsWith(" Version="))
                    // this is the version number
                    typePartValue = " Version=" + version;
                else
                    typePartValue = typePart;

                if (typePartNumber != typeParts.GetLength(0))
                    builder.Append(typePartValue + ",");
                else
                    builder.Append(typePartValue);

                typePartNumber++;
            }
            return builder.ToString();
        }

        private static string ChangeTypeVersion(Type type)
        {
            // Change a Type like this:-
            // "System.Drawing.Size, System.Drawing, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
            // to this:-
            // "System.Drawing.Size, System.Drawing, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"

            AssemblyName assemblyName = type.Assembly.GetName();
            string culture;
            if (assemblyName.CultureInfo.Equals(CultureInfo.InvariantCulture))
                culture = "neutral";
            else
                culture = assemblyName.CultureInfo.Name;

            return type.ToString() + ", " + assemblyName.Name + ", Version=" + version + ", Culture=" + culture +
                ", PublicKeyToken=" + PublicKeyTokenToString(assemblyName.GetPublicKeyToken());
        }
        private static string PublicKeyTokenToString(byte[] byteArray)
        {
            StringBuilder builder = new StringBuilder();
            foreach (byte byteElement in byteArray)
            {
                builder.Append(String.Format("{0:x2}", byteElement));
            }
            return builder.ToString();
        }
    }
}
