// File           : ExceptionFactory.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;
using System.Threading;
using System.Reflection;

namespace Exceptions
{
	/// <summary>
	/// ExceptionFactory constructs exceptions using a specific culture
	/// </summary>
	/// <remarks>The purpose of using a specific culture to construct an exception is to load resources
	/// for that culture so that exception messages are localized</remarks>
	public class ExceptionFactory
	{
        private static CultureInfo exceptionCulture;
		static ExceptionFactory()
		{
            exceptionCulture = new CultureInfo("en-US");
		}
        /// <summary>
        /// ExceptionCulture is the culture to use for creating exception objects
        /// </summary>
        /// <remarks>The default is "en-US"</remarks>
        public static CultureInfo ExceptionCulture
        {
            get {return exceptionCulture;}
            set {exceptionCulture = value;}
        }
        /// <summary>
        /// oldCultureInfo is the temporary store for the original value of CurrentUICulture
        /// </summary>
        protected static CultureInfo oldCultureInfo;
        /// <summary>
        /// SetExceptionCulture sets the CurrentUICulture to the exception culture
        /// </summary>
        protected static void SetExceptionCulture()
        {
            oldCultureInfo = null;
            if (Thread.CurrentThread.CurrentUICulture.Name != exceptionCulture.Name)
            {
                // change the culture to the exception culture
                oldCultureInfo = Thread.CurrentThread.CurrentUICulture;
                Thread.CurrentThread.CurrentUICulture = exceptionCulture;
            }
        }
        /// <summary>
        /// RestoreCulture restores the CurrentUICulture to its value before SetExceptionCulture was called
        /// </summary>
        protected static void RestoreCulture()
        {
            if (oldCultureInfo != null)
            {
                Thread.CurrentThread.CurrentUICulture = oldCultureInfo;
                oldCultureInfo = null;
            }
        }
        /// <summary>
        /// CreateException creates an exception of the given type using the ExceptionCulture culture
        /// </summary>
        /// <param name="exceptionType">The Type of the exception to create</param>
        /// <returns>An Exception of the given Type</returns>
        public static Exception CreateException(Type exceptionType)
        {
            object exception;
            SetExceptionCulture();
            try
            {
                exception = Activator.CreateInstance(exceptionType);
                if (! (exception is Exception))
                    // do not localize this
                    throw new ArgumentException("exceptionType is not an Exception Type");
            }
            finally
            {
                RestoreCulture();
            }
            return (Exception) exception;
        }
        /// <summary>
        /// CreateException creates an exception of the given type using the ExceptionCulture culture
        /// </summary>
        /// <param name="exceptionType">The Type of the exception to create</param>
        /// <param name="args">An array of objects to pass as arguments to the exception class's constructor</param>
        /// <returns>An Exception of the given Type</returns>
        public static Exception CreateException(Type exceptionType, object[] args)
        {
            object exception;
            SetExceptionCulture();
            try
            {
                exception = Activator.CreateInstance(exceptionType, args);
                if (! (exception is Exception))
                    // do not localize this
                    throw new ArgumentException("exceptionType is not an Exception Type");
            }
            finally
            {
                RestoreCulture();
            }
            return (Exception) exception;
        }
    }
}
