// File           : Internationalization.Web.UI.HotKeyAssignment.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Collections;
using System.Globalization;
using System.Reflection;
using Internationalization;

namespace Internationalization.Web.UI
{
	/// <summary>
	/// HotKeyAssignment assigns hot keys to WebControls
	/// </summary>
	public class HotKeyAssignment: HotKeyAssignmentBase
	{
        static HotKeyAssignment()
        {
            InitializeValidHotKeys();
        }
        /// <summary>
        /// InitializeValidHotKeys initializes the array of valid hot keys (excluding the keys used by the browser)
        /// </summary>
        protected static void InitializeValidHotKeys()
        {
            // TODO: This method should be made specific to both the culture and also the browser
            // being used as different browsers use different hotkeys.

            // In the English Internet Explorer the File, Edit, View, Favorites, Tools, and Help
            // menu items use the F, E, V, A, T and H hot keys.
            ValidHotKeys = new Char[] {
                           'B', 'C', 'D', 'G', 'I', 'J', 'K', 'L', 'M',
                           'N', 'O', 'P', 'Q', 'R', 'S', 'U', 'W', 'X', 'Y', 'Z'
                       };
        }
        /// <summary>
        /// OnControlCanHaveHotKey is a ControlCanHaveHotKeyEventHandler event used to determine if a control can have a hot key
        /// </summary>
        public static event ControlCanHaveHotKeyEventHandler OnControlCanHaveHotKey;

        /// <summary>
        /// CultureFormat returns the HotKeyFormat (respecting the CurrentUICulture if Format is HotKeyFormat.Culture)
        /// </summary>
        public static HotKeyFormat CultureFormat
        {
            get
            {
                if (Format == HotKeyFormat.Culture)
                    return HotKeyFormat.BracketedSuffix;
                else
                    return Format;
            }
        }
        /// <summary>
        /// AssignHotKeys assigns hot keys to all appropriate controls on a page
        /// </summary>
        /// <param name="page">The Page containing the controls to assign hot keys to</param>
        /// <remarks>Existing hot keys are preserved</remarks>
        public static void AssignHotKeys(Page page)
        {
            AssignHotKeys(page, true);
        }
        /// <summary>
        /// AssignHotKeys assigns hot keys to all appropriate controls on a page
        /// </summary>
        /// <param name="page">The Page containing the controls to assign hot keys to</param>
        /// <param name="preserveExistingHotKeys">True if existing hot keys should be preserved</param>
        public static void AssignHotKeys(Page page, bool preserveExistingHotKeys)
        {
            WebControl[] controls = GetControls(GetHtmlForm(page));
            AssignHotKeysToControls(controls, preserveExistingHotKeys);
        }
        /// <summary>
        /// GetControls returns an array of WebControls for the given HtmlForm
        /// </summary>
        /// <param name="form">The HtmlForm containing the controls</param>
        /// <returns>An array of WebControls for the given HtmlForm</returns>
        protected static WebControl[] GetControls(HtmlForm form)
        {
            ArrayList controls = new ArrayList();
            foreach(Control control in form.Controls)
            {
                if (control is WebControl)
                    controls.Add(control);
            }
            return (WebControl[]) controls.ToArray(typeof(WebControl));
        }
        /// <summary>
        /// GetHtmlForm gets the HtmlForm from a Page
        /// </summary>
        /// <param name="page">The Page to get the HtmlForm from</param>
        /// <returns>The HtmlForm from the Page</returns>
        protected static HtmlForm GetHtmlForm(Page page)
        {
            foreach(Control control in page.Controls)
            {
                if (control is HtmlForm)
                    return (HtmlForm) control;
            }
            return null;
        }
        /// <summary>
        /// AssignHotKeysToControls assigns hot keys to an array of WebControls
        /// </summary>
        /// <param name="controls">An array of WebControls</param>
        /// <param name="preserveExistingHotKeys">True if existing hot keys should be preserved</param>
        protected static void AssignHotKeysToControls(WebControl[] controls, bool preserveExistingHotKeys)
        {
            ArrayList controlsNeedingHotKeys = new ArrayList();
            ArrayList assignedCharacters = new ArrayList();

            // get a list of controls which need hot keys assigned to them and 
            // get a list of the keys which have already been assigned to controls
            // where the hot keys must be preserved
            GetControlsNeedingHotKeys(controls, controlsNeedingHotKeys, assignedCharacters, preserveExistingHotKeys);

            // assign any standard hot keys to controls
            AssignStandardHotKeys(controlsNeedingHotKeys);

            // assign "calculated" hot keys to controls
            AssignHotKeysToControlsNeedingHotKeys(controlsNeedingHotKeys, assignedCharacters);
        }
        /// <summary>
        /// GetControlsNeedingHotKeys adds controls which need hot keys to an ArrayList
        /// </summary>
        /// <param name="controls">An array of WebControls</param>
        /// <param name="controlsNeedingHotKeys">The ArrayList to which controls which need hot keys are added</param>
        /// <param name="assignedCharacters">An ArrayList of characters which have already been assigned</param>
        /// <param name="preserveExistingHotKeys">True if existing hot keys should be preserved</param>
        protected static void GetControlsNeedingHotKeys(
            WebControl[] controls, ArrayList controlsNeedingHotKeys, ArrayList assignedCharacters, bool preserveExistingHotKeys)
        {
            foreach(WebControl childControl in controls)
            {
                if (ControlCanHaveHotKey(childControl))
                {
                    if (preserveExistingHotKeys)
                    {
                        char hotKey = GetHotKey(childControl);
                        if (hotKey == NullChar)
                            controlsNeedingHotKeys.Add(childControl);
                        else
                            assignedCharacters.Add(Char.ToUpper(hotKey, CultureInfo.InvariantCulture));
                    }
                    else
                        controlsNeedingHotKeys.Add(childControl);
                }
            }
        }
        /// <summary>
        /// ControlCanHaveHotKey returns True if the given WebControl can have a hot key
        /// </summary>
        /// <param name="control">The WebControl to determine whether a hot key can be assigned</param>
        /// <returns>True if the given WebControl can have a hot key</returns>
        protected static bool ControlCanHaveHotKey(WebControl control)
        {
            bool controlCanHaveHotKey =
                (control is TextBox || control is Button || control is CheckBox || control is RadioButton);

            if (OnControlCanHaveHotKey != null)
            {
                ControlCanHaveHotKeyEventArgs args = new ControlCanHaveHotKeyEventArgs(control, controlCanHaveHotKey);
                OnControlCanHaveHotKey(null, args);
                controlCanHaveHotKey = args.CanHaveHotKey;
            }
            return controlCanHaveHotKey;
        }
        /// <summary>
        /// GetHotKey gets the WebControl's existing hot key assignment
        /// </summary>
        /// <param name="control">The WebControl to get the hot key assignment from</param>
        /// <returns>The WebControl's existing hot key assignment or NullChar if the control does not have a hot key</returns>
        protected static char GetHotKey(WebControl control)
        {
            if (control is TextBox)
            {
                if (control.AccessKey.Length == 1)
                    // the TextBox has its own AccessKey so return it
                    return control.AccessKey[0];
                else
                {
                    // the TextBox does not have its own access key but its
                    // associated Label control (if any) might have an access key
                    Label label = GetAssociatedLabelControl(control);
                    if (label == null)
                        return NullChar;
                    else
                        return GetAccessKey(label);
                }
            }
            else
                return GetAccessKey(control);
        }
        /// <summary>
        /// GetAccessKey gets the WebControl's existing access key assignment
        /// </summary>
        /// <param name="control">The WebControl to get the access key assignment from</param>
        /// <returns>The WebControl's existing access key assignment or NullChar if the control does not have an access key</returns>
        protected static char GetAccessKey(WebControl control)
        {
            if (control.AccessKey.Length == 1)
                return control.AccessKey[0];
            else
                return NullChar;
        }
        /// <summary>
        /// GetAssociatedLabelControl gets the Label control which is associated with the WebControl
        /// </summary>
        /// <param name="control">The Web Control to get the associated Label control for</param>
        /// <returns>The Label control which is associated with the WebControl</returns>
        protected static Label GetAssociatedLabelControl(WebControl control)
        {
            ControlCollection controls = control.Parent.Controls;
            int controlIndex = controls.IndexOf(control);
            while (controlIndex > 0)
            {
                // look at the control immediately before this control
                // to see if it if a WebControl and also a Label
                Control childControl = controls[controlIndex - 1];
                if (childControl is WebControl)
                {
                    if (childControl is Label)
                        return (Label) childControl;
                    else
                        return null;
                }
                controlIndex--;
            }
            return null;
        }
        /// <summary>
        /// AssignStandardHotKeys assigns standard hot keys to an ArrayList of WebControls
        /// </summary>
        /// <param name="controlsNeedingHotKeys">An ArrayList of WebControls</param>
        protected static void AssignStandardHotKeys(ArrayList controlsNeedingHotKeys)
        {
            if (StandardHotKeys.GetLength(0) > 0)
            {
                for(int controlNumber = 0; controlNumber < controlsNeedingHotKeys.Count; controlNumber++)
                {
                    WebControl control = (WebControl) controlsNeedingHotKeys[controlNumber];
                    char hotKey = SelectStandardHotKey(control);
                    if (hotKey != NullChar)
                    {
                        ApplyHotKey(control, hotKey);
                        // remove the control from the list of controls needing a hot key
                        // because this control now has a hot key
                        controlsNeedingHotKeys.RemoveAt(controlNumber);
                        controlNumber--;
                    }
                }
            }
        }
        /// <summary>
        /// GetControlText gets the label text associated with the WebControl
        /// </summary>
        /// <param name="control">The WebControl to get the associated label text for</param>
        /// <returns>The label text associated with the WebControl</returns>
        protected static string GetControlText(WebControl control)
        {
            if (control is TextBox)
            {
                // the TextBox does not have its own Text but its
                // associated Label control (if any) does
                Label label = GetAssociatedLabelControl(control);
                if (label == null)
                    return null;
                else
                    return label.Text;
            }
            else
            {
                // get the value of the Text property
                PropertyInfo propertyInfo = control.GetType().GetProperty("Text");
                if (propertyInfo == null)
                    // it doesn't have a public Text property
                    return null;
                else
                    return (string) propertyInfo.GetValue(control, new object[] {});
            }
        }
        /// <summary>
        /// SelectStandardHotKey gets the hot key which should be used for the given WebControl
        /// </summary>
        /// <param name="control">The WebControl to get the hot key for</param>
        /// <returns>The hot key which should be used for the given WebControl</returns>
        protected static char SelectStandardHotKey(WebControl control)
        {
            string text = GetControlText(control);
            if (text != null && text != String.Empty)
                return SelectStandardHotKey(text);
            else
                return NullChar;
        }
        /// <summary>
        /// ApplyHotKey applies the given hot key to the WebControl
        /// </summary>
        /// <param name="control">The WebControl to apply the hot key to</param>
        /// <param name="hotKey">The hot key to apply to the WebControl</param>
        protected static void ApplyHotKey(WebControl control, char hotKey)
        {
            if (control is TextBox)
            {
                control.AccessKey = hotKey.ToString();
                Label label = GetAssociatedLabelControl(control);
                if (label != null)
                {
                    // remove any existing hotkey from the associated Label control
                    label.AccessKey = String.Empty;
                    // change the label's Text
                    label.Text = ApplyHotKey(label.Text, hotKey, CultureFormat);
                }
            }
            else
            {
                control.AccessKey = hotKey.ToString();

                PropertyInfo propertyInfo = control.GetType().GetProperty("Text");
                if (propertyInfo != null)
                {
                    string text = (string) propertyInfo.GetValue(control, new object[] {});
                    text = ApplyHotKey(text, hotKey, CultureFormat);
                    propertyInfo.SetValue(control, text, new object[] {});
                }
            }
        }
        /// <summary>
        /// AssignHotKeysToControlsNeedingHotKeys assigns hot keys to WebControls which need hot keys assigning to
        /// </summary>
        /// <param name="controlsNeedingHotKeys">An ArrayList of WebControls which need hot keys assigned to</param>
        /// <param name="assignedCharacters">An ArrayList of hot key characters which have already been assigned</param>
        protected static void AssignHotKeysToControlsNeedingHotKeys(ArrayList controlsNeedingHotKeys, ArrayList assignedCharacters)
        {
            foreach(WebControl childControl in controlsNeedingHotKeys)
            {
                char hotKey = SelectHotKey(childControl, assignedCharacters);
                if (hotKey != NullChar)
                {
                    assignedCharacters.Add(hotKey);
                    ApplyHotKey(childControl, hotKey);
                }
            }
        }
        /// <summary>
        /// SelectHotKey returns the hot key which should be used for the given WebControl
        /// </summary>
        /// <param name="control">The WebControl to get the hot key for</param>
        /// <param name="assignedCharacters">An ArrayList of characters which have already been assigned</param>
        /// <returns>The hot key which should be used for the given WebControl</returns>
        protected static char SelectHotKey(WebControl control, ArrayList assignedCharacters)
        {
            string text = GetControlText(control);
            return SelectHotKey(text, assignedCharacters);
        }
    }
    /// <summary>
    /// ControlCanHaveHotKeyEventArgs is an EventArgs for the OnControlCanHaveHotKey event
    /// </summary>
    public class ControlCanHaveHotKeyEventArgs: EventArgs
    {
        private WebControl control;
        private bool canHaveHotKey;
        /// <summary>
        /// Contructs a ControlCanHaveHotKeyEventArgs object
        /// </summary>
        /// <param name="control">The WebControl</param>
        /// <param name="canHaveHotKey">True if the WebControl can have a hot key</param>
        public ControlCanHaveHotKeyEventArgs(WebControl control, bool canHaveHotKey)
        {
            this.control = control;
            this.canHaveHotKey = canHaveHotKey;
        }
        /// <summary>
        /// The WebControl which may or may not have a hot key
        /// </summary>
        public WebControl Control
        {
            get {return control;}
            set {control = value;}
        }
        /// <summary>
        /// CanHaveHotKey is True if the WebControl can have a hot key
        /// </summary>
        public bool CanHaveHotKey
        {
            get {return canHaveHotKey;}
            set {canHaveHotKey = value;}
        }
    }
    /// <summary>
    /// ControlCanHaveHotKeyEventHandler is an EventHandler for the OnControlCanHaveHotKey event
    /// </summary>
    public delegate void ControlCanHaveHotKeyEventHandler(object sender, ControlCanHaveHotKeyEventArgs args);
}
