// File           : Internationalization.Windows.Forms.HotKeyAssignment.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;
using System.Windows.Forms;
using System.Reflection;
using System.Globalization;
using Internationalization;

namespace Internationalization.Windows.Forms
{
	/// <summary>
	/// HotKeyAssignment assigns hot keys to Windows Forms controls
	/// </summary>
	public class HotKeyAssignment: HotKeyAssignmentBase
	{
        private static bool ignoreAcceptAndCancelButtons = false;

        /// <summary>
        /// IgnoreAcceptAndCancelButtons is True if Accept and Cancel buttons should be ignored (i.e. they should not have hot key assignments)
        /// </summary>
        public static bool IgnoreAcceptAndCancelButtons
        {
            get {return ignoreAcceptAndCancelButtons;}
            set {ignoreAcceptAndCancelButtons = value;}
        }

        /// <summary>
        /// OnControlCanHaveHotKey is a ControlCanHaveHotKeyEventHandler event used to determine if a control can have a hot key
        /// </summary>
        public static event ControlCanHaveHotKeyEventHandler OnControlCanHaveHotKey;

        /// <summary>
        /// CultureFormat returns the HotKeyFormat (respecting the CurrentUICulture if Format is HotKeyFormat.Culture)
        /// </summary>
        public static HotKeyFormat CultureFormat
        {
            get
            {
                if (Format == HotKeyFormat.Culture)
                {
                    // the actual format is dependant upon the current culture setting
                    string language = System.Threading.Thread.CurrentThread.CurrentUICulture.TwoLetterISOLanguageName;
                    if (language == "zn-CHS" || language == "zn-CHT" || language == "ko" || language == "ja")
                        // Chinese, Korean and Japanese should use a bracketed suffix embedded
                        return HotKeyFormat.BracketedSuffixEmbedded;
                    else if (language == "sv")
                        // Swedish should use a simple embedded
                        return HotKeyFormat.SimpleEmbedded;
                    else
                        // all other languages are debatably assumed to use a Latin alphabet
                        return HotKeyFormat.Embedded;
                }
                return Format;
            }
        }
        /// <summary>
        /// AssignHotKeys assigns a hot key to the control
        /// </summary>
        /// <param name="control">The control to assign a hot key to</param>
        public static void AssignHotKeys(Control control)
        {
            AssignHotKeys(control, true);
        }
        /// <summary>
        /// AssignHotKeys assign hot keys to all of the controls within a control
        /// </summary>
        /// <param name="control">The control containing controls to assign hot keys to</param>
        /// <param name="preserveExistingHotKeys">True if existing hot keys should be preserved</param>
        public static void AssignHotKeys(Control control, bool preserveExistingHotKeys)
        {
            Control[] controls = GetControlsInTabOrder(control);
            AssignHotKeysToControls(control, controls, preserveExistingHotKeys);
        }
        /// <summary>
        /// AssignHotKeysToControls assigns hot keys to controls
        /// </summary>
        /// <param name="control">The Control which owns the controls to which hot keys are assigned</param>
        /// <param name="controls">The array of Controls to which hot keys are assigned</param>
        /// <param name="preserveExistingHotKeys">True if existing hot keys should be preserved</param>
        protected static void AssignHotKeysToControls(Control control, Control[] controls, bool preserveExistingHotKeys)
        {
            ArrayList controlsNeedingHotKeys = new ArrayList();
            ArrayList assignedCharacters = new ArrayList();

            // get a list of controls which need hot keys assigned to them and 
            // get a list of the keys which have already been assigned to controls
            // where the hot keys must be preserved
            GetControlsNeedingHotKeys(controls, controlsNeedingHotKeys, assignedCharacters, preserveExistingHotKeys);

            ArrayList menuItemsNeedingHotKeys = new ArrayList();
            if (control is Form && ((Form) control).Menu != null)
            {
                // get a list of top level menu items which need hot keys assigned to them and
                // get a list of the keys which have already been assigned to top level menu items
                // where the hot keys must be preserved
                MainMenu mainMenu = ((Form) control).Menu;
                GetMenuItemsNeedingHotKeys(mainMenu.MenuItems, menuItemsNeedingHotKeys, assignedCharacters, preserveExistingHotKeys);
            }

            // assign any standard hot keys to controls
            AssignStandardHotKeys(controlsNeedingHotKeys);

            // assign any standard hot keys to menu items
            AssignStandardHotKeysToMenuItems(menuItemsNeedingHotKeys);

            // assign "calculated" hot keys to controls
            AssignHotKeysToControlsNeedingHotKeys(controlsNeedingHotKeys, assignedCharacters);

            // assign "calculated" hot keys to menu items
            AssignHotKeysToMenuItemsNeedingHotKeys(menuItemsNeedingHotKeys, assignedCharacters);

            AssignHotKeysToSubMenus(control, preserveExistingHotKeys);
        }
        /// <summary>
        /// GetControlsNeedingHotKeys fills an ArrayList with Controls which require hot keys
        /// </summary>
        /// <param name="controls">An array of all Controls</param>
        /// <param name="controlsNeedingHotKeys">An ArrayList of Controls which need hot keys</param>
        /// <param name="assignedCharacters">An ArrayList of characters which have already been assigned</param>
        /// <param name="preserveExistingHotKeys">True if existing hot keys should be preserved</param>
        protected static void GetControlsNeedingHotKeys(
            Control[] controls, ArrayList controlsNeedingHotKeys, ArrayList assignedCharacters, bool preserveExistingHotKeys)
        {
            foreach(Control childControl in controls)
            {
                if (ControlCanHaveHotKey(childControl))
                {
                    if (preserveExistingHotKeys)
                    {
                        char hotKey = GetHotKey(childControl);
                        if (hotKey == NullChar)
                            controlsNeedingHotKeys.Add(childControl);
                        else
                            assignedCharacters.Add(Char.ToUpper(hotKey, CultureInfo.InvariantCulture));
                    }
                    else
                        controlsNeedingHotKeys.Add(childControl);
                }
            }
        }
        /// <summary>
        /// AssignHotKeysToSubMenus assigns hot keys to menu items of a Control
        /// </summary>
        /// <param name="control">The Control containing all of the controls</param>
        /// <param name="preserveExistingHotKeys">True if existing hot keys should be preserved</param>
        /// <remarks>AssignHotKeysToSubMenus assigns hot keys to MainMenu menu items and ContextMenu menu items of all controls owned by the Control</remarks>
        protected static void AssignHotKeysToSubMenus(Control control, bool preserveExistingHotKeys)
        {
            // start with the MainMenu's sub menus
            if (control is Form && ((Form) control).Menu != null)
            {
                // the top level menu items have already been processed so
                // they do not need to be done again (and also
                // they belong to the form's context and are not independent)
                MainMenu mainMenu = ((Form) control).Menu;
                foreach(MenuItem menuItem in mainMenu.MenuItems)
                {
                    AssignHotKeysToMenuItemCollection(menuItem.MenuItems, preserveExistingHotKeys);
                }
            }

            // assign hot keys to all of the controls' ContextMenus
            foreach(Control childControl in control.Controls)
            {
                if (childControl.ContextMenu != null)
                    AssignHotKeysToMenuItemCollection(childControl.ContextMenu.MenuItems, preserveExistingHotKeys);
            }
        }
        /// <summary>
        /// AssignHotKeysToMenuItemCollection assigns hot keys to the menu items of a single MenuItemCollection
        /// </summary>
        /// <param name="menuItems">The MenuItemCollection of menu items to assign hot keys to</param>
        /// <param name="preserveExistingHotKeys">True if existing hot keys should be preserved</param>
        protected static void AssignHotKeysToMenuItemCollection(Menu.MenuItemCollection menuItems, bool preserveExistingHotKeys)
        {
            if (menuItems.Count > 0)
            {
                ArrayList assignedCharacters = new ArrayList();
                ArrayList menuItemsNeedingHotKeys = new ArrayList();

                // get a list of menu items which need hot keys assigned to them and
                // get a list of the keys which have already been assigned to menu items
                // where the hot keys must be preserved
                GetMenuItemsNeedingHotKeys(menuItems, menuItemsNeedingHotKeys, assignedCharacters, preserveExistingHotKeys);

                // assign any standard hot keys
                AssignStandardHotKeysToMenuItems(menuItemsNeedingHotKeys);

                // assign "calculated" hot keys to menu items
                AssignHotKeysToMenuItemsNeedingHotKeys(menuItemsNeedingHotKeys, assignedCharacters);

                // assign hot keys to each menu items' sub menus
                foreach(MenuItem menuItem in menuItems)
                {
                    AssignHotKeysToMenuItemCollection(menuItem.MenuItems, preserveExistingHotKeys);
                }
            }
        }
        /// <summary>
        /// GetMenuItemsNeedingHotKeys fills an ArrayList with menu items which need hot keys
        /// </summary>
        /// <param name="menuItems">A MenuItemCollection of menu items</param>
        /// <param name="menuItemsNeedingHotKeys">An ArrayList of menu items which need hot keys</param>
        /// <param name="assignedCharacters">An ArrayList of characters which have already been assigned</param>
        /// <param name="preserveExistingHotKeys">True if existing hot keys should be preserved</param>
        protected static void GetMenuItemsNeedingHotKeys(
            Menu.MenuItemCollection menuItems, ArrayList menuItemsNeedingHotKeys, ArrayList assignedCharacters, bool preserveExistingHotKeys)
        {
            foreach(MenuItem menuItem in menuItems)
            {
                // do not assign hot keys to menu items which are separators (i.e. MenuItem.Text == "-")
                if (menuItem.Text != "-")
                {
                    if (preserveExistingHotKeys)
                    {
                        char hotKey = GetHotKey(menuItem.Text);
                        if (hotKey == NullChar)
                            menuItemsNeedingHotKeys.Add(menuItem);
                        else
                            assignedCharacters.Add(Char.ToUpper(hotKey, CultureInfo.InvariantCulture));
                    }
                    else
                        menuItemsNeedingHotKeys.Add(menuItem);
                }
            }
        }
        /// <summary>
        /// AssignHotKeysToControlsNeedingHotKeys assigns hot keys to controls which need hot keys
        /// </summary>
        /// <param name="controlsNeedingHotKeys">An ArrayList of Controls which need hot keys</param>
        /// <param name="assignedCharacters">An ArrayList of characters which have already been assigned</param>
        protected static void AssignHotKeysToControlsNeedingHotKeys(ArrayList controlsNeedingHotKeys, ArrayList assignedCharacters)
        {
            foreach(Control childControl in controlsNeedingHotKeys)
            {
                char hotKey = SelectHotKey(childControl, assignedCharacters);
                if (hotKey != NullChar)
                {
                    assignedCharacters.Add(hotKey);
                    ApplyHotKey(childControl, hotKey);
                }
            }
        }
        /// <summary>
        /// AssignHotKeysToControlsNeedingHotKeys assigns hot keys to controls which need hot keys
        /// </summary>
        /// <param name="menuItemsNeedingHotKeys">An ArrayList of MenuItems which need hot keys</param>
        /// <param name="assignedCharacters">An ArrayList of characters which have already been assigned</param>
        protected static void AssignHotKeysToMenuItemsNeedingHotKeys(ArrayList menuItemsNeedingHotKeys, ArrayList assignedCharacters)
        {
            foreach(MenuItem menuItem in menuItemsNeedingHotKeys)
            {
                char hotKey = SelectHotKey(menuItem.Text, assignedCharacters);
                if (hotKey != NullChar)
                {
                    assignedCharacters.Add(hotKey);
                    ApplyHotKey(menuItem, hotKey);
                }
            }
        }
        /// <summary>
        /// AssignStandardHotKeys assigns standard hot keys to an ArrayList of Controls
        /// </summary>
        /// <param name="controlsNeedingHotKeys">An ArrayList of Controls</param>
        protected static void AssignStandardHotKeys(ArrayList controlsNeedingHotKeys)
        {
            if (StandardHotKeys.GetLength(0) > 0)
            {
                for(int controlNumber = 0; controlNumber < controlsNeedingHotKeys.Count; controlNumber++)
                {
                    Control control = (Control) controlsNeedingHotKeys[controlNumber];
                    char hotKey = SelectStandardHotKey(control);
                    if (hotKey != NullChar)
                    {
                        ApplyHotKey(control, hotKey);
                        // remove the control from the list of controls needing a hot key
                        // because this control now has a hot key
                        controlsNeedingHotKeys.RemoveAt(controlNumber);
                        controlNumber--;
                    }
                }
            }
        }
        /// <summary>
        /// AssignStandardHotKeysToMenuItems assigns standard hot keys to an ArrayList of MenuItems
        /// </summary>
        /// <param name="menuItemsNeedingHotKeys">An ArrayList of MenuItems</param>
        protected static void AssignStandardHotKeysToMenuItems(ArrayList menuItemsNeedingHotKeys)
        {
            if (StandardHotKeys.GetLength(0) > 0)
            {
                for(int menuItemNumber = 0; menuItemNumber < menuItemsNeedingHotKeys.Count; menuItemNumber++)
                {
                    MenuItem menuItem = (MenuItem) menuItemsNeedingHotKeys[menuItemNumber];
                    char hotKey = SelectStandardHotKey(menuItem.Text);
                    if (hotKey != NullChar)
                    {
                        ApplyHotKey(menuItem, hotKey);
                        // remove the menu item from the list of menu items needing a hot key
                        // because this menu item now has a hot key
                        menuItemsNeedingHotKeys.RemoveAt(menuItemNumber);
                        menuItemNumber--;
                    }
                }
            }
        }
        /// <summary>
        /// SelectStandardHotKey gets the hot key which should be used for the given Control
        /// </summary>
        /// <param name="control">The Control to get the hot key for</param>
        /// <returns>The hot key which should be used for the given Control</returns>
        protected static char SelectStandardHotKey(Control control)
        {
            if (control is Label)
            {
                if (!((Label) control).UseMnemonic)
                    return NullChar;
                else
                    return SelectStandardHotKey(((Label) control).Text);
            }
            else
            {
                // get the value of the Text property
                PropertyInfo propertyInfo = control.GetType().GetProperty("Text");
                if (propertyInfo == null)
                    // it doesn't have a public Text property
                    return NullChar;
                else
                {
                    string text = (string) propertyInfo.GetValue(control, new object[] {});
                    return SelectStandardHotKey(text);
                }
            }
        }
        /// <summary>
        /// ApplyHotKey applies the given hot key to the Control
        /// </summary>
        /// <param name="control">The Control to apply the hot key to</param>
        /// <param name="hotKey">The hot key to apply to the Control</param>
        protected static void ApplyHotKey(Control control, char hotKey)
        {
            // get the value of the Text property
            PropertyInfo propertyInfo = control.GetType().GetProperty("Text");
            string text = (string) propertyInfo.GetValue(control, new object[] {});
            text = ApplyHotKey(text, hotKey, CultureFormat);
            // apply the change to the control
            propertyInfo.SetValue(control, text, new object[] {});
        }
        /// <summary>
        /// ApplyHotKey applies the given hot key to the Control
        /// </summary>
        /// <param name="menuItem">The MenuItem to apply the hot key to</param>
        /// <param name="hotKey">The hot key to apply to the MenuItem</param>
        protected static void ApplyHotKey(MenuItem menuItem, char hotKey)
        {
            menuItem.Text = ApplyHotKey(menuItem.Text, hotKey, CultureFormat);
        }
        /// <summary>
        /// SelectHotKey returns the hot key which should be used for the given Control
        /// </summary>
        /// <param name="control">The Control to get the hot key for</param>
        /// <param name="assignedCharacters">An ArrayList of characters which have already been assigned</param>
        /// <returns>The hot key which should be used for the given Control</returns>
        protected static char SelectHotKey(Control control, ArrayList assignedCharacters)
        {
            // get the value of the Text property
            PropertyInfo propertyInfo = control.GetType().GetProperty("Text");
            string text = (string) propertyInfo.GetValue(control, new object[] {});
            return SelectHotKey(text, assignedCharacters);
        }
        /// <summary>
        /// GetHotKey gets the Control's existing hot key assignment
        /// </summary>
        /// <param name="control">The Control to get the hot key assignment from</param>
        /// <returns>The Control's existing hot key assignment or NullChar if the control does not have a hot key</returns>
        protected static char GetHotKey(Control control)
        {
            if (control is Label)
            {
                if (!((Label) control).UseMnemonic)
                    return NullChar;
                else
                    return GetHotKey(((Label) control).Text);
            }
            else
            {
                // get the value of the Text property
                PropertyInfo propertyInfo = control.GetType().GetProperty("Text");
                if (propertyInfo == null)
                    // it doesn't have a public Text property
                    return NullChar;
                else
                {
                    string text = (string) propertyInfo.GetValue(control, new object[] {});
                    return GetHotKey(text);
                }
            }
        }
        /// <summary>
        /// GetHotKey gets the text's existing hot key assignment
        /// </summary>
        /// <param name="text">The text to get the hot key assignment from</param>
        /// <returns>The text's existing hot key assignment or NullChar if the text does not have a hot key</returns>
        protected static char GetHotKey(string text)
        {
            int startIndex = 0;
            int foundIndex;
            while ((foundIndex = text.IndexOf("&", startIndex)) > -1)
            {
                if (foundIndex < text.Length && IsValidHotKey(text[foundIndex + 1]))
                    return text[foundIndex + 1];
                startIndex = foundIndex + 1;
            }
            return NullChar;
        }
        /// <summary>
        /// ControlCanHaveHotKey returns True if the given Control can have a hot key
        /// </summary>
        /// <param name="control">The Control to determine whether a hot key can be assigned</param>
        /// <returns>True if the given Control can have a hot key</returns>
        protected static bool ControlCanHaveHotKey(Control control)
        {
            bool controlCanHaveHotKey;
            if (control is Button)
            {
                if (ignoreAcceptAndCancelButtons)
                    // we should not assign a hotkey to this button if this button is an AcceptButton or a CancelButton
                    controlCanHaveHotKey = ! IsAcceptOrCancelButton((Button) control);
                else
                    controlCanHaveHotKey = true;
            }
            else            
                controlCanHaveHotKey =
                    (control is Form || control is Label || control is GroupBox || 
                    control is CheckBox || control is RadioButton);

            if (OnControlCanHaveHotKey != null)
            {
                ControlCanHaveHotKeyEventArgs args = new ControlCanHaveHotKeyEventArgs(control, controlCanHaveHotKey);
                OnControlCanHaveHotKey(null, args);
                controlCanHaveHotKey = args.CanHaveHotKey;
            }
            return controlCanHaveHotKey;
        }
        /// <summary>
        /// IsAcceptOrCancelButton returns True if the given Button is either the AcceptButton or the CancelButton
        /// </summary>
        /// <param name="button">The Button to test</param>
        /// <returns>True if the given Button is either the AcceptButton or the CancelButton</returns>
        protected static bool IsAcceptOrCancelButton(Button button)
        {
            Form form = null;
            Control parent = button.Parent;
            while (parent != null)
            {
                if (parent is Form)
                {
                    form = (Form) parent;
                    break;
                }
            }
            if (form == null)
                return false;
            else
                return button == form.AcceptButton || button == form.CancelButton;
        }
        /// <summary>
        /// GetControlsInTabOrder gets the controls owned by a Control in their TabOrder order
        /// </summary>
        /// <param name="control">The Control which owns zero of more controls</param>
        /// <returns>An array of controls owned by a Control in their TabOrder order</returns>
        protected static Control[] GetControlsInTabOrder(Control control)
        {
            // get the list of controls in the order of their TabIndex
            ArrayList controls = new ArrayList();
            int controlNumber = 0;
            int tabIndex = 0;
            while (controlNumber < control.Controls.Count)
            {
                Control[] foundControls = FindControlsByTabIndex(control.Controls, tabIndex);
                if (foundControls.GetLength(0) > 0)
                {
                    foreach(Control foundControl in foundControls)
                    {
                        controls.Add(foundControl);
                    }
                    controlNumber++;
                }
                tabIndex++;
            }
            return (Control[]) controls.ToArray(typeof(Control));
        }
        /// <summary>
        /// FindControlsByTabIndex gets the child controls of a Control identified by its TabIndex
        /// </summary>
        /// <param name="controls">A ControlCollection of Controls</param>
        /// <param name="tabIndex">the TabIndex which identifies the control</param>
        /// <returns>The child controls of a Control identified by its TabIndex</returns>
        protected static Control[] FindControlsByTabIndex(Control.ControlCollection controls, int tabIndex)
        {
            ArrayList foundControls = new ArrayList();
            foreach(Control control in controls)
            {
                if (control.TabIndex == tabIndex)
                {
                    foundControls.Add(control);
                    if (control.Controls.Count > 0 && control.GetType() != typeof(MenuItem))
                    {
                        // get the control's children
                        Control[] childControls = GetControlsInTabOrder(control);
                        foreach(Control childControl in childControls)
                        {
                            foundControls.Add(childControl);
                        }
                    }
                }
            }
            return (Control[]) foundControls.ToArray(typeof(Control));
        }    
    }
    /// <summary>
    /// ControlCanHaveHotKeyEventArgs is an EventArgs for the OnControlCanHaveHotKey event
    /// </summary>
    public class ControlCanHaveHotKeyEventArgs: EventArgs
    {
        private Control control;
        private bool canHaveHotKey;
        /// <summary>
        /// Contructs a ControlCanHaveHotKeyEventArgs object
        /// </summary>
        /// <param name="control">The Control</param>
        /// <param name="canHaveHotKey">True if the Control can have a hot key</param>
        public ControlCanHaveHotKeyEventArgs(Control control, bool canHaveHotKey)
        {
            this.control = control;
            this.canHaveHotKey = canHaveHotKey;
        }
        /// <summary>
        /// The Control which may or may not have a hot key
        /// </summary>
        public Control Control
        {
            get {return control;}
            set {control = value;}
        }
        /// <summary>
        /// CanHaveHotKey is True if the Control can have a hot key
        /// </summary>
        public bool CanHaveHotKey
        {
            get {return canHaveHotKey;}
            set {canHaveHotKey = value;}
        }
    }
    /// <summary>
    /// ControlCanHaveHotKeyEventHandler is an EventHandler for the OnControlCanHaveHotKey event
    /// </summary>
    public delegate void ControlCanHaveHotKeyEventHandler(object sender, ControlCanHaveHotKeyEventArgs args);
}
