﻿// File           : CultureAndRegionInfoBuilderHelper.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Text;
using System.Globalization;
using System.Threading;
using System.Xml;
using System.IO;

namespace Globalization
{
    /// <summary>
    /// CultureAndRegionInfoBuilderHelper is a helper class for the CreateCultureAndRegionInfoBuilder class
    /// </summary>
	public class CultureAndRegionInfoBuilderHelper
	{
        /// <summary>
        /// CreateCultureAndRegionInfoBuilder creates a CultureAndRegionInfoBuilder object by combining a language and a region
        /// </summary>
        /// <param name="languageCultureInfo">The CultureInfo which represents the language. The CultureInfo can be neutral (e.g. "es") or specific (e.g. "es-ES").</param>
        /// <param name="regionCultureInfo">The CultureInfo which represents the region (e.g. "en-US")</param>
        /// <returns>A CultureAndRegionInfoBuilder object which is the combinination of the language and the region</returns>
        /// <remarks>The name of the new culture is created by combining the TwoLetterISOLanguageName of the
        /// language with the TwoLetterISORegionName of the region (e.g. "es-US")</remarks>
        public static CultureAndRegionInfoBuilder CreateCultureAndRegionInfoBuilder(
			CultureInfo languageCultureInfo, CultureInfo regionCultureInfo)
		{
			return CreateCultureAndRegionInfoBuilder(
				languageCultureInfo, new RegionInfo(regionCultureInfo.Name), null);
		}
        /// <summary>
        /// CreateCultureAndRegionInfoBuilder creates a CultureAndRegionInfoBuilder object by combining a language and a region
        /// </summary>
        /// <param name="languageCultureInfo">The CultureInfo which represents the language. The CultureInfo can be neutral (e.g. "es") or specific (e.g. "es-ES").</param>
        /// <param name="regionCultureInfo">The CultureInfo which represents the region (e.g. "en-US")</param>
        /// <param name="cultureName">The name of the new culture. If cultureName is null then a name is created by
        /// combining the TwoLetterISOLanguageName of the language with the TwoLetterISORegionName of the region (e.g. "es-US")</param>
        /// <returns>A CultureAndRegionInfoBuilder object which is the combinination of the language and the region</returns>
		public static CultureAndRegionInfoBuilder CreateCultureAndRegionInfoBuilder(
			CultureInfo languageCultureInfo, CultureInfo regionCultureInfo, string cultureName)
		{
			return CreateCultureAndRegionInfoBuilder(
				languageCultureInfo, new RegionInfo(regionCultureInfo.Name), cultureName);
		}
        /// <summary>
        /// CreateCultureAndRegionInfoBuilder creates a CultureAndRegionInfoBuilder object by combining a language and a region
        /// </summary>
        /// <param name="languageCultureInfo">The CultureInfo which represents the language. The CultureInfo can be neutral (e.g. "es") or specific (e.g. "es-ES").</param>
        /// <param name="regionInfo">The RegionInfo which represents the region (e.g. "en-US")</param>
        /// <returns>A CultureAndRegionInfoBuilder object which is the combinination of the language and the region</returns>
        /// <remarks>The name of the new culture is created by combining the TwoLetterISOLanguageName of the
        /// language with the TwoLetterISORegionName of the region (e.g. "es-US")</remarks>
        public static CultureAndRegionInfoBuilder CreateCultureAndRegionInfoBuilder(
			CultureInfo languageCultureInfo, RegionInfo regionInfo)
		{
			return CreateCultureAndRegionInfoBuilder(
				languageCultureInfo, regionInfo, null);
		}
        /// <summary>
        /// CreateCultureAndRegionInfoBuilder creates a CultureAndRegionInfoBuilder object by combining a language and a region
        /// </summary>
        /// <param name="languageCultureInfo">The CultureInfo which represents the language. The CultureInfo can be neutral (e.g. "es") or specific (e.g. "es-ES").</param>
        /// <param name="regionInfo">The RegionInfo which represents the region (e.g. "en-US")</param>
        /// <param name="cultureName">The name of the new culture. If cultureName is null then a name is created by
        /// combining the TwoLetterISOLanguageName of the language with the TwoLetterISORegionName of the region (e.g. "es-US")</param>
        /// <returns>A CultureAndRegionInfoBuilder object which is the combinination of the language and the region</returns>
		public static CultureAndRegionInfoBuilder CreateCultureAndRegionInfoBuilder(
			CultureInfo languageCultureInfo, RegionInfo regionInfo, string cultureName)
		{
			// the comments in this section assume that the languageCultureInfo is "es" or "es-ES"
			// and the regionInfo is "en-US".
			if (cultureName == null || cultureName == String.Empty)
				// "es-US"
				cultureName = languageCultureInfo.TwoLetterISOLanguageName + "-" + 
					regionInfo.TwoLetterISORegionName;

			// languageNeutralCulture is "es"
			CultureInfo languageNeutralCultureInfo = GetNeutralCulture(languageCultureInfo);
			// regionCultureInfo is "en-US"
			CultureInfo regionCultureInfo = new CultureInfo(regionInfo.Name);

			CultureAndRegionInfoBuilder builder = 
				new CultureAndRegionInfoBuilder(cultureName, CultureAndRegionModifiers.None);

			builder.LoadDataFromCultureInfo(regionCultureInfo);
			builder.LoadDataFromRegionInfo(regionInfo);

			// "es"
			builder.Parent = languageNeutralCultureInfo;

			builder.CompareInfo = languageCultureInfo.CompareInfo;
			builder.TextInfo = languageCultureInfo.TextInfo;

			// "es-US"
			builder.IetfLanguageTag = cultureName;

			// "Estados Unidos"
			builder.RegionNativeName = GetNativeRegionName(regionInfo, languageCultureInfo);

			// "Spanish (United States)"
			builder.CultureEnglishName = languageNeutralCultureInfo.EnglishName + " (" + 
				regionInfo.EnglishName + ")";

			// "español (Estados Unidos)"
			builder.CultureNativeName = languageNeutralCultureInfo.NativeName + " (" +
				builder.RegionNativeName + ")";

			// "US Dollar"
			builder.CurrencyNativeName = GetNativeCurrencyName(regionInfo, languageCultureInfo);

			// copy the native month and day names from the languageCultureInfo to the CultureAndRegionInfoBuilder
			DateTimeFormatInfo builderDtfi = builder.GregorianDateTimeFormat;
			DateTimeFormatInfo languageDtfi = languageCultureInfo.DateTimeFormat;
			builderDtfi.AbbreviatedDayNames = languageDtfi.AbbreviatedDayNames;
			builderDtfi.AbbreviatedMonthGenitiveNames = languageDtfi.AbbreviatedMonthGenitiveNames;
			builderDtfi.AbbreviatedMonthNames = languageDtfi.AbbreviatedMonthNames;
			builderDtfi.DayNames = languageDtfi.DayNames;
			builderDtfi.MonthGenitiveNames = languageDtfi.MonthGenitiveNames;
			builderDtfi.MonthNames = languageDtfi.MonthNames;
			builderDtfi.ShortestDayNames = languageDtfi.ShortestDayNames;

			builder.KeyboardLayoutId = languageNeutralCultureInfo.KeyboardLayoutId;

			// "spa"
			builder.ThreeLetterISOLanguageName = languageNeutralCultureInfo.ThreeLetterISOLanguageName;
			// "ESN"
			builder.ThreeLetterWindowsLanguageName = languageNeutralCultureInfo.ThreeLetterWindowsLanguageName;
			// "es"
			builder.TwoLetterISOLanguageName = languageNeutralCultureInfo.TwoLetterISOLanguageName;

			return builder;
		}
        /// <summary>
        /// GetNativeCurrencyName gets the native currency name for a region in the given language
        /// </summary>
        /// <param name="regionInfo">The RegionInfo of the currency</param>
        /// <param name="languageCultureInfo">The language in which the currency name should be returned</param>
        /// <returns>The native currency name</returns>
        /// <remarks>The success of this method is dependant upon the presence of a .NET Framework Language
        /// Pack for the given language being installed</remarks>
		protected static string GetNativeCurrencyName(RegionInfo regionInfo, CultureInfo languageCultureInfo)
		{
			string nativeName;
			CultureInfo oldCultureInfo = Thread.CurrentThread.CurrentUICulture;
			try
			{
				Thread.CurrentThread.CurrentUICulture = languageCultureInfo;
				nativeName = regionInfo.CurrencyNativeName;
			}
			catch (Exception)
			{
				nativeName = regionInfo.CurrencyNativeName;
			}
			finally
			{
				Thread.CurrentThread.CurrentUICulture = oldCultureInfo;
			}
			return nativeName;
		}
        /// <summary>
        /// GetNativeRegionName gets the native region name for a region in the given language
        /// </summary>
        /// <param name="regionInfo">The RegionInfo of the region</param>
        /// <param name="languageCultureInfo">The language in which the region name should be returned</param>
        /// <returns>The native region name</returns>
        /// <remarks>The success of this method is dependant upon the presence of a .NET Framework Language
        /// Pack for the given language being installed</remarks>
		protected static string GetNativeRegionName(RegionInfo regionInfo, CultureInfo languageCultureInfo)
		{
			// attempt to get the native name
			// (this will only work if the appropriate .NET Framework Language Pack is installed)
			string nativeName;
			CultureInfo oldCultureInfo = Thread.CurrentThread.CurrentUICulture;
			try
			{
				Thread.CurrentThread.CurrentUICulture = languageCultureInfo;
				nativeName = regionInfo.DisplayName;
			}
			catch (Exception)
			{
				nativeName = regionInfo.NativeName;
			}
			finally
			{
				Thread.CurrentThread.CurrentUICulture = oldCultureInfo;
			}
			return nativeName;
		}
        /// <summary>
        /// GetNeutralCulture gets the neutral culture associated with the given culture
        /// </summary>
        /// <param name="cultureInfo">The CultureInfo to get the neutral culture for</param>
        /// <returns>A neutral CultureInfo (if the culture has a neutral culture)</returns>
		protected static CultureInfo GetNeutralCulture(CultureInfo cultureInfo)
		{
			if (cultureInfo.Equals(CultureInfo.InvariantCulture) || cultureInfo.IsNeutralCulture)
				// the culture is the invariant culture or is already a neutral culture
				return cultureInfo;

			CultureInfo neutralCultureInfo = cultureInfo.Parent;
			if (neutralCultureInfo.Equals(CultureInfo.InvariantCulture))
				// the parent of this non-neutral culture is the invariant culture
				// (could be a custom culture)
				return cultureInfo;
			else
				// return the neutral culture
				return neutralCultureInfo;
		}
        /// <summary>
        /// Export exports an existing CultureInfo to an LDML culture file
        /// </summary>
        /// <param name="cultureInfo">The CultureInfo to export</param>
        /// <param name="ldmlFilename">The LDML file to exdport the culture to</param>
        /// <param name="textInfoName">The name to change the TextInfo to (or null for no change)</param>
        /// <param name="sortName">The name to change the sort to (or null for no change)</param>
        /// <remarks>The purpose of exporting an culture which is known only to this operating system is
        /// to make it available to an operating system which does not know this culture. Although the target
        /// operating system can accept the new culture it will not know the necessary TextInfo and Sort names
        /// used by the culture. In these cases you must specify names for the TextInfo and Sort which
        /// the target operating system will know.</remarks>
        public static void Export(
            CultureInfo cultureInfo, string ldmlFilename, string textInfoName, string sortName)
        {
            Export(cultureInfo, ldmlFilename, textInfoName, sortName, null);
        }        
        /// <summary>
        /// Export exports an existing CultureInfo to an LDML culture file
        /// </summary>
        /// <param name="cultureInfo">The CultureInfo to export</param>
        /// <param name="ldmlFilename">The LDML file to exdport the culture to</param>
        /// <param name="textInfoName">The name to change the TextInfo to (or null for no change)</param>
        /// <param name="sortName">The name to change the sort to (or null for no change)</param>
        /// <param name="consoleFallbackName">The name to change the console fallback to (or null for no change)</param>
        /// <remarks>The purpose of exporting an culture which is known only to this operating system is
        /// to make it available to an operating system which does not know this culture. Although the target
        /// operating system can accept the new culture it will not know the necessary TextInfo and Sort names
        /// used by the culture. In these cases you must specify names for the TextInfo and Sort which
        /// the target operating system will know.</remarks>
        public static void Export(CultureInfo cultureInfo, string ldmlFilename,
            string textInfoName, string sortName, string consoleFallbackName)
        {
            RegionInfo regionInfo = new RegionInfo(cultureInfo.Name);

            CultureAndRegionInfoBuilder builder =
                new CultureAndRegionInfoBuilder(cultureInfo.Name,
                CultureAndRegionModifiers.Replacement);

            // load in the data from the existing culture and region
            builder.LoadDataFromCultureInfo(cultureInfo);
            builder.LoadDataFromRegionInfo(regionInfo);

            if (File.Exists(ldmlFilename))
                File.Delete(ldmlFilename);

            builder.Save(ldmlFilename);
            SetTextInfoAndSortAndConsoleFallbackNamesAndIetfLanguageTag(
                ldmlFilename, textInfoName, sortName, consoleFallbackName, cultureInfo.IetfLanguageTag);
        }
        private static void SetTextInfoAndSortAndConsoleFallbackNamesAndIetfLanguageTag(
            string ldmlFilename, string textInfoName, string sortName, string consoleFallbackName,
            string ietfLanguageTag)
        {
            const string msLocaleNamespace = "http://schemas.microsoft.com/globalization/2004/08/carib/ldml";

            if (textInfoName != null || sortName != null || consoleFallbackName != null
                || System.Environment.OSVersion.Version.Major >= 6 // Vista is version 6
                )
            {
                XmlDocument xmlDocument = new XmlDocument();
                using (FileStream fileStream = new FileStream(ldmlFilename, FileMode.Open))
                {
                    xmlDocument.Load(fileStream);

                    XmlNamespaceManager xmlNamespaceManager = new XmlNamespaceManager(xmlDocument.NameTable);
                    xmlNamespaceManager.AddNamespace("msLocale", msLocaleNamespace);

                    if (textInfoName != null)
                        // change the textinfo name
                        SetTypeAttribute(xmlDocument, xmlNamespaceManager,
                            "//identity/special/msLocale:textInfoName", textInfoName);

                    if (sortName != null)
                        // change the sort name
                        SetTypeAttribute(xmlDocument, xmlNamespaceManager,
                            "//identity/special/msLocale:sortName", sortName);

                    if (consoleFallbackName != null)
                        // change the console fallback name
                        SetTypeAttribute(xmlDocument, xmlNamespaceManager,
                            "//characters/special/msLocale:consoleFallbackName", consoleFallbackName);

                    if (System.Environment.OSVersion.Version.Major >= 6)
                    {
                        // On Vista the ietfLanguageTag is not written to the LDML file.
                        // Check to see if the LDML file is missing the IETF Language Tag.
                        XmlNode ietfLanguageTagXmlNode = xmlDocument.SelectSingleNode(
                            "//identity/special/msLocale:ietfLanguageTag", xmlNamespaceManager);
                        if (ietfLanguageTagXmlNode == null)
                        {
                            // the IEFT Language Tag is missing - add it back
                            XmlNode msLocaleXmlNode = xmlDocument.SelectSingleNode(
                                "//identity/special", xmlNamespaceManager);
                            if (msLocaleXmlNode is XmlElement)
                            {
                                XmlElement ietfLanguageTagXmlElement = xmlDocument.CreateElement("msLocale", "ietfLanguageTag", msLocaleNamespace);
                                ietfLanguageTagXmlElement.SetAttribute("type", ietfLanguageTag);
                                msLocaleXmlNode.AppendChild(ietfLanguageTagXmlElement);
                            }
                        }
                    }
                    
                }
                xmlDocument.Save(ldmlFilename);
            }
        }

        private static void SetTypeAttribute(
            XmlDocument xmlDocument, XmlNamespaceManager xmlNamespaceManager, string path, string value)
        {
            XmlNode xmlNode = xmlDocument.SelectSingleNode(path, xmlNamespaceManager);
            if (xmlNode is XmlElement)
                ((XmlElement)xmlNode).SetAttribute("type", value);
        }
        /// <summary>
        /// CreateChildCultureAndRegionInfoBuilder creates a CultureAndRegionInfoBuilder object which represents
        /// a child of the given culture
        /// </summary>
        /// <param name="parentCultureInfo">The parent CultureInfo (e.g. "en-US")</param>
        /// <param name="cultureName">The name of the new culture (e.g. "en-US-Sirius")</param>
        /// <param name="cultureEnglishName">The English name of the new culture (e.g. "English (United States) (Sirius Cybernetics Corporation)")</param>
        /// <param name="cultureNativeName">The native name of the new culture (e.g. "English (United States) (Sirius Cybernetics Corporation)")</param>
        /// <param name="regionEnglishName">The English name of the new region (e.g. "United States (Sirius Cybernetics Corporation)")</param>
        /// <param name="regionNativeName">The native name of the new region (e.g. "United States (Sirius Cybernetics Corporation)")</param>
        /// <returns>A CultureAndRegionInfoBuilder object which represents a child of the given culture</returns>
		public static CultureAndRegionInfoBuilder CreateChildCultureAndRegionInfoBuilder(
			CultureInfo parentCultureInfo, string cultureName,
			string cultureEnglishName, string cultureNativeName,
			string regionEnglishName, string regionNativeName)
		{
			RegionInfo parentRegionInfo = new RegionInfo(parentCultureInfo.Name);

			CultureAndRegionInfoBuilder builder =
				new CultureAndRegionInfoBuilder(cultureName,
				CultureAndRegionModifiers.None);

			builder.LoadDataFromCultureInfo(parentCultureInfo);
			builder.LoadDataFromRegionInfo(parentRegionInfo);
			builder.Parent = parentCultureInfo;
			builder.CultureEnglishName = cultureEnglishName;
			builder.CultureNativeName = cultureNativeName;
			builder.RegionEnglishName = regionEnglishName;
			builder.RegionNativeName = regionNativeName;

			return builder;
		}
	}
}
