// File           : ResourcesResourceManager.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Resources;
using System.Collections;
using System.Globalization;
using System.Data.SqlClient;
using System.Reflection;
using System.IO;

namespace Internationalization.Resources
{
    /// <summary>
    /// ResourcesResourceManager is a resource manager for reading and writing resources in .resources files
    /// </summary>
    /// <remarks>ResourcesResourceManager also acts as a base class for ResXResourceManager which reads and
    /// writes resources in .resx files</remarks>
    public class ResourcesResourceManager : ResourceManager
	{
		// TODO: Add File watching capability

		private string baseNameField;
		private string extension = "resources";
        /// <summary>
        /// The extension of the resources file (without the leading period). The only valid extensions are
        /// "resources" and "resx".
        /// </summary>
		protected string Extension
		{
			get {return extension;}
			set {extension = value;}
		}

        /// <summary>
        /// Initialize initializes the new object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">assembly is not used in this implementation</param>
        #pragma warning disable 618
        protected virtual void Initialize(string baseName, Assembly assembly)
		{
			this.baseNameField = baseName;
			ResourceSets = new Hashtable();
		}
        /// <summary>
        /// Constructs a ResourcesResourceManager object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">assembly is not used in this implementation</param>
        public ResourcesResourceManager(string baseName, Assembly assembly)
		{
			Initialize(baseName, assembly);
		}
        /// <summary>
        /// Constructs a ResourcesResourceManager object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        public ResourcesResourceManager(string baseName)
		{
			Initialize(baseName, null);
		}
        /// <summary>
        /// Constructs a ResourcesResourceManager object
        /// </summary>
        /// <param name="resourceType">The Type for which resources should be read/written</param>
        public ResourcesResourceManager(Type resourceType)
		{
			Initialize(resourceType.Name, resourceType.Assembly);
		}
        /// <summary>
        /// Provides the implementation for finding a ResourceSet
        /// </summary>
        /// <param name="cultureInfo">The CultureInfo to look for</param>
        /// <param name="createIfNotExists">If true and if the ResourceSet has not been loaded yet, load it</param>
        /// <param name="tryParents">If the ResourceSet cannot be loaded, try parent CultureInfo objects to see if they exist</param>
        /// <returns>The specified ResourceSet</returns>
        protected override ResourceSet InternalGetResourceSet(CultureInfo cultureInfo, bool createIfNotExists, bool tryParents)
		{
			if (ResourceSets.Contains(cultureInfo.Name))
				return ResourceSets[cultureInfo.Name] as ResourceSet;
			else
			{
				ResourceSet resourceSet = new ResourcesResourceSet(baseNameField, cultureInfo, extension);
				ResourceSets.Add(cultureInfo.Name, resourceSet);
				return resourceSet;
			}
		}
	}

    /// <summary>
    /// ResourcesResourceSet is a CustomResourceSet for reading and writing to/from .resources or .resx files
    /// </summary>
    public class ResourcesResourceSet : CustomResourceSet
	{
		private string baseNameField;
		private CultureInfo cultureInfo;
		private string extension;

        /// <summary>
        /// Constructs a ResourcesResourceSet object
        /// </summary>
        /// <param name="baseNameField">The base name of the resource set</param>
        /// <param name="cultureInfo">The CultureInfo identifying the culture of the resources in the resource set</param>
        /// <param name="extension">The extension of the resources file (without the leading period). The only
        /// valid extensions are "resources" and "resx".</param>
		public ResourcesResourceSet(string baseNameField, CultureInfo cultureInfo, string extension):
			base(new ResourcesResourceReader(baseNameField, cultureInfo, extension))
		{
			this.baseNameField = baseNameField;
			this.cultureInfo = cultureInfo;
			this.extension = extension;
		}
        /// <summary>
        /// GetDefaultReader gets the Type of the resource reader
        /// </summary>
        /// <returns>The Type of the resource reader</returns>
        /// <remarks>Use CreateDefaultReader instead because CreateDefaultReader passes the correct parameters
        /// to the resource reader constructor</remarks>
        public override Type GetDefaultReader()
		{
			return typeof(ResourcesResourceReader);
		}
        /// <summary>
        /// GetDefaultWriter gets the Type of the resource writer
        /// </summary>
        /// <returns>The Type of the resource writer</returns>
        /// <remarks>Use CreateDefaultWriter instead because CreateDefaultWriter passes the correct parameters
        /// to the resource writer constructor</remarks>
        public override Type GetDefaultWriter()
		{
			if (extension == "resx")
				return typeof(ResXResourceWriter);
			else if (extension == "resources")
				return typeof(ResourceWriter);
			else
				throw new ArgumentException(
                    String.Format(ResourcesResourceManagerResources.UnknownResourceExtension, extension));
		}
        /// <summary>
        /// CreateDefaultReader creates an IResourceReader for reading resources
        /// </summary>
        /// <returns>An IResourceReader for reading resources</returns>
        /// <remarks>Use CreateDefaultReader instead of GetDefaultReader because CreateDefaultReader passes the
        /// correct parameters to the resource reader constructor</remarks>
        public override IResourceReader CreateDefaultReader()
		{
			return new ResourcesResourceReader(baseNameField, cultureInfo, extension);
		}
        /// <summary>
        /// GetResourceFileName gets the name of the resource file for the base name and culture
        /// </summary>
        /// <returns>The name of the resource file for the base name and culture (e.g. "Form1.resx", 
        /// "Form1.fr.resx")</returns>
		protected virtual string GetResourceFileName()
		{
			if (cultureInfo.Equals(CultureInfo.InvariantCulture))
				return baseNameField + "." + extension;
			else
				return baseNameField + "." + cultureInfo.Name + "." + extension;
		}
        /// <summary>
        /// CreateDefaultWriter creates an IResourceReader for writing resources
        /// </summary>
        /// <returns>An IResourceWriter for writing resources</returns>
        /// <remarks>Use CreateDefaultWriter instead of GetDefaultWriter because CreateDefaultWriter passes the
        /// correct parameters to the resource writer constructor</remarks>
        public override IResourceWriter CreateDefaultWriter()
		{
			if (extension == "resx")
				return new ResXResourceWriter(GetResourceFileName());
			else if (extension == "resources")
				return new ResourceWriter(GetResourceFileName());
			else
				throw new ArgumentException(
                    String.Format(ResourcesResourceManagerResources.UnknownResourceExtension, extension));
		}
	}

    /// <summary>
    /// ResourcesResourceReader is an IResourceReader for reading resources from .resources or .resx files
    /// </summary>
    public class ResourcesResourceReader : IResourceReader
	{
		private string baseNameField;
		private CultureInfo cultureInfo;
		private string extension;

        /// <summary>
        /// Constructs a ResourcesResourceReader object
        /// </summary>
        /// <param name="baseNameField">The base name of the resource reader</param>
        /// <param name="cultureInfo">The CultureInfo identifying the culture of the resources to be read</param>
        /// <param name="extension">The extension of the resources file (without the leading period). The only
        /// valid extensions are "resources" and "resx".</param>
        public ResourcesResourceReader(string baseNameField, CultureInfo cultureInfo, string extension)
		{
			this.baseNameField = baseNameField;
			this.cultureInfo = cultureInfo;
			this.extension = extension;
		}
        /// <summary>
        /// GetResourceFileName gets the name of the resource file for the base name and culture
        /// </summary>
        /// <returns>The name of the resource file for the base name and culture (e.g. "Form1.resx", 
        /// "Form1.fr.resx")</returns>
        protected virtual string GetResourceFileName()
		{
			if (cultureInfo.Equals(CultureInfo.InvariantCulture))
				return baseNameField + "." + extension;
			else
				return baseNameField + "." + cultureInfo.Name + "." + extension;
		}
        /// <summary>
        /// GetResourceReader gets an IResourceReader which reads resources from the given file
        /// </summary>
        /// <param name="fileName">The file to read resources from</param>
        /// <returns>An IResourceReader which reads resources from the given file</returns>
		protected virtual IResourceReader GetResourceReader(string fileName)
		{
			if (extension == "resx")
				return new ResXResourceReader(fileName);
			else if (extension == "resources")
				return new ResourceReader(GetResourceFileName());
			else
				throw new ArgumentException(
                    String.Format(ResourcesResourceManagerResources.UnknownResourceExtension, extension));
		}
        /// <summary>
        /// Returns an IDictionaryEnumerator of the resources for this reader
        /// </summary>
        /// <returns>An IDictionaryEnumerator of the resources for this reader</returns>
        public System.Collections.IDictionaryEnumerator GetEnumerator()
		{
			Hashtable hashTable = new Hashtable();
			string fileName = GetResourceFileName();
			if (File.Exists(fileName))
			{
				IResourceReader reader = GetResourceReader(fileName);
				try
				{
					IDictionaryEnumerator enumerator = reader.GetEnumerator();
					while (enumerator.MoveNext())
					{
						hashTable.Add(enumerator.Key, enumerator.Value);
					}
				}
				finally
				{
					reader.Close();
				}
			}
			return hashTable.GetEnumerator();
		}
        /// <summary>
        /// Closes the resource reader after releasing any resources associated with it
        /// </summary>
        public void Close()
		{
		}
        /// <summary>
        /// Returns an IDictionaryEnumerator of the resources for this reader
        /// </summary>
        /// <returns>An IDictionaryEnumerator of the resources for this reader</returns>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			return this.GetEnumerator();
		}
        /// <summary>
        /// Releases all resources used by the object
        /// </summary>
        public void Dispose()
		{
		}
	}
}
