// File           : StandardPropertiesResourceManager.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;
using System.Reflection;
using System.Drawing;
using System.Windows.Forms;

namespace Internationalization.Resources
{
    /// <summary>
    /// StandardPropertiesResourceManager is a surrogate resource manager for applying standard property values
    /// to specific properties read from a resource manager
    /// </summary>
    /// <remarks>Use StandardPropertiesResourceManager to apply application wide settings to Font, ImeMode and
    /// RightToLeft properties. Before using StandardPropertiesResourceManager set the ResourceManagerType to
    /// the Type of ResourceManager which you use for reading/writing resources:-
    /// StandardPropertiesResourceManager.ResourceManagerType = typeof(DbResourceManager);</remarks>
    public class StandardPropertiesResourceManager : SurrogateResourceManager
	{
		private static Font font;
		private static ImeMode imeMode = (ImeMode) -1;
		private static RightToLeft rightToLeft = (RightToLeft) -1;

        /// <summary>
        /// Constructs a StandardPropertiesResourceManager object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">assembly is not used in this implementation</param>
        public StandardPropertiesResourceManager(string baseName, Assembly assembly)
            : base(baseName, assembly)
		{
		}
        /// <summary>
        /// Constructs a StandardPropertiesResourceManager object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        public StandardPropertiesResourceManager(string baseName)
            : base(baseName)
		{
		}
        /// <summary>
        /// Constructs a StandardPropertiesResourceManager object
        /// </summary>
        /// <param name="resourceType">The Type for which resources should be read/written</param>
        public StandardPropertiesResourceManager(Type resourceType)
            : base(resourceType)
		{
		}
        /// <summary>
        /// Font is the Font which all Font properties should be set to
        /// </summary>
        /// <remarks>The Font property is ignored when Font is null. Properties are matched if their name is
        /// "Font" and their Type is Font.</remarks>
		public static Font Font
		{
			get {return font;}
			set {font = value;}
		}
        /// <summary>
        /// ImeMode is the ImeMode which all ImeMode properties should be set to
        /// </summary>
        /// <remarks>The ImeMode property is ignored when ImeMode is -1. Properties are matched if their name is
        /// "ImeMode".</remarks>
        public static ImeMode ImeMode
		{
			get {return imeMode;}
			set {imeMode = value;}
		}
        /// <summary>
        /// RightToLeft is the RightToLeft which all RightToLeft properties should be set to
        /// </summary>
        /// <remarks>The RightToLeft property is ignored when RightToLeft is -1. Properties are matched if their
        /// name is "RightToLeft".</remarks>
        public static RightToLeft RightToLeft
		{
			get {return rightToLeft;}
			set {rightToLeft = value;}
		}
        /// <summary>
        /// SubstituteValue returns true if the given property value can be substituted for the actual value
        /// </summary>
        /// <param name="name">The name of the property</param>
        /// <param name="culture">The CultureInfo of the resource</param>
        /// <param name="value">The actual value of the property returned by the internal resource manager</param>
        /// <param name="substituteValue">The potential substitute value of the property</param>
        /// <returns>True if the substitute value should be used instead of the actual value</returns>
		protected virtual bool SubstituteValue(string name, CultureInfo culture, object value, object substituteValue)
		{
			return value != null && substituteValue != null && value.GetType().IsSubclassOf(substituteValue.GetType());
		}
        /// <summary>
        /// SubstituteValue returns true if the given property value can be substituted for the actual value
        /// </summary>
        /// <param name="name">The name of the property</param>
        /// <param name="culture">The CultureInfo of the resource</param>
        /// <param name="value">The actual value of the property returned by the internal resource manager</param>
        /// <param name="substituteValue">The potential substitute value of the property</param>
        /// <param name="propertyName">The name of the property</param>
        /// <returns>True if the substitute value should be used instead of the actual value</returns>
        protected virtual bool SubstituteValue(string name, CultureInfo culture, object value, object substituteValue, string propertyName)
		{
			if (value == null)
				return name.EndsWith("." + propertyName) &&	substituteValue != null;
			else
				return substituteValue != null && value.GetType().IsSubclassOf(substituteValue.GetType());
		}
        /// <summary>
        /// SubstituteValue returns true if the given property value can be substituted for the actual value
        /// </summary>
        /// <param name="name">The name of the property</param>
        /// <param name="culture">The CultureInfo of the resource</param>
        /// <param name="value">The actual value of the property returned by the internal resource manager</param>
        /// <param name="substituteValue">The potential substitute value of the property</param>
        /// <param name="propertyName">The name of the property</param>
        /// <returns>True if the substitute value should be used instead of the actual value</returns>
        protected virtual bool SubstituteValue(string name, CultureInfo culture, int value, int substituteValue, string propertyName)
		{
			return name.EndsWith("." + propertyName) &&	substituteValue != -1;
		}
        /// <summary>
        /// Gets the value of the Object resource localized for the specified culture
        /// </summary>
        /// <param name="name">The name of the resource to get</param>
        /// <param name="culture">The CultureInfo object that represents the culture for which the resource is localized</param>
        /// <returns>The value of the resource, localized for the specified culture</returns>
		public override object GetObject(string name, CultureInfo culture)
		{
			object obj = base.GetObject(name, culture);
			if (SubstituteValue(name, culture, obj, font, "Font"))
				return font;

			if (SubstituteValue(name, culture, (int) obj, (int) imeMode, "ImeMode"))
				return imeMode;

			if (SubstituteValue(name, culture, (int) obj, (int) rightToLeft, "RightToLeft"))
				return rightToLeft;

			return obj;
		}
	}
}
