// File           : SurrogateResourceManager.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Resources;
using System.Reflection;
using System.Globalization;
using System.Collections;

namespace Internationalization.Resources
{
    /// <summary>
    /// SurrogateResourceManager is a resource manager which modifies functionality provided by another resource
    /// manager
    /// </summary>
    /// <remarks>Before creating a SurrogateResourceManager object set the static ResourceManagerType to the
    /// Type of resource manager which the SurrogateResourceManager should use for retrieving resources. The
    /// surrogate then uses this internal resource manager as a slave for reading and writing resources while
    /// it overlays additional functionality on top of the slave.</remarks>
    public class SurrogateResourceManager : ResourceManager
	{
		private static Type resourceManagerType;
		private ResourceManager resourceManager;
		private string baseName;

        /// <summary>
        /// Initialize initializes the new object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">assembly is not used in this implementation</param>
        protected virtual void Initialize(string baseName, Assembly assembly)
		{
			if (resourceManagerType == null)
				throw new ArgumentException(SurrogateResourceManagerResources.ResourceManagerTypeIsNull);

			this.baseName = baseName;
			MainAssembly = assembly;
			resourceManager = CreateResourceManager(baseName, assembly);
		}
        /// <summary>
        /// Constructs a SurrogateResourceManager object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">assembly is not used in this implementation</param>
        public SurrogateResourceManager(string baseName, Assembly assembly)
		{
			Initialize(baseName, assembly);
		}
        /// <summary>
        /// Constructs a SurrogateResourceManager object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        public SurrogateResourceManager(string baseName)
		{
			Initialize(baseName, null);
		}
        /// <summary>
        /// Constructs a SurrogateResourceManager object
        /// </summary>
        /// <param name="resourceType">The Type for which resources should be read/written</param>
        public SurrogateResourceManager(Type resourceType)
		{
			Initialize(resourceType.FullName, resourceType.Assembly);
		}
        /// <summary>
        /// ResourceManagerType is the Type of the internal resource manager which reads and writes resources
        /// </summary>
        /// <example>SurrogateResourceManager.ResourceManagerType = typeof(DbResourceManager);</example>
		public static Type ResourceManagerType
		{
			get {return resourceManagerType;}
			set {resourceManagerType = value;}
		}
        /// <summary>
        /// ResourceManager is the internal resource manager which reads and writes resources
        /// </summary>
		protected ResourceManager ResourceManager
		{
			get {return resourceManager;}
		}
        /// <summary>
        /// CreateResourceManager creates a resource manager of the ResourceManagerType
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">assembly is not used in this implementation</param>
        /// <returns>A resource manager of the ResourceManagerType</returns>
		protected virtual ResourceManager CreateResourceManager(string baseName, Assembly assembly)
		{
			try
			{
				return (ResourceManager) 
					Activator.CreateInstance(resourceManagerType, new object[] {baseName, assembly});
			}
			catch (TargetInvocationException exception)
			{
				throw exception.InnerException;
			}
		}
        /// <summary>
        /// Gets the value of the Object resource localized for the specified culture
        /// </summary>
        /// <param name="name">The name of the resource to get</param>
        /// <param name="culture">The CultureInfo object that represents the culture for which the resource is
        /// localized</param>
        /// <returns>The Object resource localized for the specified culture</returns>
		public override object GetObject(string name, System.Globalization.CultureInfo culture)
		{
			return resourceManager.GetObject(name, culture);
		}
        /// <summary>
        /// Gets the ResourceSet for a particular culture
        /// </summary>
        /// <param name="culture">The CultureInfo to look for</param>
        /// <param name="createIfNotExists">If true and if the ResourceSet has not been loaded yet, load it</param>
        /// <param name="tryParents">If the ResourceSet cannot be loaded, try parent CultureInfo objects to see if they exist</param>
        /// <returns>The specified ResourceSet</returns>
		public override ResourceSet GetResourceSet(CultureInfo culture, bool createIfNotExists, bool tryParents)
		{
			return resourceManager.GetResourceSet(culture, createIfNotExists, tryParents);
		}
        /// <summary>
        /// Gets the value of the string resource localized for the specified culture
        /// </summary>
        /// <param name="name">The name of the resource to get</param>
        /// <param name="culture">The CultureInfo object that represents the culture for which the resource is
        /// localized</param>
        /// <returns>The string resource localized for the specified culture</returns>
        public override string GetString(string name, System.Globalization.CultureInfo culture)
		{
			return resourceManager.GetString(name, culture);
		}
	}
}
