// File           : TranslationResourceManager.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Resources;
using System.Reflection;
using System.Globalization;
using System.Collections;
using Internationalization.Translation;

namespace Internationalization.Resources
{
    /// <summary>
    /// TranslationResourceManager is a surrogate resource manager for automatically translating missing
    /// resource strings at runtime
    /// </summary>
    /// <remarks>TranslationResourceManager uses a collection of ITranslators to automatically translate
    /// strings when its internal resource manager does not have a value for the resource string.
    /// Before using TranslationResourceManager set the ResourceManagerType to the Type of ResourceManager
    /// which you use for reading/writing resources and add a collection of ITranslator objects:-
    /// TranslationResourceManager.ResourceManagerType = typeof(DbResourceManager);
    /// TranslationResourceManager.Translators = new TranslatorCollection();
    /// TranslationResourceManager.Translators.Add(new PseudoTranslator());
    /// TranslationResourceManager.Translators.Add(new WebServiceXTranslator());
    /// TranslationResourceManager.Translators.Add(new CloserFarTranslator());
    ///</remarks>
    public class TranslationResourceManager : SurrogateResourceManager
	{
		private static TranslatorCollection translators;
		private static int translationWriteThreshold = 1;
		private CultureInfo neutralResourcesCulture;
		private int translationWriteCount = 0;

        /// <summary>
        /// Constructs a TranslationResourceManager object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">assembly is not used in this implementation</param>
        public TranslationResourceManager(string baseName, Assembly assembly)
            : base(baseName, assembly)
		{
		}
        /// <summary>
        /// Constructs a TranslationResourceManager object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        public TranslationResourceManager(string baseName)
            : base(baseName)
		{
		}
        /// <summary>
        /// Constructs a TranslationResourceManager object
        /// </summary>
        /// <param name="resourceType">The Type for which resources should be read/written</param>
        public TranslationResourceManager(Type resourceType)
            : base(resourceType)
		{
		}
        /// <summary>
        /// Translators is the collection of ITranslator objects used for translation
        /// </summary>
        /// <example>
        /// TranslationResourceManager.Translators = new TranslatorCollection();
        /// TranslationResourceManager.Translators.Add(new PseudoTranslator());
        /// TranslationResourceManager.Translators.Add(new WebServiceXTranslator());
        /// TranslationResourceManager.Translators.Add(new CloserFarTranslator());
        /// </example>
		public static TranslatorCollection Translators
		{
			get {return translators;}
			set {translators = value;}
		}
        /// <summary>
        /// TranslationWriteThreshold is the number of translations which can be performed before the translations
        /// are written back to the resource source
        /// </summary>
        /// <remarks>The default is 1. Set TranslationWriteThreshold to 0 to turn off writing translated
        /// resources back to the source. Increase the TranslationWriteThreshold to improve performance of
        /// an individual client.</remarks>
		public static int TranslationWriteThreshold
		{
			get {return translationWriteThreshold;}
			set {translationWriteThreshold = value;}
		}
        /// <summary>
        /// NeutralResourcesCulture is the CultureInfo which represents the application's fallback culture
        /// </summary>
        /// <remarks>If the NeutralResourceLanguageAttribute is set then NeutralResourcesCulture is the
        /// NeutralResourceLanguageAttribute. Otherwise it is guessed as English.</remarks>
		protected virtual CultureInfo NeutralResourcesCulture
		{
			get
			{
				if (neutralResourcesCulture == null)
				{
					if (MainAssembly == null)
						// We have no main assembly so we cannot get the NeutralResourceLanguageAttribute.
						// We will have to make a guess.
						neutralResourcesCulture = new CultureInfo("en");
					else
					{
						neutralResourcesCulture = ResourceManager.GetNeutralResourcesLanguage(MainAssembly);
						if (neutralResourcesCulture == null || 
							neutralResourcesCulture.Equals(CultureInfo.InvariantCulture))
							// we didn't manage to get it from the main assembly or it was the invariant culture
							// so make a guess
							neutralResourcesCulture = new CultureInfo("en");
					}
				}
				return neutralResourcesCulture;
			}
		}
        /// <summary>
        /// Gets the value of the String resource localized for the specified culture
        /// </summary>
        /// <param name="name">The name of the resource to get</param>
        /// <param name="culture">The CultureInfo object that represents the culture for which the resource is localized</param>
        /// <returns>The value of the resource localized for the specified culture</returns>
        /// <remarks>GetString attempts to get the string for the given culture. If the string cannot be
        /// retrieved for the given culture it gets the string for the fallback culture attempts to
        /// translate it to the required culture. If the number of translations exceeds the
        /// TranslationWriteThreshold then the translations are written back to the source.</remarks>
		public override string GetString(string name, System.Globalization.CultureInfo culture)
		{
			if (culture == null)
				culture = CultureInfo.CurrentUICulture;

			if (culture.Equals(NeutralResourcesCulture) || culture.Equals(CultureInfo.InvariantCulture))
				// This is the fallback culture - there is no translation to do.
				return ResourceManager.GetString(name, culture);

			// get (or create) the resource set for this culture
			ResourceSet resourceSet = ResourceManager.GetResourceSet(culture, true, false);
			if (resourceSet != null)
			{
				// get the string from the resource set
				string resourceStringValue = resourceSet.GetString(name, IgnoreCase);
				if (resourceStringValue != null)
					// the resource string was found in the resource set
					return resourceStringValue;
			}

			// The string was not found in the resource set or the whole resource set was not found
			// and could not be created.

			// Get the corresponding string from the invariant culture.
			string invariantStringValue = ResourceManager.GetString(name, CultureInfo.InvariantCulture);
			if (invariantStringValue == null || invariantStringValue == String.Empty)
				// there is no equivalent in the invariant culture or the invariant culture string is empty or null
				return invariantStringValue;

			// the invariant string isn't empty so it should be possible to translate it
			CultureInfo fallbackCultureInfo = NeutralResourcesCulture;

			if (fallbackCultureInfo.TwoLetterISOLanguageName == culture.TwoLetterISOLanguageName)
				// The languages are the same. There is no translation to perform.
				return invariantStringValue;

			if (! IsOkToTranslate(fallbackCultureInfo, culture, name, invariantStringValue))
				return invariantStringValue;

			ITranslator translator = 
				translators.GetTranslator(fallbackCultureInfo.ToString(), culture.ToString());
			if (translator == null)
				throw new ApplicationException(String.Format(
					TranslationResourceManagerResources.NoTranslatorForThisLanguageCombination, 
					fallbackCultureInfo.ToString(), culture.ToString()));

			string translatedResourceStringValue = translator.Translate(
				fallbackCultureInfo.ToString(), culture.ToString(), invariantStringValue);
					
			if (translatedResourceStringValue != String.Empty && resourceSet != null)
			{
				// put the new string back into the resource set
				if (resourceSet is CustomResourceSet)
					((CustomResourceSet) resourceSet).Add(name, translatedResourceStringValue);
				else
					ForceResourceSetAdd(resourceSet, name, translatedResourceStringValue);

				WriteResources(resourceSet);
			}
			return translatedResourceStringValue;
		}
        /// <summary>
        /// IsOkToTranslate determines if a string resource should be translated
        /// </summary>
        /// <param name="fallbackCultureInfo">The fallback CultureInfo</param>
        /// <param name="cultureInfo">The CultureInfo for which the resource has been requested</param>
        /// <param name="name">The name of the resource to get</param>
        /// <param name="invariantStringValue">The string in the language of the fallback culture</param>
        /// <returns>True if the string should be translated</returns>
        /// <remarks>This method exists as a hook to allow developers to override it and provide
        /// logic to determine if a string should be translated</remarks>
		protected virtual bool IsOkToTranslate(
			CultureInfo fallbackCultureInfo, CultureInfo cultureInfo, string name, string invariantStringValue)
		{
			return true;
		}
        /// <summary>
        /// ForceResourceSetAdd adds a new resource entry to a ResourceSet
        /// </summary>
        /// <param name="resourceSet">The ResourceSet to add the new entry to</param>
        /// <param name="key">The key of the resource entry</param>
        /// <param name="resourceStringValue">The value of the resource entry</param>
		protected virtual void ForceResourceSetAdd(ResourceSet resourceSet, string key, string resourceStringValue)
		{
			FieldInfo fieldInfo = 
				resourceSet.GetType().GetField("Table", BindingFlags.Instance | BindingFlags.NonPublic);
			if (fieldInfo != null)
			{
				object tableObject = fieldInfo.GetValue(resourceSet);
				if (tableObject != null)
				{
					Hashtable table = (Hashtable) tableObject;
					table.Add(key, resourceStringValue);
				}
			}
		}
        /// <summary>
        /// WriteResources increases the write count and if the write count is greater than the
        /// TranslationWriteThreshold it writes resources back to the source
        /// </summary>
        /// <param name="resourceSet">The ResourceSet to write back to the source</param>
		protected virtual void WriteResources(ResourceSet resourceSet)
		{
			translationWriteCount++;
			if (translationWriteThreshold > 0 && translationWriteCount >= translationWriteThreshold &&
				resourceSet is CustomResourceSet)
			{
				// the current number of pending writes is greater than or equal to the write threshold so
				// it is time to write the values back to the source
				CustomResourceSet customResourceSet = ((CustomResourceSet) resourceSet);

				IResourceWriter resourceWriter = customResourceSet.CreateDefaultWriter();
				// copy all of the existing resources to the resource writer
				foreach(DictionaryEntry dictionaryEntry in customResourceSet.Table)
				{
					resourceWriter.AddResource(dictionaryEntry.Key.ToString(), dictionaryEntry.Value);
				}
				resourceWriter.Generate();
				resourceWriter.Close();
				translationWriteCount = 0;
			}
		}
	}
}
