﻿// File           : JavaScriptResourceCodeGenerator.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : February 2012
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System.Runtime.InteropServices;
using Microsoft.VisualStudio.Shell.Interop;
using System.IO;
using System.Collections;
using System;
using System.Resources;
using System.Drawing;
using System.Text;
using Microsoft.VisualStudio;
using System.Globalization;

namespace Internationalization.VisualStudio.Design.Serialization
{
    [Guid("8C3E848F-B8B3-4E9D-9AA6-C99FE01A7848")]
    public class JavaScriptResourceCodeGenerator : BaseResourceCodeGeneratorWithSiteWithoutCodeDom, IVsRefactorNotify
    {
        public override int DefaultExtension(out string ext)
        {
            ext = ".js";
            return 0;
        }

        protected override byte[] GenerateCode(string inputFileName, string inputFileContent)
        {
            StreamWriter streamWriter = new StreamWriter(new MemoryStream(), Encoding.UTF8);

            if (base.CodeGeneratorProgress != null)
            {
                base.CodeGeneratorProgress.Progress(0x4b, 100);
            }

            GenerateJavaScriptResources(inputFileName, streamWriter);

            if (base.CodeGeneratorProgress != null)
            {
                NativeMethods.ThrowOnFailure(base.CodeGeneratorProgress.Progress(100, 100));
            }

            streamWriter.Flush();
            return base.StreamToBytes(streamWriter.BaseStream);
        }

        private void GenerateJavaScriptResources(string resxFilename, StreamWriter streamWriter)
        {
            Hashtable resources = GetResources(resxFilename);

            string javaScriptResources = ResourcesToJson(resources);

            streamWriter.Write(GetJavaScriptObjectName(resxFilename) + "=");
            streamWriter.Write(javaScriptResources);
        }

        protected virtual string GetJavaScriptObjectName(string resxFilename)
        {
            //// "-" is not a legal character for an identifier in JavaScript (specific cultures (e.g. "fr-FR") include dashes)
            //return Path.GetFileNameWithoutExtension(resxFilename).Replace("-", "_");

            string objectName = Path.GetFileNameWithoutExtension(resxFilename);
            int index = objectName.LastIndexOf('.');
            if (index > -1)
            {
                string cultureName = objectName.Substring(index + 1);
                if (IsValidCultureName(cultureName))
                {
                    // remove the culture name from the object name
                    objectName = objectName.Substring(0, objectName.Length - cultureName.Length - 1);
                }
            }

            return objectName;
        }

        protected virtual Hashtable GetResources(string resxFilename)
        {
            ResXResourceReader resourceReader = new ResXResourceReader(resxFilename);
            Hashtable resources = new Hashtable();
            AddResources(resourceReader, resources);
            return resources;
        }

        protected static void AddResources(IResourceReader resourceReader, Hashtable resources)
        {
            IEnumerator enumerator = resourceReader.GetEnumerator();
            while (enumerator.MoveNext())
            {
                DictionaryEntry entry = (DictionaryEntry)enumerator.Current;
                if (entry.Value is string)
                {
                    string key = entry.Key.ToString();
                    if (!resources.ContainsKey(key))
                    {
                        resources.Add(key, entry.Value.ToString());
                    }
                }
            }
        }

        private static string ResourcesToJson(Hashtable resources)
        {
            StringBuilder builder = new StringBuilder("{");
            bool firstEntry = true;
            foreach(string resourceKey in resources.Keys)
            {
                if (firstEntry)
                    firstEntry = false;
                else
                    builder.Append(",");

                builder.Append(@"""" + resourceKey + @""": ");
                builder.Append(@"""" + resources[resourceKey].ToString() + @"""");
            }

            builder.Append("}");
            return builder.ToString();
        }

        protected CultureInfo GetCultureFromFilename(string filename)
        {
            string filenameWithoutExtension = Path.GetFileNameWithoutExtension(filename);
            string cultureName = Path.GetExtension(filenameWithoutExtension);

            if (String.IsNullOrEmpty(cultureName))
            {
                return null;
            }

            if (cultureName.StartsWith("."))
            {
                cultureName = cultureName.Substring(1);
            }

            if (IsValidCultureName(cultureName))
            {
                return new CultureInfo(cultureName);
            }

            return null;
        }

        protected bool IsValidCultureName(string cultureName)
        {
            try
            {
                CultureInfo cultureInfo = new CultureInfo(cultureName);
                return true;
            }
            catch
            {
            }

            return false;
        }
    }
}
