﻿// File           : DbResourceManagerResourceProviderFactory.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Compilation;
using System.IO;
using System.Resources;
using Internationalization.Resources;
using System.Threading;

namespace Internationalization.Resources.Web
{
    /// <summary>
    /// DbResourceManagerResourceProviderFactory is a ResourceProviderFactory for the DbResourceManager
    /// </summary>
    public class DbResourceManagerResourceProviderFactory : ResourceProviderFactory
    {
        /// <summary>
        /// Constructs a DbResourceManagerResourceProviderFactory object
        /// </summary>
        public DbResourceManagerResourceProviderFactory()
        {
        }
        /// <summary>
        /// Creates a new global resource provider
        /// </summary>
        /// <param name="classKey">The resource class to create the global resource provider for</param>
        /// <returns>A new global resource provider</returns>
        public override IResourceProvider CreateGlobalResourceProvider(string classKey)
        {
            return new DbResourceManagerResourceProvider(classKey);
        }
        /// <summary>
        /// Creates a local resource provider
        /// </summary>
        /// <param name="virtualPath">The virtual path to create the local resource provider for</param>
        /// <returns>A new local resource provider</returns>
        public override IResourceProvider CreateLocalResourceProvider(string virtualPath)
        {
            string classKey = RemoveRoot(virtualPath);
            return new DbResourceManagerResourceProvider(classKey);
        }
        /// <summary>
        /// RemoveRoot removes the root of a virtual path and returns the remainder
        /// </summary>
        /// <param name="virtualPath">The virtual path to remove the root from</param>
        /// <returns>A virtual path without the root</returns>
        protected virtual string RemoveRoot(string virtualPath)
        {
            if (virtualPath.StartsWith("/"))
                virtualPath = virtualPath.Substring(1);

            int slashIndex = virtualPath.IndexOf("/");
            if (slashIndex > -1)
                virtualPath = virtualPath.Substring(slashIndex + 1);

            return virtualPath;
        }
    }
    /// <summary>
    /// DbResourceManagerResourceProvider is a resource provider for DbResourceManager
    /// </summary>
    public class DbResourceManagerResourceProvider : IResourceProvider
    {
        private string classKey;
        private ResourceManager resourceManager;

        /// <summary>
        /// Constructs a new DbResourceManagerResourceProvider
        /// </summary>
        /// <param name="classKey">The class key to construct the resource provider for</param>
        public DbResourceManagerResourceProvider(string classKey)
        {
            this.classKey = classKey;
        }
        /// <summary>
        /// ResourceManager is the resource manager used by the resource provider
        /// </summary>
        protected ResourceManager ResourceManager
        {
            get
            {
                if (resourceManager == null)
                {
                    resourceManager = new DbResourceManager(classKey);
                    resourceManager.IgnoreCase = true;
                }
                return resourceManager;
            }
        }
        /// <summary>
        /// Gets the value of the specified Object resource for the specified culture
        /// </summary>
        /// <param name="resourceKey">The name of the resource to get</param>
        /// <param name="culture">The CultureInfo object that represents the culture for which the resource is
        /// localized</param>
        /// <returns>The value of the resource, localized for the specified culture</returns>
        public object GetObject(string resourceKey, System.Globalization.CultureInfo culture)
        {
            return ResourceManager.GetObject(resourceKey, culture);
        }
        /// <summary>
        /// ResourceReader is the IResourceReader for the virtual path
        /// </summary>
        public System.Resources.IResourceReader ResourceReader
        {
            get { return new DbResourceReader(classKey, Thread.CurrentThread.CurrentUICulture); }
        }
    }
}
