﻿// File           : ResourceManagerResourceProviderFactory.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Compilation;
using System.Resources;
using System.Reflection;
using System.IO;
using System.Web;
using Internationalization.Web.Compilation;
using System.Threading;
using System.Globalization;

namespace Internationalization.Resources.Web
{
    /// <summary>
    /// ResourceManagerResourceProviderFactory is a resource provider factory for a ResourceManager
    /// </summary>
    public class ResourceManagerResourceProviderFactory: ResourceProviderFactory
    {
        /// <summary>
        /// ResourceManagerResourceProviderFactory is a resource provider factory for a ResourceManager
        /// </summary>
        public ResourceManagerResourceProviderFactory()
        {
        }
        /// <summary>
        /// Creates a new global resource provider
        /// </summary>
        /// <param name="classKey">The resource class to create the global resource provider for</param>
        /// <returns>A new global resource provider</returns>
        public override IResourceProvider CreateGlobalResourceProvider(string classKey)
        {
            return new ResourceManagerResourceProvider("Resources." + classKey, 
                BuildManagerHelper.GetGlobalResourceAssembly());
        }
        /// <summary>
        /// Creates a local resource provider
        /// </summary>
        /// <param name="virtualPath">The virtual path to create the local resource provider for</param>
        /// <returns>A new local resource provider</returns>
        public override IResourceProvider CreateLocalResourceProvider(string virtualPath)
        {
            return new ResourceManagerResourceProvider(Path.GetFileName(virtualPath),
                BuildManagerHelper.GetLocalResourceAssembly(virtualPath.ToLower()));
        }
    }

    /// <summary>
    /// ResourceManagerResourceProvider is a ResourceProvider for a ResourceManager
    /// </summary>
    public class ResourceManagerResourceProvider : IResourceProvider
    {
        private string classKey;
        private Assembly assembly;
        private ResourceManager resourceManager;

        /// <summary>
        /// Constructs a BaseResourceManagerResourceProvider object for the given class key
        /// </summary>
        /// <param name="classKey">The name of the resources to construct the provider for</param>
        /// <param name="assembly">The assembly containing the resources</param>
        public ResourceManagerResourceProvider(string classKey, Assembly assembly)
        {
            this.classKey = classKey;
            this.assembly = assembly;
        }

        /// <summary>
        /// ResourceManager is the resource manager used by the resource provider
        /// </summary>
        protected ResourceManager ResourceManager
        {
            get
            {
                if (resourceManager == null)
                    resourceManager = CreateResourceManager();

                return resourceManager;
            }
        }
        /// <summary>
        /// CreateResourceManager constructs a new ResourceManager for the base name's assembly
        /// </summary>
        /// <returns></returns>
        protected virtual ResourceManager CreateResourceManager()
        {
            if (assembly == null)
                return null;

            ResourceManager resourceManager = new ResourceManager(classKey, assembly);
            resourceManager.IgnoreCase = true;
            return resourceManager;
        }

        /// <summary>
        /// ResourceReader is the IResourceReader provided by the resource provider
        /// </summary>
        public System.Resources.IResourceReader ResourceReader
        {
            get
            {
                Stream stream = GetManifestResourceStream(Thread.CurrentThread.CurrentUICulture);
                if (stream == null)
                    return null;

                return new ResourceReader(stream);
            }
        }
        private Stream GetManifestResourceStream(CultureInfo cultureInfo)
        {
            Assembly resourceAssembly = GetResourceAssembly(cultureInfo);

            string[] resourceNames = resourceAssembly.GetManifestResourceNames();
            Stream stream = resourceAssembly.GetManifestResourceStream(
                GetManifestResourceName(classKey, resourceAssembly));

            if (stream == null)
            {
                if (cultureInfo.Equals(CultureInfo.InvariantCulture))
                    // there are no more parents so give up
                    return null;

                return GetManifestResourceStream(cultureInfo.Parent);
            }

            return stream;
        }
        private string GetManifestResourceName(string classKey, Assembly resourceAssembly)
        {
            string cultureName = String.Empty;
            if (!resourceAssembly.GetName().CultureInfo.Equals(CultureInfo.InvariantCulture))
                cultureName = "." + resourceAssembly.GetName().CultureInfo.Name;

            return classKey.ToLowerInvariant() + cultureName + ".resources";
        }
        private Assembly GetResourceAssembly(CultureInfo cultureInfo)
        {
            if (cultureInfo.Equals(CultureInfo.InvariantCulture))
                return assembly;

            try
            {
                return assembly.GetSatelliteAssembly(cultureInfo);
            }
            catch (FileNotFoundException)
            {
                // there is no satellite assembly for this culture so try the parent culture
                return GetResourceAssembly(cultureInfo.Parent);
            }
        }

        /// <summary>
        /// Gets the value of the specified Object resource for the specified culture
        /// </summary>
        /// <param name="resourceKey">The name of the resource to get</param>
        /// <param name="culture">The CultureInfo object that represents the culture for which the resource is
        /// localized</param>
        /// <returns>The value of the resource, localized for the specified culture</returns>
        public object GetObject(string resourceKey, System.Globalization.CultureInfo culture)
        {
            return ResourceManager.GetObject(resourceKey, culture);
        }
    }
}
