// File           : CultureInfoProvider.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;

namespace Globalization
{
    /// <summary>
    /// CultureInfoProvider is a provider of CultureInfo objects
    /// </summary>
    /// <remarks>CultureInfoProvider serves two purposes:-
    /// (1) Cultures are updated with known updates before being returned to the caller
    /// (2) Cultures are CultureInfoEx objects which have additional data or functionality beyond CultureInfo objects
    /// The first issue can be solved more effectively using Custom Cultures in .NET Framework 2.0.
    /// </remarks>
    public class CultureInfoProvider
    {
        /// <summary>
        /// Retrieves an instance of a culture using the specified culture name
        /// </summary>
        /// <param name="name">The name of the culture</param>
        /// <returns>A CultureInfoEx object</returns>
        public static CultureInfoEx GetCultureInfo(string name)
        {
            CultureInfoEx cultureInfo = new CultureInfoEx(name);
            ApplyKnownUpdates(cultureInfo);
            return cultureInfo;
        }
        /// <summary>
        /// Retrieves an instance of a culture using the specified culture identifier
        /// </summary>
        /// <param name="LCID">A culture identifier</param>
        /// <returns>A CultureInfoEx object</returns>
        public static CultureInfoEx GetCultureInfo(int LCID)
        {
            CultureInfoEx cultureInfo = new CultureInfoEx(LCID);
            ApplyKnownUpdates(cultureInfo);
            return cultureInfo;
        }
        /// <summary>
        /// Retrieves an instance of a culture using the specified culture name
        /// </summary>
        /// <param name="name">The name of the culture</param>
        /// <param name="useUserOverride">A Boolean that denotes whether to use the user-selected culture settings (true) or the default culture settings (false)</param>
        /// <returns>A CultureInfoEx object</returns>
        public static CultureInfoEx GetCultureInfo(string name, bool useUserOverride)
        {
            CultureInfoEx cultureInfo = new CultureInfoEx(name, useUserOverride);
            ApplyKnownUpdates(cultureInfo);
            return cultureInfo;
        }
        /// <summary>
        /// Retrieves an instance of a culture using the specified culture identifier
        /// </summary>
        /// <param name="LCID">A culture identifier</param>
        /// <param name="useUserOverride">A Boolean that denotes whether to use the user-selected culture settings (true) or the default culture settings (false)</param>
        /// <returns>A CultureInfoEx object</returns>
        public static CultureInfoEx GetCultureInfo(int LCID, bool useUserOverride)
        {
            CultureInfoEx cultureInfo = new CultureInfoEx(LCID, useUserOverride);
            ApplyKnownUpdates(cultureInfo);
            return cultureInfo;
        }
        /// <summary>
        /// ApplyKnownUpdates applies all known updates to a given culture
        /// </summary>
        /// <param name="cultureInfo">The CultureInfo to apply updates to</param>
        /// <remarks>This method updates cultures with details which are known to be out-of-date.
        /// It should be noted that:-
        /// (1) This approach is only accurate given the knowledge available at the time of writing.
        /// It will become increasingly inaccurate with the passage of time.
        /// (2) A better solution to this problem is to use Custom Cultures.</remarks>
        public static void ApplyKnownUpdates(CultureInfo cultureInfo)
        {
            if (cultureInfo.Name == "fr-FR")
                cultureInfo.NumberFormat.CurrencySymbol = "";
            else if (cultureInfo.Name == "tr-TR")
                cultureInfo.NumberFormat.CurrencySymbol = "YTL";
        }
    }
}
