﻿// File           : GeoInfo.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Text;
using System.Runtime.InteropServices;

namespace Globalization
{
    /// <summary>
    /// GeoInfo provides access to geographical information given a GeoId
    /// </summary>
	public class GeoInfo
	{
		private int geoId;
        private int language = 1033;
        /// <summary>
        /// Constructs a GeoInfo object given a GeoId
        /// </summary>
        /// <param name="geoId">The GeoId which identifies the geographical area</param>
        /// <remarks>See the Table Of Geographical Locations
        /// (http://msdn.microsoft.com/library/default.asp?url=/library/en-us/intl/nls_locations.asp)
        /// for a complete list of GeoIds</remarks>
		public GeoInfo(int geoId)
		{
			this.geoId = geoId;
		}
        /// <summary>
        /// Constructs a GeoInfo object given a GeoId
        /// </summary>
        /// <param name="geoId">The GeoId which identifies the geographical area</param>
        /// <param name="language">The language identifier in which the information should be retrieved</param>
        /// <remarks>See the Table Of Geographical Locations
        /// (http://msdn.microsoft.com/library/default.asp?url=/library/en-us/intl/nls_locations.asp)
        /// for a complete list of GeoIds</remarks>
        public GeoInfo(int geoId, int language)
        {
            this.geoId = geoId;
            this.language = language;
        }
        /// <summary>
        /// GeoId identifies the geographical area
        /// </summary>
        public int GeoId
        {
            get { return geoId; }
        }
        /// <summary>
        /// Language identifies the language in which the information should be retrieved
        /// </summary>
        public int Language
        {
            get { return language; }
        }
        /// <summary>
        /// The latitude of the geographical area
        /// </summary>
		public double Latitude
		{
			get
			{
				string geoInfo = GetGeoInfoString(SYSGEOTYPE.GEO_LATITUDE);
				if (geoInfo != null && geoInfo != String.Empty)
				{
					try
					{
						return Double.Parse(geoInfo);
					}
					catch (FormatException)
					{
						throw new GeoInfoException("Failed to retrieve Geo Information", SYSGEOTYPE.GEO_LATITUDE, geoInfo);
					}
				}
				else
					throw new GeoInfoException("Failed to retrieve Geo Information", SYSGEOTYPE.GEO_LATITUDE, geoInfo);
			}
		}
        /// <summary>
        /// The longitude of the geographical area
        /// </summary>
		public double Longitude
		{
			get
			{
				string geoInfo = GetGeoInfoString(SYSGEOTYPE.GEO_LONGITUDE);
				if (geoInfo != null && geoInfo != String.Empty)
				{
					try
					{
						return Double.Parse(geoInfo);
					}
					catch (FormatException)
					{
						throw new GeoInfoException("Failed to retrieve Geo Information", SYSGEOTYPE.GEO_LONGITUDE, geoInfo);
					}
				}
				else
					throw new GeoInfoException("Failed to retrieve Geo Information", SYSGEOTYPE.GEO_LONGITUDE, geoInfo);
			}
		}
        /// <summary>
        /// The ISO 2 code of the geographical area
        /// </summary>
		public string Iso2
		{
			get
			{
				string geoInfo = GetGeoInfoString(SYSGEOTYPE.GEO_ISO2);
				if (geoInfo != null)
					return geoInfo;
				else
					throw new GeoInfoException("Failed to retrieve Geo Information", SYSGEOTYPE.GEO_ISO2, geoInfo);
			}
		}
        /// <summary>
        /// The ISO 3 code of the geographical area
        /// </summary>
		public string Iso3
		{
			get
			{
				string geoInfo = GetGeoInfoString(SYSGEOTYPE.GEO_ISO3);
				if (geoInfo != null)
					return geoInfo;
				else
					throw new GeoInfoException("Failed to retrieve Geo Information", SYSGEOTYPE.GEO_ISO3, geoInfo);
			}
		}
        /// <summary>
        /// The RFC 1766 code of the geographical area
        /// </summary>
		public string Rfc1766
		{
			get
			{
				string geoInfo = GetGeoInfoString(SYSGEOTYPE.GEO_RFC1766);
				if (geoInfo != null)
					return geoInfo;
				else
					throw new GeoInfoException("Failed to retrieve Geo Information", SYSGEOTYPE.GEO_RFC1766, geoInfo);
			}
		}
        /// <summary>
        /// The locale identifier of the geographical area
        /// </summary>
		public int LCID
		{
			get
			{
				string geoInfo = GetGeoInfoString(SYSGEOTYPE.GEO_LCID);
				if (geoInfo != null && geoInfo != String.Empty)
				{
					try
					{
						return Int32.Parse(geoInfo);
					}
					catch (FormatException)
					{
						throw new GeoInfoException("Failed to retrieve Geo Information", SYSGEOTYPE.GEO_LCID, geoInfo);
					}
				}
				else
					throw new GeoInfoException("Failed to retrieve Geo Information", SYSGEOTYPE.GEO_LCID, geoInfo);
			}
		}
        /// <summary>
        /// The friednly name of the geographical area
        /// </summary>
		public string FriendlyName
		{
			get
			{
				string geoInfo = GetGeoInfoString(SYSGEOTYPE.GEO_FRIENDLYNAME);
				if (geoInfo != null)
					return geoInfo;
				else
					throw new GeoInfoException("Failed to retrieve Geo Information", SYSGEOTYPE.GEO_FRIENDLYNAME, geoInfo);
			}
		}
        /// <summary>
        /// The offical name of the geographical area
        /// </summary>
		public string OfficialName
		{
			get
			{
				string geoInfo = GetGeoInfoString(SYSGEOTYPE.GEO_OFFICIALNAME);
				if (geoInfo != null)
					return geoInfo;
				else
					throw new GeoInfoException("Failed to retrieve Geo Information", SYSGEOTYPE.GEO_OFFICIALNAME, geoInfo);
			}
		}
        /// <summary>
        /// A string of the time zones of the geographical area
        /// </summary>
        /// <remarks>TimeZones will be empty until a more recent version of Windows includes this information</remarks>
		public string TimeZones
		{
			get {return GetGeoInfoString(SYSGEOTYPE.GEO_TIMEZONES);}
		}
        /// <summary>
        /// A string of the official languages of the geographical area
        /// </summary>
        /// <remarks>OfficialLanguages will be empty until a more recent version of Windows includes this information</remarks>
        public string OfficialLanguages
		{
			get { return GetGeoInfoString(SYSGEOTYPE.GEO_OFFICIALLANGUAGES); }
		}
        /// <summary>
        /// GetGeoInfoString is a wrapper around the Win32 GetGeoInfo API
        /// </summary>
        /// <param name="sysGeoType">The SYSGEOTYPE specifying the geographical information type to retrieve</param>
        /// <returns>The data corresponding to the SYSGEOTYPE</returns>
		protected virtual string GetGeoInfoString(SYSGEOTYPE sysGeoType)
		{
			// find out the length of the geo information
			int length = GetGeoInfo(geoId, sysGeoType, null, 0, language);
			if (length == 0)
				return null;
			else
			{
				StringBuilder lpGeoData = new StringBuilder(length);
				// get the geo information
				int result = GetGeoInfo(geoId, sysGeoType, lpGeoData, length, language);
				if (result == 0)
					return null;
				else
					return lpGeoData.ToString();
			}
		}
        /// <summary>
        /// The Win32 GetGeoInfo API
        /// </summary>
        /// <param name="GeoId">The GeoId which identifies the geographical area</param>
        /// <param name="GeoType">The SYSGEOTYPE specifying the geographical information type to retrieve</param>
        /// <param name="lpGeoData">A buffer to hold the geographical information</param>
        /// <param name="cchData">The length of the buffer</param>
        /// <param name="language">The language identifier in which the information should be retrieved</param>
        /// <returns></returns>
		[DllImport("kernel32", SetLastError = true)]
		protected static extern int GetGeoInfo(
			int GeoId,
			SYSGEOTYPE GeoType,
			StringBuilder lpGeoData,
			int cchData,
			int language
		);
	}
    /// <summary>
    /// SYSGEOTYPE represents the type of GEO data
    /// </summary>
	public enum SYSGEOTYPE
	{
        /// <summary>
        /// GEO_NATION
        /// </summary>
		GEO_NATION = 0x0001,
        /// <summary>
        /// GEO_LATITUDE
        /// </summary>
		GEO_LATITUDE = 0x0002,
        /// <summary>
        /// GEO_LONGITUDE
        /// </summary>
		GEO_LONGITUDE = 0x0003,
        /// <summary>
        /// GEO_ISO2
        /// </summary>
		GEO_ISO2 = 0x0004,
        /// <summary>
        /// GEO_ISO3
        /// </summary>
		GEO_ISO3 = 0x0005,
        /// <summary>
        /// GEO_RFC1766
        /// </summary>
		GEO_RFC1766 = 0x0006,
        /// <summary>
        /// GEO_LCID
        /// </summary>
		GEO_LCID = 0x0007,
        /// <summary>
        /// GEO_FRIENDLYNAME
        /// </summary>
		GEO_FRIENDLYNAME = 0x0008,
        /// <summary>
        /// GEO_OFFICIALNAME
        /// </summary>
		GEO_OFFICIALNAME = 0x0009,
        /// <summary>
        /// GEO_TIMEZONES
        /// </summary>
		GEO_TIMEZONES = 0x000A,
        /// <summary>
        /// GEO_OFFICIALLANGUAGES
        /// </summary>
		GEO_OFFICIALLANGUAGES = 0x000B,
	};
    /// <summary>
    /// GeoInfoException is a geographical information exception
    /// </summary>
	public class GeoInfoException : ApplicationException
	{
		private SYSGEOTYPE sysGeoType;
		private string geoInfo;
        /// <summary>
        /// Constructs are new GeoInfoException object
        /// </summary>
        /// <param name="message">The exception message</param>
        /// <param name="sysGeoType">The SYSGEOTYPE of the geographical information</param>
        /// <param name="geoInfo">The information corresponding to the SYSGEOTYPE</param>
		public GeoInfoException(string message, SYSGEOTYPE sysGeoType, string geoInfo): base(message)
		{
			this.sysGeoType = sysGeoType;
			this.geoInfo = geoInfo;
		}
        /// <summary>
        /// The information corresponding to the SYSGEOTYPE
        /// </summary>
		public string GeoInfo
		{
			get { return geoInfo; }
			set { geoInfo = value; }
		}
        /// <summary>
        /// The SYSGEOTYPE of the geographical information
        /// </summary>
		public SYSGEOTYPE SysGeoType
		{
			get { return sysGeoType; }
			set { sysGeoType = value; }
		}
	}
}
