// File           : PostalCodeInfo.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;

namespace Globalization
{
    /// <summary>
    /// PostalCodeInfo holds a postal code mask which is suitable for use with data entry controls
    /// </summary>
    /// <remarks>The mask uses the format supported by the .NET Framework 2.0 MaskedTextBox control but its
    /// use is not limited to this control. The class includes a store of known masks but it expected that
    /// its list is populated from an external store using PostalCodeInfo.SetMask.</remarks>
    public class PostalCodeInfo
    {
        /// <summary>
        /// masks is the Hashtable of CultureInfo/postal code combinations
        /// </summary>
        protected static Hashtable masks;

        static PostalCodeInfo()
        {
            masks = new Hashtable();
            masks.Add("en-US", "00000-9999");
            masks.Add("en-GB", "L?90? 9??");
            masks.Add("en-AU", "LLL 0000");
        }
        /// <summary>
        /// GetMask gets the postal code mask for the given culture name
        /// </summary>
        /// <param name="cultureName">The name of the culture</param>
        /// <returns>The postal code mask corresponding to the given culture (if any) or null</returns>
        public static string GetMask(string cultureName)
        {
            if (masks.ContainsKey(cultureName))
                return (string) masks[cultureName];
            else
                return null;
        }
        /// <summary>
        /// SetMask sets a postal code mask for a given culture
        /// </summary>
        /// <param name="cultureName">The name of the culture</param>
        /// <param name="mask">The mask to set for the culture</param>
        /// <remarks>If the culture is already known to PostalCodeInfo then the mask information is replaced.
        /// If the culture is now known to PostalCodeInfo then the mask information is added.</remarks>
        public static void SetMask(string cultureName, string mask)
        {
            if (masks.ContainsKey(cultureName))
                masks[cultureName] = mask;
            else
                masks.Add(cultureName, mask);
        }

        /// <summary>
        /// Consstructs a new PostalCodeInfo object
        /// </summary>
        public PostalCodeInfo()
        {
        }
        /// <summary>
        /// Constructs a new PostalCodeInfo object and initializes its Mask for the given culture
        /// </summary>
        /// <param name="cultureName">The name of the culture for which the mask should be set</param>
        public PostalCodeInfo(string cultureName)
        {
            this.mask = GetMask(cultureName);
        }
        private string mask;
        /// <summary>
        /// Mask is the postal code mask
        /// </summary>
        public string Mask
        {
            get {return mask;}
            set {mask = value;}
        }
    }
}
