// File           : FileFinder.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.IO;

namespace Internationalization.Common
{
	/// <summary>
	/// FileFinder finds files which are "close" to the expected location
	/// </summary>
	public class FileFinder
	{
		/// <summary>
		/// Find finds files by walking up and down the directory tree
		/// </summary>
		/// <param name="filename">The filename to find</param>
		/// <returns>The full path to the found file or String.Empty if the file is not found</returns>
		public static string Find(string filename)
		{
			if (File.Exists(filename))
				return filename;

			// try going up a level
			string directoryName = Path.GetDirectoryName(Path.GetFullPath(filename));
			if (directoryName == String.Empty)
				return String.Empty;

			DirectoryInfo directoryInfo = Directory.GetParent(directoryName);
			if (directoryInfo == null)
				return String.Empty;

			string parentDirectoryName = directoryInfo.FullName;
			if (parentDirectoryName == Path.GetPathRoot(filename))
				// we are at the root - don't go searching through every subdirectory of the root
				return String.Empty;

            if (parentDirectoryName == Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles))
                // this is the ProgramFiles folder and we may not have access to it so stop
                return String.Empty;

			string parentFilename = parentDirectoryName + Path.DirectorySeparatorChar + Path.GetFileName(filename);
			string testFilename = FindFileLower(parentFilename, directoryName);
			if (testFilename == String.Empty)
				// we haven't found the file yet 
				return Find(parentFilename);
			else
				// we have found the file
				return testFilename;
		}
		private static string FindFileLower(string filename, string excludeDirectoryName)
		{
			if (File.Exists(filename))
				return filename;

			string directoryName = Path.GetDirectoryName(filename);
			foreach(string directory in Directory.GetDirectories(directoryName))
			{
				if (directory != excludeDirectoryName)
				{
					// this directory is not the directory from which we have come so we should check it
					string testFilename = FindFileLower(
						directory + Path.DirectorySeparatorChar + Path.GetFileName(filename), directoryName);

					if (testFilename != String.Empty)
						// we have found the file
						return testFilename;
				}
			}
			return String.Empty;
		}
	}
}
