// File           : TargetLanguages.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;
using System.Globalization;

namespace Internationalization.Common
{
    /// <summary>
    /// TargetLanguages provides a central store of languages which a utility targets
    /// </summary>
    /// <remarks>The Resource Administrator, for example, uses TargetLanguages to store the list
    /// of languages which should be maintained.</remarks>
	public class TargetLanguages
	{
		private static CultureInfo[] languages;

        /// <summary>
        /// Languages is an array of culture-neutral CultureInfo objects
        /// </summary>
		public static CultureInfo[] Languages
		{
			get {return languages;}
		}
        /// <summary>
        /// SetLanguages sets the list of languages
        /// </summary>
        /// <param name="languageStrings">An array of (culture-neutral) ISO 639 language names</param>
		public static void SetLanguages(string[] languageStrings)
		{
			languages = LanguageStringsToCultureInfos(languageStrings);
		}
        /// <summary>
        /// LanguageStringsToCultureInfos returns an array of CultureInfo objects from an array of language names
        /// </summary>
        /// <param name="languages">An array of (culture-neutral) ISO 639 language names</param>
        /// <returns>An array of culture-neutral CultureInfo objects</returns>
		protected static CultureInfo[] LanguageStringsToCultureInfos(string[] languages)
		{
			CultureInfo[] cultureInfos = new CultureInfo[languages.GetLength(0)];
			int cultureInfoNumber = 0;
			foreach(string language in languages)
			{
				CultureInfo cultureInfo = new CultureInfo(language);
				if (! cultureInfo.IsNeutralCulture)
					throw new ArgumentException(TargetLanguagesResources.LanguageIsNotCultureNeutral, language);

				cultureInfos[cultureInfoNumber++] = cultureInfo;
			}
			// check for duplicates (don't bother checking the very last element because there are no further 
			// elements to check it against)
			for(cultureInfoNumber = 0; cultureInfoNumber < cultureInfos.GetLength(0) - 1; cultureInfoNumber++)
			{
				CultureInfo cultureInfo = cultureInfos[cultureInfoNumber];

				if (Array.IndexOf(cultureInfos, cultureInfo, cultureInfoNumber + 1) > -1)
					throw new ArgumentException(TargetLanguagesResources.LanguageIsNotUnique, cultureInfo.Name);
			}
			return cultureInfos;
		}
	}
}