﻿// File           : Google2Translator.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : March 2012
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Text;

namespace Internationalization.Translation
{
    /// <summary>
    /// Google2Translator is a translator for the Google v2 Translate API
    /// </summary>
    /// <remarks>
    /// Google v2 Translator is a paid for replacement for the deprecated free Google v1 Translator.
    /// To use the Google v2 Translator you need to sign up for an app key from Google at http://code.google.com/apis/language/translate/v2/getting_started.html.
    /// </remarks>
    public class Google2Translator : Translator
    {
        /// <summary>
        /// Gets or sets your Google v2 Translator Application Key (see http://code.google.com/apis/language/translate/v2/getting_started.html)
        /// </summary>
        public static string ApplicationKey { get; set; }

        /// <summary>
        /// Constructs a Google2Translator object
        /// </summary>
        public Google2Translator()
            : base(Google2TranslatorResources.Name, GetLanguagePairs())
        {
        }

        /// <summary>
        /// GetLanguagePairs gets the array of translatable language pairs
        /// </summary>
        /// <returns>The array of translatable language pairs</returns>
        protected static string[,] GetLanguagePairs()
        {
            // the language list contains all of the languages that can be translated to/from English
            string[] languageList = GetLanguageList();
            int length = languageList.GetLength(0);
            string[,] languagePairs = (string[,])Array.CreateInstance(typeof(string), length * 2, 2);

            int languagePairNumber = 0;
            foreach (string language in languageList)
            {
                // add the language pair for translating from English
                languagePairs[languagePairNumber, 0] = "en";
                languagePairs[languagePairNumber, 1] = language;
                languagePairNumber++;

                // add the language pair for translating to English
                languagePairs[languagePairNumber, 0] = language;
                languagePairs[languagePairNumber, 1] = "en";
                languagePairNumber++;
            }

            return languagePairs;
        }

        /// <summary>
        /// Translate translates the input text from the input language to the output language
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to translate</param>
        /// <returns>The translated text</returns>
        public override string Translate(string inputLanguage, string outputLanguage, string text)
        {
            if (!IsSupported(inputLanguage, outputLanguage))
                throw new LanguageCombinationNotSupportedException(
                    LanguageCombinationNotSupportedResources.LanguageCombinationNotSupported,
                    inputLanguage, outputLanguage);

            string translateUri = String.Format(
                "https://www.googleapis.com/language/translate/v2?key={0}&q={1}&source={2}&target={3}",
                Uri.EscapeDataString(ApplicationKey),
                Uri.EscapeDataString(text),
                Uri.EscapeDataString(inputLanguage),
                Uri.EscapeDataString(outputLanguage));

            string json = GetJson(translateUri);
            // string json = @"{ ""data"": {  ""translations"": [   {    ""translatedText"": ""Le singe est dans l&#39;arbre""   }  ] }}";
            Google2Response response = (Google2Response)JsonToObject(json, typeof(Google2Response));
            if (response.Data == null || response.Data.TranslatedTexts == null || response.Data.TranslatedTexts.GetLength(0) == 0)
                throw new ApplicationException();

            return response.Data.TranslatedTexts[0].TranslatedText;
        }

        private static object JsonToObject(string json, Type jsonType)
        {
            byte[] byteArray = Encoding.UTF8.GetBytes(json);
            MemoryStream memoryStream = new MemoryStream(byteArray);
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(jsonType);
            return serializer.ReadObject(memoryStream);
        }

        private static string GetJson(string requestUriString)
        {
            HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create(requestUriString);
            httpWebRequest.Method = "GET";
            httpWebRequest.ContentType = "text/plain";

            // Invoke the request and get the response
            WebResponse response = httpWebRequest.GetResponse();
            Stream responseStream = response.GetResponseStream();
            string output;
            using (StreamReader reader = new StreamReader(responseStream))
            {
                output = reader.ReadToEnd();
            }

            return output;
        }

        private static string[] GetLanguageList()
        {
            // string languages = @"{""data"":{""languages"":[{""language"":""en""},{""language"":""fr""},{""language"":""zh-CN""}]}}";
            string requestUriString = String.Format("https://www.googleapis.com/language/translate/v2/languages?key={0}", ApplicationKey);
            string languages = GetJson(requestUriString);

            Google2LanguagesResponse response = (Google2LanguagesResponse)JsonToObject(languages, typeof(Google2LanguagesResponse));

            List<string> languageList = new List<string>();
            foreach (Google2Language language in response.Data.Languages)
            {
                if (String.Compare(language.Language, "en", true) != 0)
                {
                    languageList.Add(language.Language);
                }
            }

            return languageList.ToArray();
        }
    }

    [DataContract]
    #pragma warning disable 649
    internal class Google2TranslatedText
    {
        [DataMember(Name = "translatedText")]
        public string TranslatedText;
    }

    [DataContract]
    internal class Google2Translations
    {
        [DataMember(Name = "translations")]
        public Google2TranslatedText[] TranslatedTexts;
    }

    [DataContract]
    internal class Google2Response
    {
        [DataMember(Name = "data")]
        public Google2Translations Data;
    }

    [DataContract]
    internal class Google2Language
    {
        [DataMember(Name = "language")]
        public string Language;
    }

    [DataContract]
    internal class Google2LanguagesList
    {
        [DataMember(Name = "languages")]
        public Google2Language[] Languages;
    }

    [DataContract]
    internal class Google2LanguagesResponse
    {

        [DataMember(Name = "data")]
        public Google2LanguagesList Data;
    }
}
