﻿// File           : GoogleTranslator.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : October 2010
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Net;
using System.IO;
using System.Text;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;

namespace Internationalization.Translation
{
    /// <summary>
    /// GoogleTranslator is a translator for the Google Translate API
    /// </summary>
    /// <remarks>
    /// Google has deprecated its free translate API as of December 2011. If you use this translator you will most likely receive a "Terms of Service violation" error.
    /// </remarks>
    public class GoogleTranslator : Translator
    {
        /// <summary>
        /// Constructs a GoogleTranslator object
        /// </summary>
        public GoogleTranslator() : base(GoogleTranslatorResources.Name, GetLanguagePairs())
        {
        }

        /// <summary>
        /// GetLanguagePairs gets the array of translatable language pairs
        /// </summary>
        /// <returns>The array of translatable language pairs</returns>
        protected static string[,] GetLanguagePairs()
        {
            // the language list contains all of the languages that can be translated to/from English
            string[] languageList = GetLanguageList();
            int length = languageList.GetLength(0);
            string[,] languagePairs = (string[,])Array.CreateInstance(typeof(string), length * 2, 2);

            int languagePairNumber = 0;
            foreach (string language in languageList)
            {
                // add the language pair for translating from English
                languagePairs[languagePairNumber, 0] = "en";
                languagePairs[languagePairNumber, 1] = language;
                languagePairNumber++;

                // add the language pair for translating to English
                languagePairs[languagePairNumber, 0] = language;
                languagePairs[languagePairNumber, 1] = "en";
                languagePairNumber++;
            }

            return languagePairs;
        }

        /// <summary>
        /// Translate translates the input text from the input language to the output language
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to translate</param>
        /// <returns>The translated text</returns>
        public override string Translate(string inputLanguage, string outputLanguage, string text)
        {
            if (!IsSupported(inputLanguage, outputLanguage))
                throw new LanguageCombinationNotSupportedException(
                    LanguageCombinationNotSupportedResources.LanguageCombinationNotSupported,
                    inputLanguage, outputLanguage);

            string languagePair = inputLanguage + "|" + outputLanguage;
            string json = this.GetJson(text, languagePair);
            GoogleResponse response = JsonToGoogleResponse(json);
            if (response.ResponseData == null && response.ResponseStatus == 403)
                // this is almost certainly a Terms of Service violation
                throw new ApplicationException(response.ResponseDetails);

            return response.ResponseData.TranslatedText;
        }

        private GoogleResponse JsonToGoogleResponse(string json)
        {
            byte[] byteArray = Encoding.UTF8.GetBytes(json);
            MemoryStream memoryStream = new MemoryStream(byteArray);
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(GoogleResponse));
            return (GoogleResponse)serializer.ReadObject(memoryStream);
        }

        private string GetJson(string text, string languagePair)
        {
            string translateUri = String.Format(
                "http://ajax.googleapis.com/ajax/services/language/translate?v=1.0&q={0}&langpair={1}",
                Uri.EscapeDataString(text),
                Uri.EscapeDataString(languagePair));
            HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create(translateUri);
            httpWebRequest.Method = "GET";
            httpWebRequest.ContentType = "text/plain";

            // Invoke the request and get the response
            WebResponse response = httpWebRequest.GetResponse();
            Stream responseStream = response.GetResponseStream();
            string output;
            using (StreamReader reader = new StreamReader(responseStream))
            {
                output = reader.ReadToEnd();
            }

            return output;
        }

        private static string[] GetLanguageList()
        {
            return new string[]
                {
                    "af", // Afrikaans
                    "sq", // Albanian
                    "ar", // Arabic
                    "be", // Belarusian
                    "bg", // Bulgarian
                    "zh", // Chinese
                    "ca", // Catalan
                    "hr", // Croatian
                    "cs", // Czech
                    "da", // Danish
                    "nl", // Dutch
                    "et", // Estonian
                    "fil", // Filipino
                    "fi", // Finnish
                    "fr", // French
                    "gl", // Galician
                    "de", // German
                    "el", // Greek
                    // Haitian Creole
                    "he", // Hebrew
                    "hi", // Hindi
                    "hu", // Hungarian
                    "is", // Icelandic
                    "id", // Indonesian
                    "ga", // Irish
                    "it", // Italian
                    "ja", // Japanese
                    "ko", // Korean
                    "lv", // Latvian
                    "lt", // Lithuanian
                    "mk", // Macedonian (FYROM)
                    "ms", // Malay
                    "mt", // Maltese
                    "no", // Norwegian
                    "fa", // Persian
                    "pl", // Polish
                    "pt", // Portuguese
                    "ro", // Romanian
                    "ru", // Russian
                    "es", // Spanish
                    "sr", // Serbian
                    "sk", // Slovak
                    "sl", // Slovenian
                    "sw", // Kiswahili
                    "sv", // Swedish
                    "th", // Thai
                    "tr", // Turkish
                    "uk", // Ukrainian
                    "vi", // Vietnamese
                    "cy" // Welsh
                    // Yiddish
                };
        }
    }

    [DataContract]
    #pragma warning disable 649
    internal class GoogleResponse
    {
        // {"responseData": {"translatedText":"ciao a tutti"}, "responseDetails": null, "responseStatus": 200}

        [DataMember(Name = "responseData")]
        public GoogleTranslatedText ResponseData;

        [DataMember(Name = "responseDetails")]
        public string ResponseDetails;

        [DataMember(Name = "responseStatus")]
        public int ResponseStatus;
    }

    [DataContract]
    internal class GoogleTranslatedText
    {
        [DataMember(Name = "translatedText")]
        public string TranslatedText;
    }
}
