// File           : HtmlTranslator.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Windows.Forms;
using System.Web;
using System.Net;
using System.IO;

namespace Internationalization.Translation
{
	/// <summary>
	/// HtmlTranslator is an abstract base class for translators based on websites
	/// </summary>
	public abstract class HtmlTranslator: Translator
	{
		private string url;

		private WebBrowserForm webBrowserForm;
		private System.Windows.Forms.WebBrowser webBrowser;

        /// <summary>
        /// Constructs an HtmlTranslator object
        /// </summary>
        /// <param name="name">name is the free form name of the translator</param>
        /// <param name="url">url is the URL of the page which translates text</param>
        public HtmlTranslator(string name, string url)
            : base(name)
		{
			this.url = url;
		}
        /// <summary>
        /// Constructs an HtmlTranslator object
        /// </summary>
        /// <param name="name">name is the free form name of the translator</param>
        /// <param name="url">url is the URL of the page which translates text</param>
        /// <param name="languagePairs">languagePairs is an array of language pair strings 
        /// supported by the translator e.g. {{"en", "fr"}, {"en", "de"}}</param>
        /// <remarks>Use this constructor if the list of language pairs supported by the translator
        /// is known at construction time</remarks>
        public HtmlTranslator(string name, string url, string[,] languagePairs)
            : base(name, languagePairs)
		{
			this.url = url;
		}
        /// <summary>
        /// Url is the URL of the page which translates text
        /// </summary>
		public string Url
		{
			get {return url;}
			set {url = value;}
		}
        /// <summary>
        /// InitializeWebBrowser initializes the internal web browser used to communicate with the web page
        /// </summary>
		protected virtual void InitializeWebBrowser()
		{
			if (webBrowser == null)
			{
				webBrowserForm = new WebBrowserForm();
				webBrowser = webBrowserForm.WebBrowser;
			}
		}
        /// <summary>
        /// GetInnerText sends the request to the URL and returns the inner text from the resulting page
        /// </summary>
        /// <param name="postData">The data to post to the page</param>
        /// <returns>The inner text of the resulting page</returns>
		protected virtual string GetInnerText(string postData)
		{
			string headers = "Content-Type: application/x-www-form-urlencoded" + (char) 10 + (char) 13;

			byte[] bytePostData = System.Text.Encoding.ASCII.GetBytes(postData);

			webBrowser.Navigate(new Uri(url), String.Empty, bytePostData, headers);

			webBrowserForm.WaitForBrowser();

			return webBrowser.Document.Body.InnerText;
		}
        /// <summary>
        /// GetPostData gets the website-specific post data for sending to the URL
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to be translated</param>
        /// <returns>The post data to send to the URL</returns>
		public abstract string GetPostData(string inputLanguage, string outputLanguage, string text);

        /// <summary>
        /// Translate translates the input text from the input language to the output language
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to translate</param>
        /// <returns>The translated text</returns>
		public override string Translate(string inputLanguage, string outputLanguage, string text)
		{
			if (! IsSupported(inputLanguage, outputLanguage))
				throw new LanguageCombinationNotSupportedException(
					LanguageCombinationNotSupportedResources.LanguageCombinationNotSupported,
                    inputLanguage, outputLanguage);

			InitializeWebBrowser();

			string innerText = GetInnerText(GetPostData(inputLanguage, outputLanguage, text));
			return GetTranslation(inputLanguage, outputLanguage, innerText);
		}
        /// <summary>
        /// Encode encodes the input text
        /// </summary>
        /// <param name="text">The text to be translated</param>
        /// <returns>An encoded version of the text to be translated</returns>
		protected string Encode(string text)
		{
            return HttpUtility.UrlEncode(text);
		}
        /// <summary>
        /// GetTranslation extracts the translated text from the inner text returned from the website
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="innerText">The inner text returned from the website</param>
        /// <returns>The translated text</returns>
        /// <remarks>This implementation is unlikely to be useful for any HtmlTranslator as the extraction
        /// of the translated text is very specific to the individual website. As such this implementation
        /// is of use mainly during development of new HtmlTranslators.</remarks>
		protected virtual string GetTranslation(string inputLanguage, string outputLanguage, string innerText)
		{
			return innerText;
		}
	}
}
