﻿// File           : MicrosoftTranslator.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : March 2009
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Net;
using System.IO;
using System.Text;

namespace Internationalization.Translation
{
    /// <summary>
    /// MicrosoftTranslator is a translator for the Microsoft Translator
    /// </summary>
    /// <remarks>The Microsoft Translator is available for non-commercial use.
    /// To use the Microsoft Translator for commercial use contact Microsoft.
    /// To get an application ID for use with this translator go to
    /// http://search.live.com/developers/appids.aspx.</remarks>
    public class MicrosoftTranslator : Translator
    {
        /// <summary>
        /// Gets or sets the SearchApplicationId used to connect to the Microsoft Translator
        /// </summary>
        /// <remarks>Your search application ID can be requested from http://search.live.com/developers/createapp.aspx.
        /// This is NOT your AJAX API application ID (starting "AJX") nor your widget application ID (starting
        /// "WDG"). It is a separate ID used for search.</remarks>
        public static string SearchApplicationId { get; set; }

        /// <summary>
        /// Constructs a MicrosoftTranslator object
        /// </summary>
        public MicrosoftTranslator() : base(MicrosoftTranslatorResources.Name, GetLanguagePairs())
        {
        }

        /// <summary>
        /// GetLanguagePairs gets the array of translatable language pairs
        /// </summary>
        /// <returns>The array of translatable language pairs</returns>
        protected static string[,] GetLanguagePairs()
        {
            // the language list contains all of the languages that can be translated to/from English
            string[] languageList = GetLanguageList();
            int length = languageList.GetLength(0);
            string[,] languagePairs = (string[,]) Array.CreateInstance(typeof(string), (length - 1) * 2 , 2);

            int languagePairNumber = 0;
            foreach (string language in languageList)
            {
                if (language != "en")
                {
                    // add the language pair for translating from English
                    languagePairs[languagePairNumber, 0] = "en";
                    languagePairs[languagePairNumber, 1] = language;
                    languagePairNumber++;

                    // add the language pair for translating to English
                    languagePairs[languagePairNumber, 0] = language;
                    languagePairs[languagePairNumber, 1] = "en";
                    languagePairNumber++;
                }
            }

            return languagePairs;
        }

        /// <summary>
        /// GetSearchApplicationId throws an exception if SearchApplicationId is empty; otherwise it returns the SearchApplicationId
        /// </summary>
        /// <returns>The SearchApplicationId</returns>
        protected static string GetSearchApplicationId()
        {
            if (String.IsNullOrEmpty(SearchApplicationId))
                throw new TranslationException(MicrosoftTranslatorResources.SearchApplicationIdNotSet);

            return SearchApplicationId;
        }

        /// <summary>
        /// GetLanguageList gets the array of supported languages
        /// </summary>
        /// <returns>The array of supported languages</returns>
        private static string[] GetLanguageList()
        {
            WebRequest webRequest = System.Net.WebRequest.Create(
                String.Format(@"http://api.microsofttranslator.com/V1/Http.svc/GetLanguages?appId={0}", GetSearchApplicationId()));

            WebResponse webResponse = webRequest.GetResponse();

            string languageList;
            Stream stream = webResponse.GetResponseStream();
            using (StreamReader reader = new System.IO.StreamReader(stream))
            {
                // Create a string for the contents of the web response
                languageList = reader.ReadToEnd();
            }

            // languageList is a value like "ar\rzh-CHS\rzh-CHT\rnl\ren\rfr\rde\rit\rja\rko\rpl\rpt\rru\res\r"
            languageList = languageList.Replace("\n", String.Empty);
            if (languageList.EndsWith("\r"))
                // strip the trailing \r
                languageList = languageList.Substring(0, languageList.Length - 1);

            return languageList.Split('\r');
        }

        /// <summary>
        /// Translate translates the input text from the input language to the output language
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to translate</param>
        /// <returns>The translated text</returns>
        public override string Translate(string inputLanguage, string outputLanguage, string text)
        {
            if (!IsSupported(inputLanguage, outputLanguage))
                throw new LanguageCombinationNotSupportedException(
                    LanguageCombinationNotSupportedResources.LanguageCombinationNotSupported,
                    inputLanguage, outputLanguage);

            string translateUri = String.Format(
                "http://api.microsofttranslator.com/V1/Http.svc/Translate?appId={0}&from={1}&to={2}",
                GetSearchApplicationId(),
                inputLanguage, outputLanguage);

            HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create(translateUri);
            httpWebRequest.Method = "POST";
            httpWebRequest.ContentType = "text/plain";
            AssignTextToHttpWebRequest(text, httpWebRequest);

            // Invoke the request and get the response
            WebResponse response = httpWebRequest.GetResponse();
            Stream responseStream = response.GetResponseStream();
            string output;
            using (StreamReader reader = new StreamReader(responseStream))
            {
                output = reader.ReadToEnd();
            }
            return output;
        }

        private static void AssignTextToHttpWebRequest(string text, HttpWebRequest httpWebRequest)
        {
            byte[] bytes = Encoding.ASCII.GetBytes(text);
            Stream requestStream = null;
            try
            {
                httpWebRequest.ContentLength = bytes.Length;
                requestStream = httpWebRequest.GetRequestStream();
                requestStream.Write(bytes, 0, bytes.Length);
            }
            finally
            {
                if (requestStream != null)
                {
                    requestStream.Close();
                }
            }
        }
    }
}
