// File           : PseudoTranslator.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Text;

namespace Internationalization.Translation
{
    /// <summary>
    /// The PseudoTranslationPadding enumeration specifies how padding should be applied by the PseudoTranslator
    /// </summary>
	public enum PseudoTranslationPadding {
        /// <summary>
        /// None means no padding is applied to either the left or right side of the string
        /// </summary>
        None,
        /// <summary>
        /// Left means padding is applied to the left side of the string
        /// </summary>
        Left,
        /// <summary>
        /// Right means padding is applied to the right side of the string
        /// </summary>
        Right,
        /// <summary>
        /// Both means padding is applied to both (left and right) sides of the string
        /// </summary>
        Both};

	/// <summary>
    /// PseudoTranslator is a translator which performs a pseudo translation
	/// </summary>
	/// <remarks>PseudoTranslator is based upon an idea expressed in "Developing International Software"
	/// by Dr. International</remarks>
	public class PseudoTranslator: Translator
	{
		private static PseudoTranslationPadding padding = PseudoTranslationPadding.Both;

        private static bool excludeMarkup = true;

        /// <summary>
        /// Constructs a PseudoTranslator object
        /// </summary>
		public PseudoTranslator(): base(PseudoTranslatorResources.Name, 
			new string[,] {{"*", Internationalization.Common.PseudoTranslation.CultureInfo.Name}})
		{
		}

        /// <summary>
        /// ExcludeMarkup specifies whether markup should be excluded from the pseudo translation
        /// </summary>
        public static bool ExcludeMarkup
        {
            get { return excludeMarkup; }
            set { excludeMarkup = value; }
        }

        /// <summary>
        /// Padding is a PseudoTranslationPadding enumeration specifying how padding should be applied to the
        /// translated string
        /// </summary>
		public static PseudoTranslationPadding Padding
		{
			get {return padding;}
			set {padding = value;}
		}
        /// <summary>
        /// Translate translates the input text from the input language to the output language
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to translate</param>
        /// <returns>The translated text</returns>
        public override string Translate(string inputLanguage, string outputLanguage, string text)
		{
			if (! IsSupported(inputLanguage, outputLanguage))
				throw new LanguageCombinationNotSupportedException(
					LanguageCombinationNotSupportedResources.LanguageCombinationNotSupported,
                    inputLanguage, outputLanguage);

			return PseudoTranslate(text);
		}
        /// <summary>
        /// PseudoTranslate translates a string to pseudo translated version of the same string
        /// </summary>
        /// <param name="text">The text to translate</param>
        /// <returns>A pseudo translated and potentially padded string</returns>
        /// <remarks>The string is padded on the left and/or right according to the Padding property.</remarks>
		protected virtual string PseudoTranslate(string text)
		{
			StringBuilder stringBuilder = new StringBuilder("[");
			if (padding == PseudoTranslationPadding.Left || padding == PseudoTranslationPadding.Both)
			{
				// add padding on the left
				for(int padCount = 0; padCount < GetPaddingOnOneSideCount(text); padCount++)
				{
					stringBuilder.Append("!!! ");
				}
			}

			bool previousCharacterIsSlash = false;
            bool insideMarkup = false;

			for(int index = 0; index < text.Length; index++)
			{
                char chr = text[index];
                if (chr == '<' || (index + 4 <= text.Length && text.Substring(index, 4) == "&lt;"))
                {
                    insideMarkup = true;
                    stringBuilder.Append(chr);
                }
                else if (chr == '>' || (index > 4 && text.Substring(index - 3, 4) == "&gt;"))
                {
                    insideMarkup = false;
                    stringBuilder.Append(chr);
                }
                else if (insideMarkup && ExcludeMarkup)
                    // we are currently inside some markup and markup should be excluded from the pseudo translation process
                    stringBuilder.Append(chr);
                else
                {
                    if (previousCharacterIsSlash)
                        stringBuilder.Append(chr);
                    else
                        stringBuilder.Append(ConvertCharacter(chr));

                    previousCharacterIsSlash = (chr == @"\"[0]);
                }
			}

			if (padding == PseudoTranslationPadding.Right || padding == PseudoTranslationPadding.Both)
			{
				// add padding on the right
				for(int padCount = 0; padCount < GetPaddingOnOneSideCount(text); padCount++)
				{
					stringBuilder.Append(" !!!");
				}
			}
			stringBuilder.Append("]");
			return stringBuilder.ToString();
		}
        /// <summary>
        /// GetPaddingOnOneSideCount returns the number of padding blocks which should be applied to 
        /// one side of a string
        /// </summary>
        /// <param name="text">The text to get the padding size count for</param>
        /// <returns>The number of padding blocks for one side of a string</returns>
        /// <remarks>A padding block is "!!! " (left side) or " !!!" (right side)</remarks>
		protected virtual int GetPaddingOnOneSideCount(string text)
		{
			int paddingCount;
			if (text.Length < 10)
				// pad size by 400% (the length of the padding string is 4 so to increase the size by 400%
				// the pad count should be the same as the length of the string)
				paddingCount = text.Length;
			else
				// pad size by 30%
				paddingCount = (int) (text.Length * 0.3) / 4;

			int paddingOnOneSideCount;
			switch (padding)
			{
				case (PseudoTranslationPadding.Both) : {paddingOnOneSideCount = (int) (paddingCount / 2); break;}
				case (PseudoTranslationPadding.Left) :
				case (PseudoTranslationPadding.Right): {paddingOnOneSideCount = paddingCount; break;}
				default : return 0;
			}
			if (paddingOnOneSideCount > 0)
				return paddingOnOneSideCount;
			else
				return 1;
		}
        /// <summary>
        /// ConvertCharacter converts a non-pseudo translated character to a pseudo translated character
        /// </summary>
        /// <param name="chr">The character to convert</param>
        /// <returns>The pseudo translated character</returns>
        /// <remarks>The success of the pseudo translation is dependant upon the character set of the incoming
        /// text. This algorithm works best with English. It could be modified for other languages.</remarks>
		protected virtual char ConvertCharacter(char chr)
		{
			switch (chr)
			{
				case 'A' : return 'Ă';
				case 'B' : return 'ß';
				case 'C' : return 'Č';
				case 'D' : return 'Đ';
				case 'E' : return 'Ĕ';
				case 'F' : return 'F';
				case 'G' : return 'Ğ';
				case 'H' : return 'Ħ';
				case 'I' : return 'Ĭ';
				case 'J' : return 'Ĵ';
				case 'K' : return 'Ķ';
				case 'L' : return 'Ľ';
				case 'M' : return 'M';
				case 'N' : return 'Ň';
				case 'O' : return 'Ŏ';
				case 'P' : return 'Ρ';
				case 'Q' : return 'Q';
				case 'R' : return 'Ř';
				case 'S' : return 'Š';
				case 'T' : return 'Ť';
				case 'U' : return 'Ŭ';
				case 'V' : return 'V';
				case 'W' : return 'Ŵ';
				case 'X' : return 'X';
				case 'Y' : return 'Ÿ';
				case 'Z' : return 'Ž';
				case 'a' : return 'ä';
				case 'b' : return 'þ';
				case 'c' : return 'č';
				case 'd' : return 'đ';
				case 'e' : return 'ĕ';
				case 'f' : return 'ƒ';
				case 'g' : return 'ğ';
				case 'h' : return 'ħ';
				case 'i' : return 'ĭ';
				case 'j' : return 'ĵ';
				case 'k' : return 'ĸ';
				case 'l' : return 'ľ';
				case 'm' : return 'm';
				case 'n' : return 'ŉ';
				case 'o' : return 'ŏ';
				case 'p' : return 'p';
				case 'q' : return 'q';
				case 'r' : return 'ř';
				case 's' : return 'š';
				case 't' : return 'ť';
				case 'u' : return 'ǔ';
				case 'v' : return 'v';
				case 'w' : return 'ŵ';
				case 'x' : return 'χ';
				case 'y' : return 'Ў';
				case 'z' : return 'ž';
				default  : return chr;
			}
		}
	}
}
