// File           : Translator.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Text;
using System.Text.RegularExpressions;

namespace Internationalization.Translation
{
	/// <summary>
    /// Translator is a base class implementation of ITranslator
	/// </summary>
	public abstract class Translator: ITranslator
	{
		private bool enabled = true;
		private string name;
		private string[,] languagePairs;

        /// <summary>
        /// Constructs a new Translator object
        /// </summary>
        /// <param name="name">name is the free form name of the translator</param>
        /// <remarks>Use this constructor if the list of language pairs supported by the translator
        /// is either dynamic or is not known at construction time or no pairs are supported</remarks>
		public Translator(string name)
		{
			this.name = name;
		}
        /// <summary>
        /// Constructs a new Translator object
        /// </summary>
        /// <param name="name">name is the free form name of the translator</param>
        /// <param name="languagePairs">languagePairs is an array of language pair strings 
        /// supported by the translator e.g. {{"en", "fr"}, {"en", "de"}}</param>
        /// <remarks>Use this constructor if the list of language pairs supported by the translator
        /// is known at construction time</remarks>
        public Translator(string name, string[,] languagePairs)
		{
			this.name = name;
			this.languagePairs = languagePairs;
		}
        /// <summary>
        /// The name of the translator
        /// </summary>
		public string Name
		{
			get {return name;}
		}
        /// <summary>
        /// Enabled indicates whether the translator is enabled or not
        /// </summary>
        /// <remarks>A translator is often disabled after it has failed. You could set Enabled back to
        /// true again during the course of some translator scavenging operation.</remarks>
		public bool Enabled
		{
			get {return enabled;}
			set {enabled = value;}
		}
        /// <summary>
        /// LanguagePairs is an array of language pair strings supported by the translator e.g. {{"en", "fr"}, {"en", "de"}}
        /// </summary>
		public virtual string[,] LanguagePairs
		{
			get {return languagePairs;}
		}
        /// <summary>
        /// IsSupported returns true if the given language pair is supported by the translator
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <returns>True if the given language pair is supported by the translator</returns>
        /// <remarks>The implementation of this method in this base class is simply to iterate through the
        /// list of supported language pairs looking for a match. If your list is dynamic override this
        /// implementation to provide suitable logic.</remarks>
		public virtual bool IsSupported(string inputLanguage, string outputLanguage)
		{
			if (languagePairs == null)
				return false;

			if (inputLanguage.Length < 2 || outputLanguage.Length < 2)
				return false;

			for(int pairNumber = 0; pairNumber < languagePairs.GetLength(0); pairNumber++)
			{
				// If the "from" language is "*" it matches all input languages.
				if ((inputLanguage == languagePairs[pairNumber, 0] || languagePairs[pairNumber, 0] == "*")
					&& outputLanguage == languagePairs[pairNumber, 1])
					return true;
			}
			return false;
		}
        /// <summary>
        /// Translate translates the input text from the input language to the output language
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to translate</param>
        /// <returns>The translated text</returns>
		public abstract string Translate(string inputLanguage, string outputLanguage, string text);

        /// <summary>
        /// ConvertFromEscapedNumerics converts text with embedded escaped numerics back into plain text
        /// </summary>
        /// <param name="text">A string which potentially contains escaped numerics</param>
        /// <returns>A plain text string with all escaped numerics converted back to plain text</returns>
		public static string ConvertFromEscapedNumerics(string text)
		{
			string ampersandHash = @"[" + Regex.Escape("&") + "][" + Regex.Escape("#") + @"]";
			string literalSemiColon = @"[" + Regex.Escape(";") + @"]";

			// this matches "&#" followed by two or three numerics followed by ";"
			string pattern = 
				ampersandHash + @"(\d\d)" + literalSemiColon + @"|" + 
				ampersandHash + @"(\d\d\d)" + literalSemiColon;

			text = Regex.Replace(text, pattern, new MatchEvaluator(EscapeText));

			return text;
		}
        /// <summary>
        /// EscapeText is a MatchEvaluator used by ConvertFromEscapedNumerics to replace escaped numerics with
        /// their equivalent characters
        /// </summary>
        /// <param name="match">The Regex Match object</param>
        /// <returns>The string to replace the escaped numeric with</returns>
		protected static string EscapeText(Match match)
		{
			string text = match.ToString();
			string characterNumber = text.Substring(2, text.Length - 3);
			return ((Char) Int32.Parse(characterNumber)).ToString();
		}
        /// <summary>
        /// ConvertToEscapedNumerics converts strings with characters higher than 7-bit ASCII to strings
        /// containing escaped numerics
        /// </summary>
        /// <param name="text">The text potentially containing characters higher than 7-bit ASCII</param>
        /// <returns>A string containing escaped numerics</returns>
		public static string ConvertToEscapedNumerics(string text)
		{
			StringBuilder stringBuilder = new StringBuilder();

			foreach(char chr in text)
			{
				if (chr > 127)
					stringBuilder.Append(@"&#" + ((int) chr).ToString() + ";");
				else
					stringBuilder.Append(chr);
			}
			return stringBuilder.ToString();
		}
        /// <summary>
        /// ConvertPunctuationToEscapedNumerics converts strings with punctuation characters to strings
        /// containing escaped numerics
        /// </summary>
        /// <param name="text">The text potentially containing punctuation characters</param>
        /// <returns>A string containing escaped numerics</returns>
        public static string ConvertPunctuationToEscapedNumerics(string text)
		{
			StringBuilder stringBuilder = new StringBuilder();

			foreach(char chr in text)
			{
				if (Char.IsPunctuation(chr))
					stringBuilder.Append(@"&#" + ((int) chr).ToString() + ";");
				else
					stringBuilder.Append(chr);
			}
			return stringBuilder.ToString();
		}
	}
}
