// File           : TranslatorCollection.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;

namespace Internationalization.Translation
{
    /// <summary>
    /// TranslatorCollection is a collection of ITranslator objects
    /// </summary>
	public class TranslatorCollection : CollectionBase  
	{
        /// <summary>
        /// Indexer for collection
        /// </summary>
        /// <param name="index">Ordinal position of ITranslator object</param>
        /// <returns>The ITranslator object at the given index</returns>
		public ITranslator this[int index]
		{
			get  {return((ITranslator) List[index]);}
			set  {List[index] = value;}
		}
        /// <summary>
        /// Adds an ITranslator object to the collection
        /// </summary>
        /// <param name="value">The ITranslator object to add to the collection</param>
        /// <returns>The ordinal position of the added ITranslator object</returns>
		public int Add(ITranslator value)
		{
			return(List.Add(value));
		}
        /// <summary>
        /// IndexOf returns the index of the given ITranslator object
        /// </summary>
        /// <param name="value">The ITranslator object to get the index for</param>
        /// <returns>The index of the given ITranslator object</returns>
		public int IndexOf(ITranslator value)
		{
			return(List.IndexOf(value));
		}
        /// <summary>
        /// IndexOf returns the index of the given ITranslator.Name
        /// </summary>
        /// <param name="name">The ITranslator.Name to get the index for</param>
        /// <returns>The index of the given ITranslator.Name</returns>
		public int IndexOf(string name)
		{
			for(int index = 0; index < List.Count; index++)
			{
				if (((ITranslator) List[index]).Name == name)
					return index;
			}
			return -1;
		}
        /// <summary>
        /// GetTranslator gets an ITranslator object for the given language pair
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <returns>An ITranslator object which supports the given language pair</returns>
		public ITranslator GetTranslator(string inputLanguage, string outputLanguage)
		{
			int index = IndexOf(inputLanguage, outputLanguage);
			if (index == -1)
				return null;
			return (ITranslator) List[index];
		}
        /// <summary>
        /// GetEnabledTranslator gets an enabled ITranslator object for the given language pair
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <returns>An enabled ITranslator object which supports the given language pair</returns>
        public ITranslator GetEnabledTranslator(string inputLanguage, string outputLanguage)
		{
			int index = EnabledIndexOf(inputLanguage, outputLanguage);
			if (index == -1)
				return null;
			return (ITranslator) List[index];
		}
        /// <summary>
        /// IndexOf returns the index of the ITranslator object which supports the given language pair
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <returns>The index of the ITranslator object which supports the language pair</returns>
		public int IndexOf(string inputLanguage, string outputLanguage)
		{
			for(int index = 0; index < List.Count; index++)
			{
				if (((ITranslator) List[index]).IsSupported(inputLanguage, outputLanguage))
					return index;
			}
			return -1;
		}
        /// <summary>
        /// EnabledIndexOf returns the index of the enabled ITranslator object which supports the given language pair
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <returns>The index of the enabled ITranslator object which supports the language pair</returns>
        public int EnabledIndexOf(string inputLanguage, string outputLanguage)
		{
			for(int index = 0; index < List.Count; index++)
			{
				ITranslator translator = (ITranslator) List[index];
				if (translator.Enabled && translator.IsSupported(inputLanguage, outputLanguage))
					return index;
			}
			return -1;
		}
        /// <summary>
        /// Insert inserts an ITranslator object at the given ordinal position
        /// </summary>
        /// <param name="index">The ordinal position at which the ITranslator object should be inserted</param>
        /// <param name="value">The ITranslator object to insert</param>
		public void Insert(int index, ITranslator value)
		{
			List.Insert(index, value);
		}
        /// <summary>
        /// Remove removes an ITranslator object from the collection
        /// </summary>
        /// <param name="value">The ITranslator to remove</param>
		public void Remove(ITranslator value)
		{
			List.Remove(value);
		}
        /// <summary>
        /// Contains returns true if the collection contains the given ITranslator object
        /// </summary>
        /// <param name="value">The ITranslator object to test the collection for</param>
        /// <returns>True if the ITranslator object is in the collection</returns>
		public bool Contains(ITranslator value)
		{
			return(List.Contains(value));
		}
	}
}