// File           : MainForm.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Configuration;
using System.Text;
using System.Globalization;
using System.Resources;
using System.Reflection;
using System.IO;
using System.Collections.Specialized;
using System.Xml;
using Internationalization.Common;
using Internationalization.Translation;
using Internationalization.Translation.ResearchServices;
using Internationalization.ResourceAdministration;
using System.Threading;

namespace Internationalization.ResourceAdministrator
{
	/// <summary>
	/// MainForm is the main form of the Resource Administrator
	/// </summary>
	public class MainForm : System.Windows.Forms.Form, ITranslationHelper
	{
		private string invariantLanguage;
		private string targetLanguages;
		private string source;
		private bool automaticTranslation = true;
        private string resourcesGovernorClassName;

		private StatusBarPanel invariantLanguagePanel;
		private StatusBarPanel targetLanguagesPanel;
		private StatusBarPanel sourcePanel;
		private StatusBarPanel automaticTranslationPanel;

		private int resourceEntriesAdded;

		private IResourcesGovernor resourcesGovernor;
		private SetOfResourceSetsCollection setOfResourceSetsCollection;
		// allTranslators is the complete list of translators which are available to this application
		// (regardless of whether they are working or not). After the initial load this list does not
		// change.
		private TranslatorCollection allTranslators;
		// targetLanguageTranslators is a list of translators which is in parallel with the 
		// TargetLanguages.Languages list. Each element in targetLanguageTranslators corresponds
		// to the same ordinal position in the TargetLanguages.Languages list so the translator
		// for the second target language is the second element in the targetLanguageTranslators
		// list. Note that the elements of this list are volatile (but the number is always the
		// same as the number of TargetLanguages.Languages). So a translator which is used to 
		// translate English into Russian might fail and be replaced with another translator or
		// no translator at all.
		private ArrayList targetLanguageTranslators;

        private ExportForm exportForm;

		private IntegrityCheckForm integrityCheckForm;
		private System.Windows.Forms.GroupBox groupBoxResource;
		private System.Windows.Forms.Splitter splitterTargetLanguagesStatus;
		private System.Windows.Forms.GroupBox groupBoxTargetLanguagesStatus;
		private System.Windows.Forms.ListBox listBoxTargetLanguagesStatus;
		private System.Windows.Forms.Splitter splitter1;
		private System.Windows.Forms.GroupBox groupBox3;
		private System.Windows.Forms.TreeView treeViewResources;
		private System.Windows.Forms.StatusBar statusBar;
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuItemExit;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuItemLanguages;
		private System.Windows.Forms.MenuItem menuItemTranslators;
		private System.Windows.Forms.MenuItem menuItem6;
		private System.Windows.Forms.MenuItem menuItemIntegrityCheck;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem menuItemAutomaticTranslation;
		private System.Windows.Forms.ContextMenu contextMenuResources;
		private System.Windows.Forms.MenuItem menuItemCreateMissingResources;
		private System.Windows.Forms.MenuItem menuItemAddMissingResourceEntries;
        private System.Windows.Forms.MenuItem menuItemResourcesSource;
        private MenuItem menuItemExport;
        private ResourceEditorControl resourceEditorControl;
        private IContainer components;

        /// <summary>
        /// Constructs a MainForm object
        /// </summary>
		public MainForm()
		{
			InitializeComponent();

            resourceEditorControl.ResourceSetChanging +=
                new ResourceSetChangingEventHandler(resourceEditorControlResourceSetChanging);

            resourceEditorControl.DataNodeCreating +=
                new DataNodeCreatingEventHandler(resourceEditorControlDataNodeCreating);

            resourceEditorControl.FileRefCreating +=
                new FileRefCreatingEventHandler(resourceEditorControlFileRefCreating);

            resourceEditorControl.ResourceAbsoluteFileName +=
                new ResourceFileNameEventHandler(resourceEditorControlResourceAbsoluteFileName);

            resourceEditorControl.ResourceRelativeFileName +=
                new ResourceFileNameEventHandler(resourceEditorControlResourceRelativeFileName);

            InitializeStatusBar();

            InitializeTranslators();

            if (! LoadConfiguration())
            {
                if (! SetSourceAndInvariantLanguage())
                {
                    MessageBox.Show(MainFormResources.MustHaveASource);
                    System.Environment.Exit(-1);
                }
            }
            else
			    ResourceAdministrationHelper.RefreshTargetLanguageTranslators(invariantLanguage, allTranslators, ref targetLanguageTranslators);
		}

        void resourceEditorControlResourceAbsoluteFileName(object sender, ResourceFileNameEventArgs eventArgs)
        {
            string path = resourcesGovernor.ResourceFileAbsolutePath;
            if (path != null && path != String.Empty)
            {
                if (path.EndsWith(Path.DirectorySeparatorChar.ToString()))
                    eventArgs.FileName = path + Path.GetFileName(eventArgs.FileName);
                else
                    eventArgs.FileName = path + Path.DirectorySeparatorChar + Path.GetFileName(eventArgs.FileName);
            }
        }
        void resourceEditorControlResourceRelativeFileName(object sender, ResourceFileNameEventArgs eventArgs)
        {
            string path = resourcesGovernor.ResourceFileRelativePath;
            if (path == null && path == String.Empty)
                // strip out the path information completely
                eventArgs.FileName = Path.GetFileName(eventArgs.FileName);
            else
            {
                if (path.EndsWith(Path.DirectorySeparatorChar.ToString()))
                    eventArgs.FileName = path + Path.GetFileName(eventArgs.FileName);
                else
                    eventArgs.FileName = path + Path.DirectorySeparatorChar + Path.GetFileName(eventArgs.FileName);
            }
        }
        private void resourceEditorControlDataNodeCreating(object sender, DataNodeCreatingEventArgs eventArgs)
        {
            string baseName = treeViewResources.SelectedNode.Text;
            SetOfResourceSets setOfResourceSets = (SetOfResourceSets)setOfResourceSetsCollection[baseName];
            IResourceGovernor resourceGovernor = setOfResourceSets.GetResourceGovernor(CultureInfo.InvariantCulture);
            eventArgs.Value = resourceGovernor.CreateDataNode(eventArgs.Name, eventArgs.Value);
        }
        private void resourceEditorControlFileRefCreating(object sender, FileRefCreatingEventArgs eventArgs)
        {
            string baseName = treeViewResources.SelectedNode.Text;
            SetOfResourceSets setOfResourceSets = (SetOfResourceSets)setOfResourceSetsCollection[baseName];
            IResourceGovernor resourceGovernor = setOfResourceSets.GetResourceGovernor(CultureInfo.InvariantCulture);
            eventArgs.FileRef = resourceGovernor.CreateFileRef(eventArgs.FileName, eventArgs.TypeName);
        }
        private void InitializeStatusBar()
		{
			invariantLanguagePanel = new StatusBarPanel();
			statusBar.Panels.Add(invariantLanguagePanel);
			invariantLanguagePanel.Text = MainFormResources.InvariantLanguage;
			invariantLanguagePanel.AutoSize = StatusBarPanelAutoSize.Contents;

			targetLanguagesPanel = new StatusBarPanel();
			statusBar.Panels.Add(targetLanguagesPanel);
			targetLanguagesPanel.Text = MainFormResources.TargetLanguages;
			targetLanguagesPanel.AutoSize = StatusBarPanelAutoSize.Contents;

			sourcePanel = new StatusBarPanel();
			statusBar.Panels.Add(sourcePanel);
			sourcePanel.Text = MainFormResources.Source;
			sourcePanel.AutoSize = StatusBarPanelAutoSize.Contents;

			automaticTranslationPanel = new StatusBarPanel();
			statusBar.Panels.Add(automaticTranslationPanel);
			automaticTranslationPanel.Text = MainFormResources.AutomaticTranslation;
			automaticTranslationPanel.AutoSize = StatusBarPanelAutoSize.Contents;
		}
        /// <summary>
        /// InvariantLanguage is the ISO 639 code of the invariant (fallback) resources
        /// </summary>
		public string InvariantLanguage
		{
			get {return invariantLanguage;}
			set
			{
				invariantLanguage = value;
				invariantLanguagePanel.Text = MainFormResources.InvariantLanguage + ": " + invariantLanguage;
			}
		}
        /// <summary>
        /// TargetLanguagesString is the comma delimited list of ISO 639 target languages
        /// </summary>
        /// <example>TargetLanguages = "fr, de, sq";</example>
		public string TargetLanguagesString
		{
			get {return targetLanguages;}
			set
			{
				targetLanguages = value;
				targetLanguagesPanel.Text = MainFormResources.TargetLanguages + ": " + targetLanguages;
			}
		}
        /// <summary>
        /// Source is the source of the resources
        /// </summary>
		public string Source
		{
			get {return source;}
			set
			{
				source = value;
				sourcePanel.Text = MainFormResources.Source + ": " + source;
			}
		}
        /// <summary>
        /// AutomaticTranslation specifies whether the Resource Administrator should perform automatic translation
        /// </summary>
		public bool AutomaticTranslation
		{
			get {return automaticTranslation;}
			set
			{
				automaticTranslation = value;
				if (automaticTranslation)
					automaticTranslationPanel.Text = MainFormResources.AutomaticTranslationIsOn;
				else
					automaticTranslationPanel.Text = MainFormResources.AutomaticTranslationIsOff;

				menuItemAutomaticTranslation.Checked = automaticTranslation;
			}
		}
        private string GetAppSettingsStringValue(AppSettingsReader appSettingsReader, string key)
        {
            try
            {
                return appSettingsReader.GetValue(key, typeof(System.String)).ToString();
            }
            catch (InvalidOperationException)
            {
                return String.Empty;
            }
        }
		private bool LoadConfiguration()
		{
            bool configurationLoaded = true;
            bool isResXCommandLineParameter = IsResXCommandLineParameter();
			AppSettingsReader appSettingsReader = new AppSettingsReader();

			string invariantLanguage = GetAppSettingsStringValue(appSettingsReader, "InvariantLanguage");
            if (invariantLanguage == String.Empty)
            {
                if (isResXCommandLineParameter)
                    // default to the O/S language (probably "en")
                    InvariantLanguage = Thread.CurrentThread.CurrentUICulture.Parent.TwoLetterISOLanguageName;
                else
                    configurationLoaded = false;
            }
            else
                InvariantLanguage = invariantLanguage;

            string targetLanguages = GetAppSettingsStringValue(appSettingsReader, "TargetLanguages");
			if (targetLanguages != String.Empty)
			{
				TargetLanguages.SetLanguages(targetLanguages.Split(new char[] {','}));

				StringBuilder languageList = new StringBuilder();
				for(int languageNumber = 0; languageNumber < TargetLanguages.Languages.GetLength(0); languageNumber++)
				{
					CultureInfo language = TargetLanguages.Languages[languageNumber];
					if (languageNumber < TargetLanguages.Languages.GetLength(0) - 1)
						languageList.Append(language.Name + ",");
					else
						languageList.Append(language.Name);
				}
				TargetLanguagesString = languageList.ToString();
			}
            else
                TargetLanguages.SetLanguages(new string[] {});

            if (isResXCommandLineParameter)
            {
                resourcesGovernorClassName = "ResXResourcesGovernor";
                Source = Path.GetDirectoryName(GetResXPathFromCommandLine());
                InitializeResourcesGovernor();
                LoadResources();
            }
            else
            {
                resourcesGovernorClassName = GetAppSettingsStringValue(appSettingsReader, "ResourcesGovernor");
                if (resourcesGovernorClassName == String.Empty)
                    resourcesGovernorClassName = "ResXResourcesGovernor";

                string sourceValue = GetAppSettingsStringValue(appSettingsReader, "Source");
                if (sourceValue == String.Empty)
                    configurationLoaded = false;
                else
                {
                    Source = sourceValue;
                    InitializeResourcesGovernor();
                    if (!resourcesGovernor.Exists)
                        configurationLoaded = false;
                    else
                        LoadResources();
                }
            }

			string automaticTranslation = GetAppSettingsStringValue(appSettingsReader, "AutomaticTranslation");
			if (automaticTranslation != String.Empty)
				AutomaticTranslation = (automaticTranslation == "1");

            return configurationLoaded;
		}
        private string GetResXPathFromCommandLine()
        {
            string[] args = System.Environment.GetCommandLineArgs();
            if (args.GetLength(0) > 1)
            {
                string extension = Path.GetExtension(args[1]);
                if (String.Compare(extension, ".resx", StringComparison.OrdinalIgnoreCase) == 0)
                    return args[1];
            }
            return null;
        }
        private string GetResXFilenameWithoutExtensionFromCommandLine()
        {
            string[] args = System.Environment.GetCommandLineArgs();
            if (args.GetLength(0) > 1)
            {
                string extension = Path.GetExtension(args[1]);
                if (String.Compare(extension, ".resx", StringComparison.OrdinalIgnoreCase) == 0)
                    return Path.GetFileNameWithoutExtension(args[1]);
            }
            return null;
        }
        private bool IsResXCommandLineParameter()
        {
            string[] args = System.Environment.GetCommandLineArgs();
            if (args.GetLength(0) > 1)
            {
                string extension = Path.GetExtension(args[1]);
                if (String.Compare(extension, ".resx", StringComparison.OrdinalIgnoreCase) == 0)
                    return true;
            }
            return false;
        }
        private void InitializeResourcesGovernor()
        {
            resourcesGovernor = ResourceAdministrationHelper.CreateResourcesGovernor(resourcesGovernorClassName, Source);
            setOfResourceSetsCollection = new SetOfResourceSetsCollection(resourcesGovernor);
        }
        private void AddSetting(XmlTextWriter writer, string elementName, string elementValue)
        {
            writer.WriteStartElement("add");
            writer.WriteAttributeString("key", elementName);
            writer.WriteAttributeString("value", elementValue);
            writer.WriteEndElement();
        }
        private void SaveConfiguration()
        {
            string configFileName = Application.ExecutablePath + ".config";
            XmlTextWriter writer = new XmlTextWriter(configFileName, System.Text.Encoding.Unicode);
            writer.Formatting = Formatting.Indented;
            writer.WriteStartElement("configuration");
            writer.WriteStartElement("appSettings");
            AddSetting(writer, "InvariantLanguage", invariantLanguage);
            AddSetting(writer, "Source", source);
            AddSetting(writer, "TargetLanguages", TargetLanguagesString);
            AddSetting(writer, "AutomaticTranslation", (automaticTranslation ? "1" : "0"));
            AddSetting(writer, "ResourcesGovernor", resourcesGovernorClassName);
            AddSetting(writer, "PseudoTranslationCultureInfoName", Internationalization.Common.PseudoTranslation.CultureInfo.Name);
            if (!String.IsNullOrEmpty(MicrosoftTranslator.SearchApplicationId))
                AddSetting(writer, "MicrosoftTranslatorApplicationId", MicrosoftTranslator.SearchApplicationId);

            if (!String.IsNullOrEmpty(Google2Translator.ApplicationKey))
                AddSetting(writer, "Google2TranslatorApplicationKey", Google2Translator.ApplicationKey);

            writer.WriteEndElement();
            writer.WriteEndElement();
            writer.Close();
        }
		private void LoadResources()
		{
			treeViewResources.Nodes.Clear();
			string[] baseNames = resourcesGovernor.GetInvariantCultureBaseNames();
			foreach(string baseName in baseNames)
			{
				treeViewResources.Nodes.Add(baseName);
			}
            if (treeViewResources.Nodes.Count > 0)
            {
                string resxFilenameWithoutExtension = GetResXFilenameWithoutExtensionFromCommandLine();
                if (resxFilenameWithoutExtension != null)
                {
                    foreach (TreeNode node in treeViewResources.Nodes)
                    {
                        if (String.Compare(
                            node.Text, resxFilenameWithoutExtension, StringComparison.OrdinalIgnoreCase) == 0)
                        {
                            treeViewResources.SelectedNode = node;
                            break;
                        }
                    }
                }
                else
                    treeViewResources.SelectedNode = treeViewResources.Nodes[0];
            }
		}
		private void InitializeTranslators()
		{
            string pseudoTranslationCultureInfoName = ConfigurationManager.AppSettings["PseudoTranslationCultureInfoName"];
            if (!String.IsNullOrEmpty(pseudoTranslationCultureInfoName))
                Internationalization.Common.PseudoTranslation.CultureInfo = new CultureInfo(pseudoTranslationCultureInfoName);

			allTranslators = new TranslatorCollection();
			allTranslators.Add(new PseudoTranslator());

            bool microsoftOrGoogleTranslatorAdded = false;

            string microsoftTranslatorApplicationId = ConfigurationManager.AppSettings["MicrosoftTranslatorApplicationId"];
            if (!String.IsNullOrEmpty(microsoftTranslatorApplicationId))
            {
                // there is an AppSetting for MicrosoftTranslatorApplicationId so add it as a translator
                MicrosoftTranslator.SearchApplicationId = microsoftTranslatorApplicationId;
                try
                {
                    allTranslators.Add(new MicrosoftTranslator());
                    microsoftOrGoogleTranslatorAdded = true;
                }
                catch
                {
                    // the most likely reason for this is that there is no internet connection
                }
            }

            // Google has deprecated their free translator API as of December 2011.
            // allTranslators.Add(new GoogleTranslator());

            string google2TranslatorApplicationKey = ConfigurationManager.AppSettings["Google2TranslatorApplicationKey"];
            if (!String.IsNullOrEmpty(google2TranslatorApplicationKey))
            {
                // there is an AppSetting for Google2TranslatorApplicationKey so add it as a translator
                Google2Translator.ApplicationKey = google2TranslatorApplicationKey;
                try
                {
                    allTranslators.Add(new Google2Translator());
                    microsoftOrGoogleTranslatorAdded = true;
                }
                catch
                {
                    // the most likely reason for this is that there is no internet connection
                }
            }

            ITranslator closerFarTranslator = new CloserFarTranslator();
            closerFarTranslator.Enabled = !microsoftOrGoogleTranslatorAdded;
            allTranslators.Add(closerFarTranslator);

            ITranslator altaVistaTranslator = new AltaVistaTranslator();
            altaVistaTranslator.Enabled = !microsoftOrGoogleTranslatorAdded;
            allTranslators.Add(altaVistaTranslator);

            ITranslator freeTranslationTranslator = new FreeTranslationTranslator();
            freeTranslationTranslator.Enabled = !microsoftOrGoogleTranslatorAdded;
			allTranslators.Add(freeTranslationTranslator);
            
            ITranslator webServiceXTranslator = new WebServiceXTranslator();
            webServiceXTranslator.Enabled = false;
            allTranslators.Add(webServiceXTranslator);

            try
			{
				allTranslators.Add(new Office2003ResearchServicesTranslator());
			}
			catch (TranslationException)
			{
				// Ignore this exception. This happens when running this program on a machine which
				// does not have Office 2003 installed on it or the Translation service has not been installed yet.
				// The consequence of this is that we simply don't add the Office 2003 Research Services Translator.
			}
            // Add the RandomTranslator but ensure that it is not enabled because
            // this is a translator that should only be used if the user specifically selects it.
            ITranslator randomTranslator = new RandomTranslator();
            randomTranslator.Enabled = false;
            allTranslators.Add(randomTranslator);
        }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(MainForm));
            this.statusBar = new System.Windows.Forms.StatusBar();
            this.groupBoxResource = new System.Windows.Forms.GroupBox();
            this.resourceEditorControl = new Internationalization.ResourceAdministration.ResourceEditorControl();
            this.splitterTargetLanguagesStatus = new System.Windows.Forms.Splitter();
            this.groupBoxTargetLanguagesStatus = new System.Windows.Forms.GroupBox();
            this.listBoxTargetLanguagesStatus = new System.Windows.Forms.ListBox();
            this.splitter1 = new System.Windows.Forms.Splitter();
            this.groupBox3 = new System.Windows.Forms.GroupBox();
            this.treeViewResources = new System.Windows.Forms.TreeView();
            this.mainMenu = new System.Windows.Forms.MainMenu(this.components);
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.menuItemExport = new System.Windows.Forms.MenuItem();
            this.menuItemExit = new System.Windows.Forms.MenuItem();
            this.menuItem3 = new System.Windows.Forms.MenuItem();
            this.menuItemResourcesSource = new System.Windows.Forms.MenuItem();
            this.menuItemLanguages = new System.Windows.Forms.MenuItem();
            this.menuItemTranslators = new System.Windows.Forms.MenuItem();
            this.menuItem6 = new System.Windows.Forms.MenuItem();
            this.menuItemIntegrityCheck = new System.Windows.Forms.MenuItem();
            this.menuItem2 = new System.Windows.Forms.MenuItem();
            this.menuItemAutomaticTranslation = new System.Windows.Forms.MenuItem();
            this.contextMenuResources = new System.Windows.Forms.ContextMenu();
            this.menuItemCreateMissingResources = new System.Windows.Forms.MenuItem();
            this.menuItemAddMissingResourceEntries = new System.Windows.Forms.MenuItem();
            this.groupBoxResource.SuspendLayout();
            this.groupBoxTargetLanguagesStatus.SuspendLayout();
            this.groupBox3.SuspendLayout();
            this.SuspendLayout();
            // 
            // statusBar
            // 
            resources.ApplyResources(this.statusBar, "statusBar");
            this.statusBar.Name = "statusBar";
            this.statusBar.ShowPanels = true;
            // 
            // groupBoxResource
            // 
            this.groupBoxResource.Controls.Add(this.resourceEditorControl);
            this.groupBoxResource.Controls.Add(this.splitterTargetLanguagesStatus);
            this.groupBoxResource.Controls.Add(this.groupBoxTargetLanguagesStatus);
            resources.ApplyResources(this.groupBoxResource, "groupBoxResource");
            this.groupBoxResource.Name = "groupBoxResource";
            this.groupBoxResource.TabStop = false;
            // 
            // resourceEditorControl
            // 
            this.resourceEditorControl.AllowComments = false;
            resources.ApplyResources(this.resourceEditorControl, "resourceEditorControl");
            this.resourceEditorControl.Name = "resourceEditorControl";
            this.resourceEditorControl.ReadOnly = false;
            this.resourceEditorControl.ResourceSet = null;
            this.resourceEditorControl.UseFileRefs = false;
            // 
            // splitterTargetLanguagesStatus
            // 
            resources.ApplyResources(this.splitterTargetLanguagesStatus, "splitterTargetLanguagesStatus");
            this.splitterTargetLanguagesStatus.Name = "splitterTargetLanguagesStatus";
            this.splitterTargetLanguagesStatus.TabStop = false;
            // 
            // groupBoxTargetLanguagesStatus
            // 
            this.groupBoxTargetLanguagesStatus.Controls.Add(this.listBoxTargetLanguagesStatus);
            resources.ApplyResources(this.groupBoxTargetLanguagesStatus, "groupBoxTargetLanguagesStatus");
            this.groupBoxTargetLanguagesStatus.Name = "groupBoxTargetLanguagesStatus";
            this.groupBoxTargetLanguagesStatus.TabStop = false;
            // 
            // listBoxTargetLanguagesStatus
            // 
            resources.ApplyResources(this.listBoxTargetLanguagesStatus, "listBoxTargetLanguagesStatus");
            this.listBoxTargetLanguagesStatus.FormattingEnabled = true;
            this.listBoxTargetLanguagesStatus.Name = "listBoxTargetLanguagesStatus";
            // 
            // splitter1
            // 
            resources.ApplyResources(this.splitter1, "splitter1");
            this.splitter1.Name = "splitter1";
            this.splitter1.TabStop = false;
            // 
            // groupBox3
            // 
            this.groupBox3.Controls.Add(this.treeViewResources);
            resources.ApplyResources(this.groupBox3, "groupBox3");
            this.groupBox3.Name = "groupBox3";
            this.groupBox3.TabStop = false;
            // 
            // treeViewResources
            // 
            resources.ApplyResources(this.treeViewResources, "treeViewResources");
            this.treeViewResources.Name = "treeViewResources";
            this.treeViewResources.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.treeViewResources_AfterSelect);
            // 
            // mainMenu
            // 
            this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem1,
            this.menuItem3});
            // 
            // menuItem1
            // 
            this.menuItem1.Index = 0;
            this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItemExport,
            this.menuItemExit});
            resources.ApplyResources(this.menuItem1, "menuItem1");
            // 
            // menuItemExport
            // 
            this.menuItemExport.Index = 0;
            resources.ApplyResources(this.menuItemExport, "menuItemExport");
            this.menuItemExport.Click += new System.EventHandler(this.menuItemExport_Click);
            // 
            // menuItemExit
            // 
            this.menuItemExit.Index = 1;
            resources.ApplyResources(this.menuItemExit, "menuItemExit");
            this.menuItemExit.Click += new System.EventHandler(this.menuItemExit_Click);
            // 
            // menuItem3
            // 
            this.menuItem3.Index = 1;
            this.menuItem3.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItemResourcesSource,
            this.menuItemLanguages,
            this.menuItemTranslators,
            this.menuItem6,
            this.menuItemIntegrityCheck,
            this.menuItem2,
            this.menuItemAutomaticTranslation});
            resources.ApplyResources(this.menuItem3, "menuItem3");
            // 
            // menuItemResourcesSource
            // 
            this.menuItemResourcesSource.Index = 0;
            resources.ApplyResources(this.menuItemResourcesSource, "menuItemResourcesSource");
            this.menuItemResourcesSource.Click += new System.EventHandler(this.menuItemResourcesSource_Click);
            // 
            // menuItemLanguages
            // 
            this.menuItemLanguages.Index = 1;
            resources.ApplyResources(this.menuItemLanguages, "menuItemLanguages");
            this.menuItemLanguages.Click += new System.EventHandler(this.menuItemLanguages_Click);
            // 
            // menuItemTranslators
            // 
            this.menuItemTranslators.Index = 2;
            resources.ApplyResources(this.menuItemTranslators, "menuItemTranslators");
            this.menuItemTranslators.Click += new System.EventHandler(this.menuItemTranslators_Click);
            // 
            // menuItem6
            // 
            this.menuItem6.Index = 3;
            resources.ApplyResources(this.menuItem6, "menuItem6");
            // 
            // menuItemIntegrityCheck
            // 
            this.menuItemIntegrityCheck.Index = 4;
            resources.ApplyResources(this.menuItemIntegrityCheck, "menuItemIntegrityCheck");
            this.menuItemIntegrityCheck.Click += new System.EventHandler(this.menuItemIntegrityCheck_Click);
            // 
            // menuItem2
            // 
            this.menuItem2.Index = 5;
            resources.ApplyResources(this.menuItem2, "menuItem2");
            // 
            // menuItemAutomaticTranslation
            // 
            this.menuItemAutomaticTranslation.Index = 6;
            resources.ApplyResources(this.menuItemAutomaticTranslation, "menuItemAutomaticTranslation");
            this.menuItemAutomaticTranslation.Click += new System.EventHandler(this.menuItemAutomaticTranslation_Click);
            // 
            // contextMenuResources
            // 
            this.contextMenuResources.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItemCreateMissingResources,
            this.menuItemAddMissingResourceEntries});
            // 
            // menuItemCreateMissingResources
            // 
            this.menuItemCreateMissingResources.Index = 0;
            resources.ApplyResources(this.menuItemCreateMissingResources, "menuItemCreateMissingResources");
            this.menuItemCreateMissingResources.Click += new System.EventHandler(this.menuItemCreateMissingResources_Click);
            // 
            // menuItemAddMissingResourceEntries
            // 
            this.menuItemAddMissingResourceEntries.Index = 1;
            resources.ApplyResources(this.menuItemAddMissingResourceEntries, "menuItemAddMissingResourceEntries");
            this.menuItemAddMissingResourceEntries.Click += new System.EventHandler(this.menuItemAddMissingResourceEntries_Click);
            // 
            // MainForm
            // 
            resources.ApplyResources(this, "$this");
            this.ContextMenu = this.contextMenuResources;
            this.Controls.Add(this.groupBoxResource);
            this.Controls.Add(this.splitter1);
            this.Controls.Add(this.groupBox3);
            this.Controls.Add(this.statusBar);
            this.Menu = this.mainMenu;
            this.Name = "MainForm";
            this.groupBoxResource.ResumeLayout(false);
            this.groupBoxTargetLanguagesStatus.ResumeLayout(false);
            this.groupBox3.ResumeLayout(false);
            this.ResumeLayout(false);

        }
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
            ResourcesGovernors.RegisterResourcesGovernorType("ResXResourcesGovernor", typeof(ResXResourcesGovernor));
            ResourcesGovernors.RegisterResourcesGovernorType(
                "ResourcesResourcesGovernor", typeof(ResourcesResourcesGovernor));
            ResourcesGovernors.RegisterResourcesGovernorType("DbResourcesGovernor", typeof(DbResourcesGovernor));

			Application.Run(new MainForm());
		}

		private void treeViewResources_AfterSelect(object sender, System.Windows.Forms.TreeViewEventArgs e)
		{
			string baseName = treeViewResources.SelectedNode.Text;
			SetOfResourceSets setOfResourceSets = setOfResourceSetsCollection.GetSetOfResourceSets(baseName);

			IResourceGovernor resourceGovernor = setOfResourceSets.GetResourceGovernor(CultureInfo.InvariantCulture);
			ResourceSet resourceSet = setOfResourceSets.GetResourceSet(resourceGovernor);
            resourceEditorControl.ResourceSet = resourceSet;
            resourceEditorControl.AllowComments = resourcesGovernor.SupportsComments;
            resourceEditorControl.UseFileRefs = resourcesGovernor.SupportsFileReferences;
            resourceEditorControl.ReadOnly = resourceGovernor.ResourceIsReadOnly();

			if (resourceGovernor.ResourceIsReadOnly())
				groupBoxResource.Text = baseName + " " + MainFormResources.ReadOnly;
			else
				groupBoxResource.Text = baseName;

			UpdateTargetLanguagesStatus(setOfResourceSets, baseName);
		}
		private void UpdateTargetLanguagesStatus()
		{
			string baseName = treeViewResources.SelectedNode.Text;
			SetOfResourceSets setOfResourceSets = setOfResourceSetsCollection.GetSetOfResourceSets(baseName);
			UpdateTargetLanguagesStatus(setOfResourceSets, baseName);
		}
		private void UpdateTargetLanguagesStatus(SetOfResourceSets setOfResourceSets, string baseName)
		{
			bool oneOrMoreResourcesDoNotExist = false;
			bool problems = false;
			listBoxTargetLanguagesStatus.Items.Clear();
			int languageNumber = 0;
			foreach(CultureInfo cultureInfo in TargetLanguages.Languages)
			{
				string resourceName = baseName + "." + cultureInfo.Name;

				IResourceGovernor targetResourceGovernor = setOfResourceSets.GetResourceGovernor(cultureInfo);
				if (targetResourceGovernor.ResourceExists())
				{
					if (targetResourceGovernor.ResourceIsReadOnly())
					{
						listBoxTargetLanguagesStatus.Items.Add(String.Format(MainFormResources.ResourceIsReadOnly, resourceName));
						problems = true;
					}
					else if (automaticTranslation)
					{
						// we have to do automatic translation so check that there is a translator for this language
						ITranslator translator = (ITranslator) targetLanguageTranslators[languageNumber];
						if (translator == null)
						{
                            listBoxTargetLanguagesStatus.Items.Add(String.Format(MainFormResources.ResourceHasNoTranslator, resourceName));
							problems = true;
						}
						else
                            listBoxTargetLanguagesStatus.Items.Add(String.Format(MainFormResources.ResourceIsOk, resourceName));
					}
					else
                        listBoxTargetLanguagesStatus.Items.Add(String.Format(MainFormResources.ResourceIsOk, resourceName));
				}
				else
				{
                    listBoxTargetLanguagesStatus.Items.Add(String.Format(MainFormResources.ResourceDoesNotExist, resourceName));
					problems = true;
					oneOrMoreResourcesDoNotExist = true;
				}
				languageNumber++;
			}
			if (problems)
			{
				splitterTargetLanguagesStatus.Visible = true;
				groupBoxTargetLanguagesStatus.Visible = true;
			}
			else
			{
				splitterTargetLanguagesStatus.Visible = false;
				groupBoxTargetLanguagesStatus.Visible = false;
			}
			menuItemCreateMissingResources.Enabled = oneOrMoreResourcesDoNotExist;
		}

        private void resourceEditorControlResourceSetChanging(object sender, ResourceSetChangeEventArgs eventArgs)
        {
			string baseName = treeViewResources.SelectedNode.Text;
			SetOfResourceSets setOfResourceSets = (SetOfResourceSets) setOfResourceSetsCollection[baseName];
			IResourceGovernor resourceGovernor = setOfResourceSets.GetResourceGovernor(CultureInfo.InvariantCulture);
			ResourceSet resourceSet = setOfResourceSets.GetResourceSet(resourceGovernor);

            // write the change to the invariant culture
            resourceGovernor.WriteResourceSet(resourceSet);

			// apply the same change to all of the target language resources
			foreach(CultureInfo cultureInfo in TargetLanguages.Languages)
			{
				IResourceGovernor targetResourceGovernor = setOfResourceSets.GetResourceGovernor(cultureInfo);
				if (targetResourceGovernor.ResourceExists() && ! targetResourceGovernor.ResourceIsReadOnly())
				{
					// the target language resource exists and is not read only so apply the change
					resourceSet = setOfResourceSets.GetResourceSet(targetResourceGovernor);
					ApplyChangeToResource(targetResourceGovernor, resourceSet, eventArgs, baseName, setOfResourceSets);
				}
			}
		}
        private bool IsTranslatable(string resourceKey, object resourceValue)
        {
            return (ResourceUtilities.IsLocalizable(resourceKey, resourceValue) && resourceValue != null);
        }
		private void ApplyChangeToResource(IResourceGovernor resourceGovernor, 
			ResourceSet resourceSet, ResourceSetChangeEventArgs eventArgs, string baseName, SetOfResourceSets setOfResourceSets)
		{
			string inputLanguage = invariantLanguage;
			Hashtable resourceTable = ResourceUtilities.GetResourceSetTable(resourceSet);
			switch (eventArgs.Action)
			{
				case ResourceSetAction.Add:
				{
					object resourceValue = eventArgs.ResourceValue;
					if (automaticTranslation && IsTranslatable(eventArgs.ResourceKey, eventArgs.ResourceValue))
						// translate the input string to the target language
						resourceValue = Translate(
							inputLanguage, resourceGovernor.CultureInfo, resourceValue.ToString(), baseName, setOfResourceSets);

					resourceTable.Add(eventArgs.ResourceKey, resourceValue);
					resourceGovernor.WriteResourceSet(resourceSet);
					break;
				}
				case ResourceSetAction.Delete:
				{
					if (resourceTable.ContainsKey(eventArgs.ResourceKey))
					{
						resourceTable.Remove(eventArgs.ResourceKey);
						resourceGovernor.WriteResourceSet(resourceSet);
					}
					break;
				}
				case ResourceSetAction.Change:
				{
                    string oldName;
                    object newResourceValue;
                    if (resourceTable.ContainsKey(eventArgs.OldResourceKey))
                    {
                        oldName = eventArgs.OldResourceKey;
                        newResourceValue = resourceTable[eventArgs.OldResourceKey];
                    }
                    else
                    {
                        // this is an edit of an entry which does not exist in this resource set
                        oldName = eventArgs.ResourceKey;
                        newResourceValue = eventArgs.ResourceValue;
                    }

                    if (IsTranslatable(eventArgs.ResourceKey, eventArgs.ResourceValue))
                    {
                        bool translationRequired;
                        string newValueString = eventArgs.ResourceValue.ToString();
                        string oldValueString;
                        if (resourceTable.ContainsKey(eventArgs.OldResourceKey))
                        {
                            oldValueString = eventArgs.OldResourceValue.ToString();
                            translationRequired = oldValueString != newValueString;
                        }
                        else
                        {
                            // this is an edit of an entry which does not exist in this resource set yet
                            oldValueString = eventArgs.ResourceValue.ToString();
                            translationRequired = true;
                        }

                        if (automaticTranslation && translationRequired)
                        {
                            // the value has been changed and the target language is not the invariant culture
                            // (so the string must be translated)
                            newResourceValue = Translate(
                                inputLanguage, resourceGovernor.CultureInfo, newValueString, baseName, setOfResourceSets);
                        }
                    }

					if (oldName != eventArgs.ResourceKey)
					{
						// remove the old key and add the new key
						resourceTable.Remove(oldName);
						resourceTable.Add(eventArgs.ResourceKey, newResourceValue);
					}
					else
					{
						// the name hasn't changed but the value has
						if (resourceTable.ContainsKey(eventArgs.ResourceKey))
							resourceTable[eventArgs.ResourceKey] = newResourceValue;
						else
							// the key didn't already exist in the target language file
							resourceTable.Add(eventArgs.ResourceKey, newResourceValue);
					}
					resourceGovernor.WriteResourceSet(resourceSet);
					break;
				}
			}
		}
        /// <summary>
        /// Translate translates a single resource string
        /// </summary>
        /// <param name="inputLanguage">The language of the resource string</param>
        /// <param name="targetCultureInfo">The CultureInfo of the target language</param>
        /// <param name="text">The string to translate</param>
        /// <returns>The translated string</returns>
        public string Translate(string inputLanguage, CultureInfo targetCultureInfo, string text)
		{
			return Translate(inputLanguage, targetCultureInfo, text, null, null);
		}
        /// <summary>
        /// Translate translates a single resource string
        /// </summary>
        /// <param name="inputLanguage">The language of the resource string</param>
        /// <param name="targetCultureInfo">The CultureInfo of the target language</param>
        /// <param name="text">The string to translate</param>
        /// <param name="baseName">The base name of the resource</param>
        /// <param name="setOfResourceSets">The SetOfResourceSets containing the ResourceSets</param>
        /// <returns>The translated string</returns>
        private string Translate(
			string inputLanguage, CultureInfo targetCultureInfo, string text, string baseName, SetOfResourceSets setOfResourceSets)
		{
			// get the translator used to translate this language pair
			ITranslator translator = null;
			int languageNumber = -1;
			for(languageNumber = 0; languageNumber < TargetLanguages.Languages.GetLength(0); languageNumber++)
			{
				if (targetCultureInfo == TargetLanguages.Languages[languageNumber])
				{
					translator = (ITranslator) targetLanguageTranslators[languageNumber];
					break;
				}
			}
			// translate the text using the translator which was found
			while (translator != null)
			{
				try
				{
					text = translator.Translate(inputLanguage, targetCultureInfo.Name, text);
					break;
				}
				catch (TranslationException)
				{
					string oldTranslatorName = translator.Name;
					// this translator has failed
					translator.Enabled = false;
					// look for a new translator which offers the same facility
					translator = allTranslators.GetEnabledTranslator(inputLanguage, targetCultureInfo.Name);
					// update our translator list with the new (or null) translator
					targetLanguageTranslators[languageNumber] = translator;
					// tell the user what happened
					if (translator == null)
					{
						MessageBox.Show(String.Format(MainFormResources.TranslatorFailedNoReplacement, 
							oldTranslatorName, inputLanguage, targetCultureInfo.Name));

						if (setOfResourceSets != null && baseName != null)
							UpdateTargetLanguagesStatus(setOfResourceSets, baseName);
					}
					else
						MessageBox.Show(String.Format(MainFormResources.TranslatorFailedReplacedBy, oldTranslatorName, translator.Name));
				}
			}
			return text;
		}
		private void AddResourceSet(SetOfResourceSets setOfResourceSets, string baseName, CultureInfo cultureInfo)
		{
			IResourceGovernor resourceGovernor = resourcesGovernor.GetResourceGovernor(baseName, cultureInfo);
			if (resourceGovernor.ResourceExists())
				setOfResourceSets.AddResourceSet(resourceGovernor);
			else
				setOfResourceSets.AddResourceSet(resourceGovernor, null);
		}

		private void SetTranslators()
		{
			TranslatorsForm form = new TranslatorsForm(allTranslators);
			form.ShowDialog();
			// The Enabled status of the translators might have changed. A translator which
			// was previously enabled might have been disabled and therefore should not be used.
			ResourceAdministrationHelper.RefreshTargetLanguageTranslators(invariantLanguage, allTranslators, ref targetLanguageTranslators);

			// update the target languages' status
			string baseName = treeViewResources.SelectedNode.Text;
			SetOfResourceSets setOfResourceSets = (SetOfResourceSets) setOfResourceSetsCollection[baseName];
			UpdateTargetLanguagesStatus(setOfResourceSets, baseName);
            SaveConfiguration();
		}

		private void SetLanguages()
		{
			LanguagesForm form = new LanguagesForm(invariantLanguage);
			if (form.ShowDialog() == DialogResult.OK)
			{
				CultureInfo[] selectedLanguages = NeutralDisplayNamesToCultureInfos(form.SelectedLanguages);

				// compare the selected list of languages against the old list of languages
				// and get a list of newly added languages
				ArrayList addedLanguages = new ArrayList();
				foreach(CultureInfo cultureInfo in selectedLanguages)
				{
					if (Array.IndexOf(TargetLanguages.Languages, cultureInfo) == -1)
						addedLanguages.Add(cultureInfo);
				}

				// compare the list of target languages against the list of selected languages
				// and get a list of deletes languages
				ArrayList deletedLanguages = new ArrayList();
				foreach(CultureInfo cultureInfo in TargetLanguages.Languages)
				{
					if (Array.IndexOf(selectedLanguages, cultureInfo) == -1)
						// the target language is no longer in the list of selected languages
						deletedLanguages.Add(cultureInfo);
				}

				if (deletedLanguages.Count > 0)
				{
					string deletedLanguageList = LanguagesToList(deletedLanguages);
					MessageBox.Show(
						MainFormResources.YouHaveRemovedSomeLanguages + System.Environment.NewLine +
						MainFormResources.TheResourcesHaveNotBeenDeleted + System.Environment.NewLine +
						MainFormResources.YouMustDeleteTheResourcesYourself + System.Environment.NewLine + System.Environment.NewLine +
                        deletedLanguageList, MainFormResources.TargetLanguagesDeleted);
				}
				bool createResources = false;
				if (addedLanguages.Count > 0)
				{
					string addedLanguageList = LanguagesToList(addedLanguages);
					createResources = (MessageBox.Show(
						MainFormResources.YouHaveAddedSomeLanguages + 
						System.Environment.NewLine + System.Environment.NewLine + addedLanguageList +
						System.Environment.NewLine + System.Environment.NewLine +
						MainFormResources.CreateCorrespondingResources, MainFormResources.TargetLanguagesAdded, 
						MessageBoxButtons.YesNo) == DialogResult.Yes);
				}

				if (deletedLanguages.Count > 0 || addedLanguages.Count > 0)
				{
					// the list of target languages has changed
					try
					{
						Cursor.Current = Cursors.WaitCursor;
						// update the list of target languages
						string[] selectedLanguagesNames = new string[selectedLanguages.GetLength(0)];
						int selectedLanguageNumber = 0;
						string selectedLanguagesString = String.Empty;
						foreach(CultureInfo selectedLanguage in selectedLanguages)
						{
							if (selectedLanguageNumber == 0)
								selectedLanguagesString = selectedLanguage.Name;
							else
								selectedLanguagesString = selectedLanguagesString + "," + selectedLanguage.Name;

							selectedLanguagesNames[selectedLanguageNumber++] = selectedLanguage.Name;
						}
						TargetLanguages.SetLanguages(selectedLanguagesNames);
						TargetLanguagesString = selectedLanguagesString;
						targetLanguageTranslators = null;
						ResourceAdministrationHelper.RefreshTargetLanguageTranslators(invariantLanguage, allTranslators, ref targetLanguageTranslators);
						setOfResourceSetsCollection = new SetOfResourceSetsCollection(resourcesGovernor);
						if (createResources)
							CreateResources(addedLanguages);
						UpdateTargetLanguagesStatus();
					}
					finally
					{
						Cursor.Current = Cursors.Default;
					}

					if (createResources)
					{
						if (resourcesGovernor is ResXResourcesGovernor)
							MessageBox.Show(MainFormResources.ResxFilesCreated, MainFormResources.NewResxFilesAdded);
						else
							MessageBox.Show(MainFormResources.NewResourcesCreated, MainFormResources.NewResourcesAdded);
					}
					else
						MessageBox.Show(MainFormResources.NoResourcesAddedDeleted, MainFormResources.ProcessComplete);
				}
			}
		}
		private void CreateResources(ArrayList addedLanguages)
		{
			foreach(TreeNode node in treeViewResources.Nodes)
			{
				string baseName = node.Text;
				SetOfResourceSets setOfResourceSets = setOfResourceSetsCollection.GetSetOfResourceSets(baseName);
				ResourceSet invariantResourceSet = setOfResourceSets.GetResourceSet(CultureInfo.InvariantCulture);
				foreach(CultureInfo addedLanguage in addedLanguages)
				{
					IResourceGovernor resourceGovernor = setOfResourceSets.GetResourceGovernor(addedLanguage);
					if (! resourceGovernor.ResourceExists())
					{
						// there is no existing resource for this language
                        Hashtable invariantResourceTable =
                            ResourceUtilities.GetResourceSetTable(invariantResourceSet);

                        if (ResourceAdministrationHelper.IsWindowsForm(invariantResourceTable))
                        {
                            ResourceSet addedLanguageResourceSet =
                                ResourceAdministrationHelper.CreateCultureNeutralWindowsFormResourceSet(
                                invariantResourceTable);

                            resourceGovernor.WriteResourceSet(addedLanguageResourceSet);
                        }
                        else
                            resourceGovernor.WriteResourceSet(invariantResourceSet);

                        if (automaticTranslation)
							TranslateResourceSet(resourceGovernor);
					}
				}
			}
		}
        /// <summary>
        /// TranslateResourceSet translates a resource set
        /// </summary>
        /// <param name="resourceGovernor">The IResourceGovernor of the resource set</param>
        public void TranslateResourceSet(IResourceGovernor resourceGovernor)
		{
			ResourceSet resourceSet = resourceGovernor.ReadResourceSet();
            Hashtable table = ResourceUtilities.GetResourceSetTable(resourceSet);
			bool translationMade = false;
			ArrayList dictionaryEntries = new ArrayList();
			foreach(DictionaryEntry dictionaryEntry in table)
			{
				dictionaryEntries.Add(dictionaryEntry);
			}
			foreach(DictionaryEntry dictionaryEntry in dictionaryEntries)
			{
                ResourcesDataNodeInterpreter dataNodeInterpreter = new ResourcesDataNodeInterpreter(
                    dictionaryEntry.Key.ToString(), dictionaryEntry.Value);
				if (ResourceUtilities.IsLocalizable(dataNodeInterpreter.Name, dataNodeInterpreter.Value) &&
                    dataNodeInterpreter.Value != null && 
					dataNodeInterpreter.Value is string && dataNodeInterpreter.Value.ToString().Length > 0)
				{
                    string translatedValue = Translate(invariantLanguage,
                        resourceGovernor.CultureInfo, dataNodeInterpreter.Value.ToString(), null, null);

                    dataNodeInterpreter.Value = translatedValue;

                    table[dictionaryEntry.Key] = dataNodeInterpreter.DataNode;

					translationMade = true;
				}
			}
			if (translationMade)
				// write the changes back
				resourceGovernor.WriteResourceSet(resourceSet);
		}
		private string LanguagesToList(ArrayList languages)
		{
			if (languages.Count == 0)
				return String.Empty;
			else
			{
				StringBuilder stringBuilder = new StringBuilder();
				// add the first language
				stringBuilder.Append(((CultureInfo) languages[0]).DisplayName);
				// add the remaining languages
				for(int languageNumber = 1; languageNumber < languages.Count; languageNumber++)
				{
					stringBuilder.Append(System.Environment.NewLine + ((CultureInfo) languages[languageNumber]).DisplayName);
				}
				return stringBuilder.ToString();
			}
		}
		private CultureInfo[] NeutralDisplayNamesToCultureInfos(StringCollection displayNames)
		{
			CultureInfo[] cultureInfos = new CultureInfo[displayNames.Count];
			int cultureInfoNumber = 0;
			foreach(string displayName in displayNames)
			{
				cultureInfos[cultureInfoNumber++] = NeutralDisplayNameToCultureInfo(displayName);
			}
			return cultureInfos;
		}
		private CultureInfo NeutralDisplayNameToCultureInfo(string displayName)
		{
			foreach(CultureInfo cultureInfo in CultureInfo.GetCultures(CultureTypes.NeutralCultures))
			{
				if (cultureInfo.DisplayName == displayName)
					return cultureInfo;
			}
			return null;
		}

		private void DoIntegrityCheck()
		{
			if (MessageBox.Show(
				MainFormResources.PerformIntegrityCheck, MainFormResources.IntegrityCheck, MessageBoxButtons.YesNo) == DialogResult.Yes)
			{
				string[] baseNames = new string[treeViewResources.Nodes.Count];
				int baseNameNumber = 0;
				foreach(TreeNode node in treeViewResources.Nodes)
				{
					baseNames[baseNameNumber++] = node.Text;
				}

				integrityCheckForm = new IntegrityCheckForm(setOfResourceSetsCollection, baseNames, 
					new IntegrityCheckInformationEventHandler(IntegrityCheckInformation), this, 
					automaticTranslation, invariantLanguage);

				integrityCheckForm.Show();
				IntegrityCheck.Check(setOfResourceSetsCollection, baseNames, new IntegrityCheckInformationEventHandler(IntegrityCheckInformation));
				if (integrityCheckForm.InformationCount > 0)
				{
					integrityCheckForm.ShowButtons();
					integrityCheckForm.Hide();
					integrityCheckForm.ShowDialog();
					UpdateTargetLanguagesStatus();
				}
				else
				{
					integrityCheckForm.Close();
					MessageBox.Show(MainFormResources.IntegrityCheckCompleteNoErrors, MainFormResources.IntegrityCheck);
				}
			}
		}
		private void IntegrityCheckInformation(object sender, IntegrityCheckInformationEventArgs eventArgs)
		{
			integrityCheckForm.AddInformation(eventArgs);
		}

		private void menuItemLanguages_Click(object sender, System.EventArgs e)
		{
			SetLanguages();
		}

		private void menuItemTranslators_Click(object sender, System.EventArgs e)
		{
			SetTranslators();
		}

		private void menuItemIntegrityCheck_Click(object sender, System.EventArgs e)
		{
			DoIntegrityCheck();
		}

		private void menuItemAutomaticTranslation_Click(object sender, System.EventArgs e)
		{
			AutomaticTranslation = ! AutomaticTranslation;
            SaveConfiguration();
		}

		private void menuItemCreateMissingResources_Click(object sender, System.EventArgs e)
		{
			string baseName = treeViewResources.SelectedNode.Text;
			SetOfResourceSets setOfResourceSets = setOfResourceSetsCollection.GetSetOfResourceSets(baseName);
			CreateMissingResourcesForm form = new CreateMissingResourcesForm();
			form.MissingResources = GetMissingResources(setOfResourceSets, baseName);
			if (form.ShowDialog() == DialogResult.OK)
			{
				CultureInfo[] checkedMissingResources = form.CheckedMissingResources;
				if (checkedMissingResources.GetLength(0) > 0)
				{
					// the user selected one or more languages to create resources for
					CreateMissingResources(setOfResourceSets, checkedMissingResources);
					UpdateTargetLanguagesStatus();
					MessageBox.Show(MainFormResources.MissingResourcesCreated);
				}
			}		
		}
		private void CreateMissingResources(SetOfResourceSets setOfResourceSets, CultureInfo[] checkedMissingResources)
		{
			ResourceSet invariantResourceSet = setOfResourceSets.GetResourceSet(CultureInfo.InvariantCulture);
            Hashtable invariantResourceTable = ResourceUtilities.GetResourceSetTable(invariantResourceSet);

			foreach(CultureInfo missingResource in checkedMissingResources)
			{
				IResourceGovernor resourceGovernor = setOfResourceSets.GetResourceGovernor(missingResource);
				if (ResourceAdministrationHelper.IsWindowsForm(invariantResourceTable))
					// write only those invariant resource entries which should have a translation
					resourceGovernor.WriteResourceSet(
						ResourceAdministrationHelper.CreateCultureNeutralWindowsFormResourceSet(invariantResourceTable));
				else
					// write a copy of the invariant resources to the target language
					resourceGovernor.WriteResourceSet(invariantResourceSet);

				if (automaticTranslation)
					TranslateResourceSet(resourceGovernor);
			}
		}
		private CultureInfo[] GetMissingResources(SetOfResourceSets setOfResourceSets, string baseName)
		{
			ArrayList missingResources = new ArrayList();
			foreach(CultureInfo cultureInfo in TargetLanguages.Languages)
			{
				IResourceGovernor targetResourceGovernor = setOfResourceSets.GetResourceGovernor(cultureInfo);
				if (! targetResourceGovernor.ResourceExists())
					missingResources.Add(cultureInfo);
			}
			return (CultureInfo[]) missingResources.ToArray(typeof(CultureInfo));
		}

		private void menuItemAddMissingResourceEntries_Click(object sender, System.EventArgs e)
		{
			string baseName = treeViewResources.SelectedNode.Text;
			SetOfResourceSets setOfResourceSets = setOfResourceSetsCollection.GetSetOfResourceSets(baseName);
			AddMissingResourceEntriesForm form = new AddMissingResourceEntriesForm();
			form.Resources = TargetLanguages.Languages;
			if (form.ShowDialog() == DialogResult.OK)
			{
				CultureInfo[] checkedResources = form.CheckedResources;
				if (checkedResources.GetLength(0) > 0)
				{
					// the user selected one or more languages to add resources for
					int resourcesAdded = AddResources(setOfResourceSets, checkedResources);
					if (resourcesAdded == 0)
						MessageBox.Show(MainFormResources.AllResourcesUpToDate);
					else
						MessageBox.Show(String.Format(MainFormResources.NewResourceEntriesAdded, resourcesAdded));
				}
			}		
		}
		private int AddResources(SetOfResourceSets setOfResourceSets, CultureInfo[] checkedResources)
		{
			resourceEntriesAdded = 0;
			ResourceSet invariantResourceSet = setOfResourceSets.GetResourceSet(CultureInfo.InvariantCulture);
			foreach(CultureInfo resource in checkedResources)
			{
                if (resource != null)
                {
                    IResourceGovernor resourceGovernor = setOfResourceSets.GetResourceGovernor(resource);
                    if (resourceGovernor.ResourceExists())
                    {
                        IntegrityCheck.CompareResources(invariantResourceSet, resourceGovernor,
                            new IntegrityCheckInformationEventHandler(AddResourcesInformation), true, automaticTranslation,
                            this, invariantLanguage);
                    }
                }
			}
			return resourceEntriesAdded;
		}
		private void AddResourcesInformation(object sender, IntegrityCheckInformationEventArgs eventArgs)
		{
			resourceEntriesAdded++;
		}

        private void menuItemResourcesSource_Click(object sender, System.EventArgs e)
        {
            SetSourceAndInvariantLanguage();
        }
        private bool SetSourceAndInvariantLanguage()
        {
            ResourcesSourceForm form = new ResourcesSourceForm();
            form.ResourcesGovernorClassName = resourcesGovernorClassName;
            form.InvariantLanguage = invariantLanguage;
            form.Source = source;
            if (form.ShowDialog() == DialogResult.OK)
            {
                invariantLanguage = form.InvariantLanguage;
                resourcesGovernorClassName = form.ResourcesGovernorClassName;
                Source = form.Source;
                SaveConfiguration();
                InitializeResourcesGovernor();
                LoadResources();
                ResourceAdministrationHelper.RefreshTargetLanguageTranslators(invariantLanguage, allTranslators, ref targetLanguageTranslators);
                return true;
            }
            return false;
        }

        private void menuItemExport_Click(object sender, EventArgs e)
        {
            if (exportForm == null)
                exportForm = new ExportForm();

            exportForm.Initialize(resourcesGovernor, setOfResourceSetsCollection);
            exportForm.ShowDialog();
        }

        private void menuItemExit_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
