﻿// File           : ExportResources.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : June 2008
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using Internationalization.Common;
using System.Globalization;
using System.IO;

namespace Internationalization.ResourceAdministration
{
    /// <summary>
    /// ExportResources is a class for exporting resources from one IResourcesGovernor to another
    /// </summary>
    public class ExportResources
    {
        private bool recurseFolders;
        private string sourceResourcesGovernorTypeName;
        private string targetResourcesGovernorTypeName;
        private string source;
        private string target;
        private string namespacePrefix;
        private string addPrefix;
        private string addSuffix;
        private bool includePrefixSuffixWhenEmpty;
        private bool removeAppGlobalResourcesFolderName;
        private bool removeAppLocalResourcesFolderName;
        private bool exportInvariantCulture = true;
        private ExportProgress exportProgress;
        private CreateResourcesGovernor sourceCreateResourcesGovernor;
        private CreateResourcesGovernor targetCreateResourcesGovernor;

        /// <summary>
        /// RecurseFolders specifies whether to recurse through folders if the source is file based
        /// </summary>
        public bool RecurseFolders
        {
            get { return recurseFolders; }
            set { recurseFolders = value; }
        }
        /// <summary>
        /// The type name of the IResourcesGovernor source
        /// </summary>
        public string SourceResourcesGovernorTypeName
        {
            get { return sourceResourcesGovernorTypeName; }
            set { sourceResourcesGovernorTypeName = value; }
        }
        /// <summary>
        /// The type name of the IResourcesGovernor target
        /// </summary>
        public string TargetResourcesGovernorTypeName
        {
            get { return targetResourcesGovernorTypeName; }
            set { targetResourcesGovernorTypeName = value; }
        }
        /// <summary>
        /// The source parameter passed to the source IResourceGovernor
        /// </summary>
        public string Source
        {
            get { return source; }
            set { source = value; }
        }
        /// <summary>
        /// The target parameter passed to the target IResourceGovernor
        /// </summary>
        public string Target
        {
            get { return target; }
            set { target = value; }
        }
        /// <summary>
        /// The namespace prefix to add to resource base names
        /// </summary>
        public string NamespacePrefix
        {
            get { return namespacePrefix; }
            set { namespacePrefix = value; }
        }
        /// <summary>
        /// The prefix to add to resource strings in the target resource entries
        /// </summary>
        public string AddPrefix
        {
            get { return addPrefix; }
            set { addPrefix = value; }
        }
        /// <summary>
        /// The suffix to add to resource strings in the target resource entries
        /// </summary>
        public string AddSuffix
        {
            get { return addSuffix; }
            set { addSuffix = value; }
        }
        /// <summary>
        /// Specifies whether to include the prefix and suffix when the resource entry is empty
        /// </summary>
        public bool IncludePrefixSuffixWhenEmpty
        {
            get { return includePrefixSuffixWhenEmpty; }
            set { includePrefixSuffixWhenEmpty = value; }
        }
        /// <summary>
        /// Specifies whether App_GlobalResources folder name should be removed from the target's base name
        /// </summary>
        public bool RemoveAppGlobalResourcesFolderName
        {
            get { return removeAppGlobalResourcesFolderName; }
            set { removeAppGlobalResourcesFolderName = value; }
        }
        /// <summary>
        /// Specifies whether App_LocalResources folder name should be removed from the target's base name
        /// </summary>
        public bool RemoveAppLocalResourcesFolderName
        {
            get { return removeAppLocalResourcesFolderName; }
            set { removeAppLocalResourcesFolderName = value; }
        }
        /// <summary>
        /// Specifies whether to export the invariant culture (default is true)
        /// </summary>
        public bool ExportInvariantCulture
        {
            get { return exportInvariantCulture; }
            set { exportInvariantCulture = value; }
        }
        /// <summary>
        /// SourceCreateResourcesGovernor is a delegate for creating the source IResourcesGovernor
        /// </summary>
        /// <remarks>SourceCreateResourcesGovernor and SourceResourcesGovernorTypeName are mutually exclusive.
        /// SourceCreateResourcesGovernor takes precedence over SourceResourcesGovernorTypeName.</remarks>
        public CreateResourcesGovernor SourceCreateResourcesGovernor
        {
            get { return sourceCreateResourcesGovernor; }
            set { sourceCreateResourcesGovernor = value; }
        }
        /// <summary>
        /// TargetCreateResourcesGovernor is a delegate for creating the target IResourcesGovernor
        /// </summary>
        /// <remarks>TargetCreateResourcesGovernor and TargetResourcesGovernorTypeName are mutually exclusive.
        /// TargetCreateResourcesGovernor takes precedence over TargetResourcesGovernorTypeName.</remarks>
        public CreateResourcesGovernor TargetCreateResourcesGovernor
        {
            get { return targetCreateResourcesGovernor; }
            set { targetCreateResourcesGovernor = value; }
        }
        /// <summary>
        /// ExportProgress is the ExportProgress delegate to use to provide feedback on the progress of the
        /// export operation
        /// </summary>
        public ExportProgress ExportProgress
        {
            get { return exportProgress; }
            set { exportProgress = value; }
        }
        /// <summary>
        /// Export exports resources from the source IResourceGovernor to the target IResourceGovernor
        /// </summary>
        public void Export()
        {
            if (!RecurseFolders)
                // don't recurse through folders, just export the resources in Source
                ExportOne(Source, Target, String.Empty);
            else
            {
                // recurse through folders (but only if the source is file based)
                IResourcesGovernor sourceResourcesGovernor = GetSourceResourcesGovernor(Source);
                IResourcesGovernor targetResourcesGovernor = GetTargetResourcesGovernor(Target);

                if (!sourceResourcesGovernor.IsFileBased)
                    // the source is not file based so there is nothing to recurse through
                    ExportOne(Source, Target, NamespacePrefix); // String.Empty);
                else
                    RecursiveExport(Source, Target, targetResourcesGovernor.IsFileBased, NamespacePrefix); // String.Empty);
            }
        }
        private IResourcesGovernor GetSourceResourcesGovernor(string source)
        {
            IResourcesGovernor resourcesGovernor;
            if (sourceCreateResourcesGovernor != null)
                resourcesGovernor = sourceCreateResourcesGovernor(source);
            else
                resourcesGovernor =
                    ResourceAdministrationHelper.CreateResourcesGovernor(SourceResourcesGovernorTypeName, source);

            return resourcesGovernor;
        }
        private IResourcesGovernor GetTargetResourcesGovernor(string target)
        {
            IResourcesGovernor resourcesGovernor;
            if (targetCreateResourcesGovernor != null)
                resourcesGovernor = targetCreateResourcesGovernor(source);
            else
                resourcesGovernor =
                    ResourceAdministrationHelper.CreateResourcesGovernor(TargetResourcesGovernorTypeName, target);

            return resourcesGovernor;
        }
        private void RecursiveExport(
            string sourceFolder, string target, bool isTargetFileBased, string baseNamePrefix)
        {
            // start by exporting the resources in this folder
            ExportOne(sourceFolder, target, RemoveAppResourcesFolderName(baseNamePrefix));

            string[] directories = Directory.GetDirectories(sourceFolder);
            foreach (string directory in directories)
            {
                string subFolders = directory.Substring(Source.Length + 1);
                if (isTargetFileBased)
                {
                    // change the target folder to keep in synch with the source folder
                    target = Target + Path.DirectorySeparatorChar + subFolders;
                }
                else
                {
                    // change the baseNamePrefix to keep in synch with the source folder
                    baseNamePrefix = subFolders.Replace(Path.DirectorySeparatorChar, '/') + '/';
                }
                RecursiveExport(directory, target, isTargetFileBased, baseNamePrefix);
            }

        }
        private string RemoveAppResourcesFolderName(string folderName)
        {
            if (folderName != null)
            {
                if (removeAppGlobalResourcesFolderName)
                {
                    // remove "App_GlobalResources/" from the folder name
                    folderName = folderName.Replace("App_GlobalResources/", String.Empty);
                }
                if (removeAppLocalResourcesFolderName)
                {
                    // remove "App_LocalResources/" from the folder name
                    folderName = folderName.Replace("App_LocalResources/", String.Empty);
                }
            }
            return folderName;
        }
        private void ExportOne(string source, string target, string baseNamePrefix)
        {
            IResourcesGovernor sourceResourcesGovernor = GetSourceResourcesGovernor(source);
            IResourcesGovernor targetResourcesGovernor = GetTargetResourcesGovernor(target);

            GetSetOfResourceSets getSetOfResourceSets = delegate(string baseName)
            {
                return CreateSetOfResourceSets(sourceResourcesGovernor, baseName);
            };

            ResourceAdministrationHelper.ExportResources(
                sourceResourcesGovernor,
                targetResourcesGovernor,
                getSetOfResourceSets,
                exportProgress,
                baseNamePrefix,
                AddPrefix,
                AddSuffix,
                IncludePrefixSuffixWhenEmpty,
                exportInvariantCulture);

        }
        private SetOfResourceSets CreateSetOfResourceSets(
            IResourcesGovernor resourcesGovernor, string baseName)
        {
            SetOfResourceSets setOfResourceSets = new SetOfResourceSets(baseName);

            // add the ResourceSet for the invariant culture
            AddResourceSet(setOfResourceSets, resourcesGovernor, baseName, CultureInfo.InvariantCulture);
            if (TargetLanguages.Languages != null)
            {
                // add the ResourceSets for all of the target languages
                foreach (CultureInfo cultureInfo in TargetLanguages.Languages)
                {
                    AddResourceSet(setOfResourceSets, resourcesGovernor, baseName, cultureInfo);
                }
            }
            return setOfResourceSets;
        }
        private void AddResourceSet(SetOfResourceSets setOfResourceSets, IResourcesGovernor resourcesGovernor,
            string baseName, CultureInfo cultureInfo)
        {
            IResourceGovernor resourceGovernor = resourcesGovernor.GetResourceGovernor(baseName, cultureInfo);
            if (resourceGovernor.ResourceExists())
                setOfResourceSets.AddResourceSet(resourceGovernor);
            else
                setOfResourceSets.AddResourceSet(resourceGovernor, null);
        }
    }
    /// <summary>
    /// CreateResourcesGovernor is a delegate for creating an IResourcesGovernor
    /// </summary>
    /// <param name="source">The source of the resources (e.g. a path, a database connection string)</param>
    /// <returns>An IResourcesGovernor for the source</returns>
    public delegate IResourcesGovernor CreateResourcesGovernor(string source);
}
