// File           : IResourcesGovernor.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;
using System.Resources;
using System.Collections;
using System.Text;

namespace Internationalization.ResourceAdministration
{
	/// <summary>
	/// IResourcesGovernor is an interface for managing a complete set of resources
	/// </summary>
    /// <remarks>Take care to notice the distinction between two similarly named interfaces:
    /// IResourcesGovernor and IResourceGovernor. The former managers a set of resources whereas the latter manages
    /// a single resource. Further note the potential terminology confusion between a set of resources, a resource
    /// and a resource entry. A set of resources would be, for example, the complete set of resx files in a
    /// folder. A resource would be a single resx file and a resource entry would be one entry within a single
    /// resx file.</remarks>
	public interface IResourcesGovernor
	{
        /// <summary>
        /// GetInvariantCultureBaseNames gets an array of invariant culture base names in the source of resources
        /// </summary>
        /// <returns>An array of invariant culture base names in the source of resources</returns>
        /// <remarks>For a resx resources governor for a folder containing Form1.resx, Form1.fr.resx,
        /// Form2.resx, Form2.fr.resx and Form2.fr-FR.resx files this method returns {"Form1", "Form2"}.</remarks>
		string[] GetInvariantCultureBaseNames();
        /// <summary>
        /// GetResourceGovernor gets an IResourceGovernor for the given base name and culture
        /// </summary>
        /// <param name="baseName">The base name of the resource</param>
        /// <param name="cultureInfo">The CultureInfo of the resource</param>
        /// <returns>An IResourceGovernor for the given base name and culture</returns>
		IResourceGovernor GetResourceGovernor(string baseName, CultureInfo cultureInfo);
        /// <summary>
        /// GetExistingCultures gets an array of CultureInfo objects for resources which exist for the given
        /// base name
        /// </summary>
        /// <param name="baseName">The base name of the resource</param>
        /// <returns>An array of CultureInfo objects for resources which exist for the given base name</returns>
        /// <remarks>For a resx resources governor for a folder containing Form1.resx, Form1.fr.resx,
        /// and Form1.fr-FR.resx files this method returns {invariant CultureInfo, French CultureInfo,
        /// French (France) CultureInfo} when the base name given is "Form1".</remarks>
        CultureInfo[] GetExistingCultures(string baseName);
        /// <summary>
        /// GetExistingCultures gets an array of CultureInfo objects for resources which exist for the given
        /// base name
        /// </summary>
        /// <param name="baseName">The base name of the resource</param>
        /// <param name="cultureInfo">The CultureInfo for which to restrict the cultures</param>
        /// <returns>An array of CultureInfo objects for resources which exist for the given base name</returns>
        /// <remarks>For a resx resources governor for a folder containing Form1.resx, Form1.fr.resx,
        /// and Form1.fr-FR.resx files this method returns {invariant CultureInfo, French CultureInfo,
        /// French (France) CultureInfo} when the base name given is "Form1".</remarks>
        CultureInfo[] GetExistingCultures(string baseName, CultureInfo cultureInfo);
        /// <summary>
        /// IsFileBased returns true if the resources are file based
        /// </summary>
        bool IsFileBased {get;}
        /// <summary>
        /// IsDatabaseBased returns true if the resources are database based
        /// </summary>
        bool IsDatabaseBased { get;}
        /// <summary>
        /// SupportsComments is true if the resource governor allows comments to be stored with resource entries
        /// </summary>
        bool SupportsComments { get; set; }
        /// <summary>
        /// SupportsFileReferences is true if the resource governor allows resource entry values to be file
        /// references
        /// </summary>
        bool SupportsFileReferences { get; set; }
        /// <summary>
        /// CommentsAreOptional is true if the resource governor can optionally work with or without comment support
        /// </summary>
        bool CommentsAreOptional { get;}
        /// <summary>
        /// FileReferencesAreOptional is true if the resource governor can optionally work with or without
        /// file reference support
        /// </summary>
        bool FileReferencesAreOptional { get;}
        /// <summary>
        /// Exists returns true if the resource source exists
        /// </summary>
        /// <remarks>The interpretation of "existence" is dependent upon the physical implementation of the
        /// resources. For file based resources existence is determined by the existence of the folder
        /// which contains the resource files. For database resources existence is determined by the existence
        /// of the database.</remarks>
        bool Exists { get; }
        /// <summary>
        /// The absolute path to resource files (if the resources are filed based)
        /// </summary>
        string ResourceFileAbsolutePath { get; }
        /// <summary>
        /// The relative path to resource files (if the resources are filed based)
        /// </summary>
        string ResourceFileRelativePath { get; }
        /// <summary>
        /// SupportedTypes is an array of all of the Types supported by the resource format
        /// </summary>
        /// <remarks>If SupportedTypes is null there are no restrictions on Types.</remarks>
        Type[] SupportedTypes { get; }
        /// <summary>
        /// Create creates the resource source
        /// </summary>
        /// <remarks>For file based resources Create creates the folder which contains the resource files.
        /// For database based resources Create creates the database and the resources table.</remarks>
        void Create();
    }
	/// <summary>
    /// IResourceGovernor is an interface for managing a resource
	/// </summary>
	public interface IResourceGovernor
	{
        /// <summary>
        /// ReadResourceSet reads the resource entries from the source into a ResourceSet
        /// </summary>
        /// <returns>A ResourceSet containing the resource entries read from the source</returns>
        ResourceSet ReadResourceSet();
        /// <summary>
        /// WriteResourceSet writes resource entries in a ResourceSet back to the source
        /// </summary>
        /// <param name="resourceSet">The ResourceSet containing the resource entries to write</param>
        void WriteResourceSet(ResourceSet resourceSet);
        /// <summary>
        /// ResourceExists determines if the resource exists
        /// </summary>
        /// <returns>True if the resource exists</returns>
        /// <remarks>For file based resources ResourceExists returns true if the file exists</remarks>
		bool ResourceExists();
        /// <summary>
        /// ResourceIsReadOnly returns true if the resource is read-only
        /// </summary>
        /// <returns>True if the resource is read-only</returns>
        /// <remarks>For file based resources ResourceIsReadOnly returns true if the file is marked as read-only.
        /// This is often true for files which are under version control and have not been checked out.</remarks>
		bool ResourceIsReadOnly();
        /// <summary>
        /// ResourceKeyExists returns true if the given resource entry key exists in the resource
        /// </summary>
        /// <param name="key">The key to check for existence</param>
        /// <returns>True if the given resource entry key exists in the resource</returns>
		bool ResourceKeyExists(string key);
        /// <summary>
        /// AddResourceSet adds resource entries from one ResourceSet to another
        /// </summary>
        /// <param name="resourceSet1">The ResourceSet into which resource entries from the second 
        /// ResourceSet are added</param>
        /// <param name="resourceSet2">The ResourceSet containing resource entries to add to the first ResourceSet</param>
        /// <returns>True if any resource entries were added to the first ResourceSet</returns>
        /// <remarks>AddResourceSet adds all of the entries in resourceSet2 which are not already in
        /// resourceSet1 to resourceSet1. So resourceSet1 might be for "Form1.fr.resx" and resourceSet2
        /// might be for "Form1.resx". The result is a flattened (Single File Mode) ResourceSet in resourceSet1.
        /// </remarks>
        bool AddResourceSet(ResourceSet resourceSet1, ResourceSet resourceSet2);
        /// <summary>
        /// ReintegrateResourceSet reintegrates resource entries from an incoming ResourceSet to a master ResourceSet
        /// </summary>
        /// <param name="flattenedMasterResourceSet">A ResourceSet containing the complete set of resource entries
        /// from parent cultures</param>
        /// <param name="masterResourceSet">The original (unflattened) ResourceSet to which incoming resource entries
        /// are added or modified</param>
        /// <param name="incomingResourceSet">The incoming (unflattened) ResourceSet containing added or modified resource entries</param>
        /// <param name="acceptResourceEntry">An AcceptResourceEntry delegate which can be used to filter incoming
        /// resource entries. acceptResourceEntry can be null to ignore this filtering process</param>
        /// <returns>True if changes were made to the master ResourceSet</returns>
        bool ReintegrateResourceSet(ResourceSet flattenedMasterResourceSet,
            ResourceSet masterResourceSet, ResourceSet incomingResourceSet, AcceptResourceEntry acceptResourceEntry);
        /// <summary>
        /// CreateDataNode creates a data node given a name and a value
        /// </summary>
        /// <param name="name">The name of the data node</param>
        /// <param name="value">The value of the data node</param>
        /// <returns>A data node representing the supplied data</returns>
        /// <remarks>The exact implementation of the data node is dependant upon the functionality supported by
        /// the ResourceGovernor. A .NET Framework 2.0 ResxResourceGovernor, for example, creates ResXDataNode
        /// objects whereas a ResourcesResourceGovernor simply returns the data value given (because it lacks
        /// support for complex data nodes).</remarks>
        object CreateDataNode(string name, object value);
        /// <summary>
        /// CreateFileRef creates a file reference object given a file name and data type
        /// </summary>
        /// <param name="fileName">The file name to create the file reference for</param>
        /// <param name="typeName">The data type of the data stored in the file</param>
        /// <returns>This implementation throws a NotImplementedException</returns>
        /// <remarks>The implementation of this method is dependant upon the functionality supported by
        /// the ResourceGovernor. A .NET Framework 2.0 ResxResourceGovernor, for example, creates ResXFileRef
        /// objects whereas a ResourcesResourceGovernor throws a NotImplementedException (because it lacks
        /// support for file references). Check the corresponding ResourcesGovernor.SupportsFileReferences
        /// property before calling CreateFileRef to determine if file references are supported.</remarks>
        object CreateFileRef(string fileName, string typeName);
        /// <summary>
        /// CreateFileRef creates a file reference object given a file name and data type
        /// </summary>
        /// <param name="fileName">The file name to create the file reference for</param>
        /// <param name="typeName">The data type of the data stored in the file</param>
        /// <param name="textFileEncoding">The Encoding mechanism of the file reference (if the file is a text file)</param>
        /// <returns>This implementation throws a NotImplementedException</returns>
        /// <remarks>The implementation of this method is dependant upon the functionality supported by
        /// the ResourceGovernor. A .NET Framework 2.0 ResxResourceGovernor, for example, creates ResXFileRef
        /// objects whereas a ResourcesResourceGovernor throws a NotImplementedException (because it lacks
        /// support for file references). Check the corresponding ResourcesGovernor.SupportsFileReferences
        /// property before calling CreateFileRef to determine if file references are supported.</remarks>
        object CreateFileRef(string fileName, string typeName, Encoding textFileEncoding);
        /// <summary>
        /// BaseName is the qualified base name which the resources represent
        /// </summary>
        string BaseName { get;}
        /// <summary>
        /// CultureInfo is the CultureInfo identifying the culture of the resources
        /// </summary>
        CultureInfo CultureInfo { get;}
	}
    /// <summary>
    /// AcceptResourceEntry is a delegate used by ReintegrateResourceSet to filter incoming resource entries
    /// </summary>
    /// <param name="entry">The DictionaryEntry representing the incoming resource entry</param>
    /// <param name="originalValue">The original value of the resource entry in the master resource set</param>
    /// <returns>True if the incoming change should be accepted. False if the incoming change should be ignored.</returns>
    /// <example>The following example of an AcceptResourceEntry delegate rejects incoming entries which are not
    /// strings or images. You would take this approach if your translators use WinRes to translate forms but
    /// you do not want them to be able to change control positions, sizes and other properties.
    /// 
    /// private bool AcceptStringAndImageResourceEntry(DictionaryEntry entry, object originalValue)
    /// {
    ///     return entry.Value is string || entry.Value is Image;
    /// }
    /// </example>
    public delegate bool AcceptResourceEntry(DictionaryEntry entry, object originalValue);
}
