// File           : LanguagesForm.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Globalization;
using System.Collections.Specialized;
using Internationalization.Common;

namespace Internationalization.ResourceAdministration
{
	/// <summary>
	/// LanguagesForm is a form for letting the user select a number of languages
	/// </summary>
	public class LanguagesForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.Button buttonOK;
		private System.Windows.Forms.Button buttonCancel;
		private System.Windows.Forms.CheckedListBox checkedListBoxLanguages;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

        /// <summary>
        /// Constructs a LanguagesForm object
        /// </summary>
        /// <param name="fromLanguage">The language from which text is translated</param>
		public LanguagesForm(string fromLanguage)
		{
			InitializeComponent();

			InitializeLanguages(fromLanguage);
		}
		private void InitializeLanguages(string fromLanguage)
		{
			// list the target languages (checked) first
			CultureInfo[] targetLanguages = TargetLanguages.Languages;
			foreach(CultureInfo cultureInfo in targetLanguages)
			{
				checkedListBoxLanguages.Items.Add(cultureInfo.DisplayName, true);
			}
			// list all of the other neutral cultures (unchecked) next
			foreach(CultureInfo cultureInfo in CultureInfo.GetCultures(CultureTypes.NeutralCultures))
			{
				if (Array.IndexOf(targetLanguages, cultureInfo) == -1 && ! cultureInfo.Equals(CultureInfo.InvariantCulture)
					&& cultureInfo.Name != fromLanguage)
					// the neutral culture is not already in the list of target languages and
					// it is not the invariant culture and it is not the "from" language
					checkedListBoxLanguages.Items.Add(cultureInfo.DisplayName, false);
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(LanguagesForm));
            this.panel1 = new System.Windows.Forms.Panel();
            this.buttonCancel = new System.Windows.Forms.Button();
            this.buttonOK = new System.Windows.Forms.Button();
            this.checkedListBoxLanguages = new System.Windows.Forms.CheckedListBox();
            this.panel1.SuspendLayout();
            this.SuspendLayout();
            // 
            // panel1
            // 
            this.panel1.Controls.Add(this.buttonCancel);
            this.panel1.Controls.Add(this.buttonOK);
            resources.ApplyResources(this.panel1, "panel1");
            this.panel1.Name = "panel1";
            // 
            // buttonCancel
            // 
            this.buttonCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            resources.ApplyResources(this.buttonCancel, "buttonCancel");
            this.buttonCancel.Name = "buttonCancel";
            // 
            // buttonOK
            // 
            this.buttonOK.DialogResult = System.Windows.Forms.DialogResult.OK;
            resources.ApplyResources(this.buttonOK, "buttonOK");
            this.buttonOK.Name = "buttonOK";
            // 
            // checkedListBoxLanguages
            // 
            this.checkedListBoxLanguages.CheckOnClick = true;
            resources.ApplyResources(this.checkedListBoxLanguages, "checkedListBoxLanguages");
            this.checkedListBoxLanguages.FormattingEnabled = true;
            this.checkedListBoxLanguages.Name = "checkedListBoxLanguages";
            // 
            // LanguagesForm
            // 
            this.AcceptButton = this.buttonOK;
            this.CancelButton = this.buttonCancel;
            resources.ApplyResources(this, "$this");
            this.Controls.Add(this.checkedListBoxLanguages);
            this.Controls.Add(this.panel1);
            this.Name = "LanguagesForm";
            this.panel1.ResumeLayout(false);
            this.ResumeLayout(false);

		}
		#endregion

        /// <summary>
        /// SelectedLanguages is the StringCollection of language names which are selected
        /// </summary>
		public StringCollection SelectedLanguages
		{
			get
			{
				StringCollection selectedLanguages = new StringCollection();
				foreach(string displayName in checkedListBoxLanguages.CheckedItems)
				{
					selectedLanguages.Add(displayName);
				}
				return selectedLanguages;
			}
		}
        /// <summary>
        /// SelectedLanguageCultureInfos is the CultureInfo array of the languages which are selected
        /// </summary>
		public CultureInfo[] SelectedLanguageCultureInfos
		{
			get
			{
				StringCollection selectLanguages = SelectedLanguages;
				CultureInfo[] cultureInfos = new CultureInfo[selectLanguages.Count];
				int cultureInfoNumber = 0;
				foreach(string selectLanguage in selectLanguages)
				{
					cultureInfos[cultureInfoNumber++] = NeutralDisplayNameToCultureInfo(selectLanguage);
				}
				return cultureInfos;
			}
		}
		private CultureInfo NeutralDisplayNameToCultureInfo(string displayName)
		{
			foreach(CultureInfo cultureInfo in CultureInfo.GetCultures(CultureTypes.NeutralCultures))
			{
				if (cultureInfo.DisplayName == displayName)
					return cultureInfo;
			}
			return null;
		}
	}
}
