// File           : ResourceAdministrationHelper.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;
using System.Collections.Specialized;
using System.Globalization;
using System.Resources;
using System.Reflection;
using System.Data;
using Internationalization.Common;
using Internationalization.Translation;

namespace Internationalization.ResourceAdministration
{
	/// <summary>
	/// ResourceAdministrationHelper provides a collection of utilities for administering resources
	/// </summary>
	public class ResourceAdministrationHelper
	{
        /// <summary>
        /// Constructs a ResourceAdministrationHelper object
        /// </summary>
		public ResourceAdministrationHelper()
		{
		}
        /// <summary>
        /// RefreshTargetLanguageTranslators refreshes the list of target language translators
        /// </summary>
        /// <param name="invariantLanguage">The language which is being translated from</param>
        /// <param name="allTranslators">A TranslatorCollection of all of the available translators</param>
        /// <param name="targetLanguageTranslators">The ArrayList of target language translators</param>
        /// <remarks>The list of target language translators is always the same length as the list of
        /// target languages (as represented in TargetLanguages.Languages). Each element in this list
        /// corresponds to the same element number in the list of target languages. Where no enabled translator
        /// can be found to translate the language pair the translator is null.</remarks>
		public static void RefreshTargetLanguageTranslators(
			string invariantLanguage, TranslatorCollection allTranslators, ref ArrayList targetLanguageTranslators)
		{
			if (targetLanguageTranslators == null)
			{
				targetLanguageTranslators = new ArrayList();
				foreach(CultureInfo cultureInfo in TargetLanguages.Languages)
				{
					targetLanguageTranslators.Add(
						allTranslators.GetEnabledTranslator(invariantLanguage, cultureInfo.Name));
				}
			}
			else
			{
				for(int languageNumber = 0; languageNumber < TargetLanguages.Languages.GetLength(0); languageNumber++)
				{
					ITranslator translator;
					if (languageNumber >= targetLanguageTranslators.Count)
					{
						targetLanguageTranslators.Add(null);
						translator = null;
					}
					else
						translator = (ITranslator) targetLanguageTranslators[languageNumber];

					if (translator == null || ! translator.Enabled)
					{
						// either there isn't a translator for this language pair or it has been disabled since it was initialized
						CultureInfo cultureInfo = TargetLanguages.Languages[languageNumber];

						targetLanguageTranslators[languageNumber] =
							allTranslators.GetEnabledTranslator(invariantLanguage, cultureInfo.Name);
					}
				}
			}
		}
        /// <summary>
        /// IsWindowsForm returns true if the resource Hashtable represents a Windows Form
        /// </summary>
        /// <param name="resourceTable">The Hashtable of resource entries</param>
        /// <returns>True if the resource Hashtable represents a Windows Form</returns>
		public static bool IsWindowsForm(Hashtable resourceTable)
		{
			if (resourceTable.ContainsKey(">>$this.Type"))
			{
                ResourcesDataNodeInterpreter dataNodeInterpreter =
                    new ResourcesDataNodeInterpreter(">>$this.Type", resourceTable[">>$this.Type"]);
				return dataNodeInterpreter.Value.ToString().StartsWith("System.Windows.Forms.Form,");
			}
			return false;
		}
        /// <summary>
        /// CreateCultureNeutralWindowsFormResourceSet creates a new culture neutral ResourceSet for a Windows Form
        /// </summary>
        /// <param name="invariantResourceTable">The Hashtable of resource entries for the invariant Windows Form</param>
        /// <returns>A culture neutral ResourceSet for a Windows Form</returns>
		public static ResourceSet CreateCultureNeutralWindowsFormResourceSet(Hashtable invariantResourceTable)
		{
			ResourceSet resourceSet = new ResourceSet(new EmptyResourceReader());
			Hashtable resourceTable = ResourceUtilities.GetResourceSetTable(resourceSet);
			foreach(DictionaryEntry dictionaryEntry in invariantResourceTable)
			{
                ResourcesDataNodeInterpreter dataNodeInterpreter = new ResourcesDataNodeInterpreter(
                    dictionaryEntry.Key.ToString(), dictionaryEntry.Value);
				if (dataNodeInterpreter.Value != null &&
                    ResourceUtilities.IsLocalizable(dataNodeInterpreter.Name, dataNodeInterpreter.Value))
					// this entry might be localized
					resourceTable.Add(dataNodeInterpreter.Name, dataNodeInterpreter.DataNode);
			}
			return resourceSet;
		}
        /// <summary>
        /// CreateResourcesGovernor creates an IResourcesGovernor given a resources governor class name and
        /// the source of the resources which the governor represents
        /// </summary>
        /// <param name="resourcesGovernorClassName">The unqualified name of the resources governor class</param>
        /// <param name="sourceValue">The source of the resources. For a file based resources governor this would
        /// be the path to the resources. For a database based resources governor this would be the connection
        /// string.</param>
        /// <returns>An IResourcesGovernor for the given resources governor class name and source</returns>
        public static IResourcesGovernor CreateResourcesGovernor(string resourcesGovernorClassName, string sourceValue)
        {
            // This method will usually equate to "return new ResXResourcesGovernor(sourceValue);".
            Type resourcesGovernorType;
            if (resourcesGovernorClassName.IndexOf(",") == -1)
            {
                // the class name does not have an assembly name so try to load it from ResourceAdministration.dll
                // and assume a known namespace
                Assembly assembly = Assembly.LoadFrom("ResourceAdministration.dll");
                resourcesGovernorType = assembly.GetType("Internationalization.ResourceAdministration." + resourcesGovernorClassName, true, true);
            }
            else
                resourcesGovernorType = Type.GetType(resourcesGovernorClassName);

            return (IResourcesGovernor) Activator.CreateInstance(resourcesGovernorType, new object[] {sourceValue});
        }
        /// <summary>
        /// ExportResources exports the resources from one IResourcesGovernor to another
        /// </summary>
        /// <param name="sourceResourcesGovernor">The IResourcesGovernor of the source of the resources</param>
        /// <param name="targetResourcesGovernor">The IResourcesGovernor of the target of the resources</param>
        /// <param name="getSetOfResourceSets">A GetSetOfResourceSet delegate to get a SetOfResourceSets</param>
        /// <param name="exportProgress">An ExportProgress delegate to report on the progress of the export operation</param>
        /// <param name="baseNamePrefix">The prefix to apply to the base name of target resources</param>
        /// <param name="addPrefix">A prefix to add to converted resource entry values</param>
        /// <param name="addSuffix">A suffix to add to converted resource entry values</param>
        /// <param name="includePrefixSuffixWhenEmpty">Specifies whether prefixes and suffixes should be includes for empty resource values</param>
        /// <returns>True if the operation was successful</returns>
        public static bool ExportResources(
            IResourcesGovernor sourceResourcesGovernor,
            IResourcesGovernor targetResourcesGovernor,
            GetSetOfResourceSets getSetOfResourceSets,
            ExportProgress exportProgress,
            string baseNamePrefix,
            string addPrefix,
            string addSuffix,
            bool includePrefixSuffixWhenEmpty)
        {
            return ExportResources(sourceResourcesGovernor,
            targetResourcesGovernor,
            getSetOfResourceSets,
            exportProgress,
            baseNamePrefix,
            addPrefix,
            addSuffix,
            includePrefixSuffixWhenEmpty,
            true);
        }
        /// <summary>
        /// ExportResources exports the resources from one IResourcesGovernor to another
        /// </summary>
        /// <param name="sourceResourcesGovernor">The IResourcesGovernor of the source of the resources</param>
        /// <param name="targetResourcesGovernor">The IResourcesGovernor of the target of the resources</param>
        /// <param name="getSetOfResourceSets">A GetSetOfResourceSet delegate to get a SetOfResourceSets</param>
        /// <param name="exportProgress">An ExportProgress delegate to report on the progress of the export operation</param>
        /// <param name="baseNamePrefix">The prefix to apply to the base name of target resources</param>
        /// <param name="addPrefix">A prefix to add to converted resource entry values</param>
        /// <param name="addSuffix">A suffix to add to converted resource entry values</param>
        /// <param name="includePrefixSuffixWhenEmpty">Specifies whether prefixes and suffixes should be includes for empty resource values</param>
        /// <param name="exportInvariantCulture">Specifies whether the invariant culture should be exported</param>
        /// <returns>True if the operation was successful</returns>
        public static bool ExportResources(
            IResourcesGovernor sourceResourcesGovernor,
            IResourcesGovernor targetResourcesGovernor,
            GetSetOfResourceSets getSetOfResourceSets,
            ExportProgress exportProgress,
            string baseNamePrefix,
            string addPrefix,
            string addSuffix,
            bool includePrefixSuffixWhenEmpty,
            bool exportInvariantCulture)
        {
            string[] baseNames = sourceResourcesGovernor.GetInvariantCultureBaseNames();
            int position = 1;

            bool convertDataNodes =
                (sourceResourcesGovernor.SupportsComments && !targetResourcesGovernor.SupportsComments) ||
                (sourceResourcesGovernor.SupportsFileReferences && !targetResourcesGovernor.SupportsFileReferences);

            foreach (string baseName in baseNames)
            {
                if (exportProgress != null)
                    exportProgress(null,
                        new ExportProgressEventArgs(baseName, position++, baseNames.GetLength(0)));

                SetOfResourceSets sourceSetOfResourceSet = getSetOfResourceSets(baseName);

                if (sourceSetOfResourceSet.AnyExist())
                {
                    if (!targetResourcesGovernor.Exists)
                        targetResourcesGovernor.Create();

                    bool originalResourceSetsModified = false;

                    if (exportInvariantCulture)
                    {
                        // export the invariant culture
                        originalResourceSetsModified = ResourceAdministrationHelper.ExportResourceSet(
                            sourceSetOfResourceSet.GetResourceSet(CultureInfo.InvariantCulture),
                            targetResourcesGovernor.GetResourceGovernor(baseNamePrefix + baseName, CultureInfo.InvariantCulture),
                            targetResourcesGovernor.SupportedTypes,
                            convertDataNodes,
                            addPrefix,
                            addSuffix,
                            includePrefixSuffixWhenEmpty);
                    }

                    if (TargetLanguages.Languages != null)
                    {
                        // export all of the cultures for the target languages
                        foreach (CultureInfo cultureInfo in TargetLanguages.Languages)
                        {
                            if (ResourceAdministrationHelper.ExportResourceSet(
                                sourceSetOfResourceSet.GetResourceSet(cultureInfo),
                                targetResourcesGovernor.GetResourceGovernor(baseNamePrefix + baseName, cultureInfo),
                                targetResourcesGovernor.SupportedTypes,
                                convertDataNodes,
                                addPrefix,
                                addSuffix,
                                includePrefixSuffixWhenEmpty))
                                originalResourceSetsModified = true;
                        }
                    }
                    if (originalResourceSetsModified)
                    {
                        // the resource sets have been changed so re-read them
                        sourceSetOfResourceSet.Refresh();
                    }
                }
            }
            return true;
        }

        /// <summary>
        /// ExportResourceSet exports a ResourceSet using an IResourceGovernor
        /// </summary>
        /// <param name="resourceSet">The ResourceSet to export</param>
        /// <param name="targetResourceGovernor">The IResourceGovernor to export the ResourceSet to</param>
        /// <param name="supportedTypes">An array of all of the Types supported by the resource format</param>
        /// <param name="convertDataNodes">Specifies whether data nodes should be converted to simple types</param>
        /// <param name="addPrefix">A prefix to add to converted resource entry values</param>
        /// <param name="addSuffix">A suffix to add to converted resource entry values</param>
        /// <param name="includePrefixSuffixWhenEmpty">Specifies whether prefixes and suffixes should be includes for empty resource values</param>
        /// <returns>True if changes were made to the original ResourceSet</returns>
        public static bool ExportResourceSet(ResourceSet resourceSet, IResourceGovernor targetResourceGovernor,
            Type[] supportedTypes, bool convertDataNodes, string addPrefix, string addSuffix, 
            bool includePrefixSuffixWhenEmpty)
        {
            bool changesMade = false;
            if (resourceSet != null)
            {
                if (addPrefix != String.Empty || addSuffix != String.Empty)
                {
                    // add the prefixes and suffixes
                    AddPrefixAndSuffix(resourceSet, includePrefixSuffixWhenEmpty, addPrefix, addSuffix);
                    changesMade = true;
                }
                if (convertDataNodes)
                {
                    // the source supports comments and/or file references but the target does not so
                    // the data nodes need to be converted to simple types before they can be exported
                    ResourceSet targetResourceSet = ResourceAdministrationHelper.
                        CreateResourceSetWithSimpleDataNodes(resourceSet, ref changesMade);
                    if (changesMade)
                        resourceSet = targetResourceSet;
                }
                if (supportedTypes != null)
                {
                    ResourceSet targetResourceSet = ResourceAdministrationHelper.
                        CreateResourceSetWithSupportedTypesOnly(resourceSet, supportedTypes, ref changesMade);
                    if (changesMade)
                        resourceSet = targetResourceSet;
                }

                targetResourceGovernor.WriteResourceSet(resourceSet);
            }
            return changesMade;
        }
        private static void AddPrefixAndSuffix(
            ResourceSet resourceSet, bool includePrefixSuffixWhenEmpty, string addPrefix, string addSuffix)
        {
            IEnumerator enumerator = resourceSet.GetEnumerator();
            while (enumerator.MoveNext())
            {
                DictionaryEntry entry = (DictionaryEntry)enumerator.Current;
                ResourcesDataNodeInterpreter dataNodeInterpreter =
                    new ResourcesDataNodeInterpreter(entry.Key.ToString(), entry.Value);

                if (dataNodeInterpreter.Value is String && ResourceUtilities.IsLocalizable(entry.Key.ToString(), dataNodeInterpreter.Value))
                {
                    if (includePrefixSuffixWhenEmpty || !String.IsNullOrEmpty(dataNodeInterpreter.Value.ToString()))
                    {
                        if (addPrefix != String.Empty)
                            dataNodeInterpreter.Value = addPrefix + dataNodeInterpreter.Value;
                        if (addSuffix != String.Empty)
                            dataNodeInterpreter.Value = dataNodeInterpreter.Value + addSuffix;
                    }
                }
            }
        }
        /// <summary>
        /// CreateResourceSetWithSimpleDataNodes creates a new ResourceSet with simple data types instead of data nodes
        /// </summary>
        /// <param name="resourceSet">The source ResourceSet</param>
        /// <param name="changesMade">A value passed by reference that is set to true if any data nodes needed to be converted to simple types</param>
        /// <returns>A new ResourceSet with simple types instead of data nodes</returns>
        /// <remarks>Use this method to convert ResourceSets from sources that support data nodes to sources
        /// that do not support data nodes</remarks>
        public static ResourceSet CreateResourceSetWithSimpleDataNodes(ResourceSet resourceSet, ref bool changesMade)
        {
            // convert data nodes to values with simple types
            ResourceSet targetResourceSet = new ResourceSet(new EmptyResourceReader());
            Hashtable targetHashtable = ResourceUtilities.GetResourceSetTable(targetResourceSet);
            IEnumerator enumerator = resourceSet.GetEnumerator();
            while (enumerator.MoveNext())
            {
                DictionaryEntry entry = (DictionaryEntry)enumerator.Current;

                ResourcesDataNodeInterpreter dataNodeInterpreter =
                    new ResourcesDataNodeInterpreter(entry.Key.ToString(), entry.Value);

                // convert the resource entry from a data node to a value with a simple type
                targetHashtable.Add(entry.Key, dataNodeInterpreter.Value);
                if (dataNodeInterpreter.IsDataNode)
                {
                    changesMade = true;
                }
            }
            return targetResourceSet;
        }
        /// <summary>
        /// CreateResourceSetWithSupportedTypesOnly creates a new ResourceSet that contains only resource entries in array of the supported types
        /// </summary>
        /// <param name="resourceSet">The source ResourceSet</param>
        /// <param name="supportedTypes">An array of Type objects that are supported by the target resource format</param>
        /// <param name="changesMade">A value passed by reference that is set to true if not all of the resource entries from the source were included in the new ResourceSet</param>
        /// <returns>A new ResourceSet with only resource entries that are one of the supported types</returns>
        /// <remarks>Use this method to convert ResourceSets to targets that do not support as many types as the
        /// source</remarks>
        public static ResourceSet CreateResourceSetWithSupportedTypesOnly(ResourceSet resourceSet, Type[] supportedTypes, ref bool changesMade)
        {
            // remove all resource entries that have values that are not in the supported types
            ResourceSet targetResourceSet = new ResourceSet(new EmptyResourceReader());
            Hashtable targetHashtable = ResourceUtilities.GetResourceSetTable(targetResourceSet);
            IEnumerator enumerator = resourceSet.GetEnumerator();
            while (enumerator.MoveNext())
            {
                DictionaryEntry entry = (DictionaryEntry)enumerator.Current;

                ResourcesDataNodeInterpreter dataNodeInterpreter =
                    new ResourcesDataNodeInterpreter(entry.Key.ToString(), entry.Value);

                if (TypeIsSupported(dataNodeInterpreter.Value.GetType(), supportedTypes))
                    // the type is supported so include this entry in the target list
                    targetHashtable.Add(entry.Key, entry.Value);
                else
                    // the type is not supported so it is not included (and therefore changes have been made)
                    changesMade = true;
            }
            return targetResourceSet;
        }
        private static bool TypeIsSupported(Type type, Type[] supportedTypes)
        {
            foreach (Type supportedType in supportedTypes)
            {
                if (type == supportedType)
                    return true;
            }
            return false;
        }
    }

    /// <summary>
    /// GetSetOfResourceSets is a delegate to return a SetOfResourceSets object for a given base name
    /// </summary>
    /// <param name="baseName">The base name to get the SetOfResourceSets for</param>
    /// <returns>A SetOfResourceSets object for a given base name</returns>
    public delegate SetOfResourceSets GetSetOfResourceSets(string baseName);

    /// <summary>
    /// ExportProgress is a delegate used to report on the progress of an export operation
    /// </summary>
    /// <param name="sender">null</param>
    /// <param name="args">An ExportProgressEventArgs that represents the progress of the export operation</param>
    public delegate void ExportProgress(object sender, ExportProgressEventArgs args);

    /// <summary>
    /// ExportProgressEventArgs represents the progress of an export operation
    /// </summary>
    public class ExportProgressEventArgs
    {
        private int position;
        /// <summary>
        /// The current position of the progress
        /// </summary>
        public int Position
        {
            get { return position; }
            set { position = value; }
        }
        private int maximum;
        /// <summary>
        /// The maximum size of the operation
        /// </summary>
        public int Maximum
        {
            get { return maximum; }
            set { maximum = value; }
        }
        private string baseName;
        /// <summary>
        /// The base name of the resource export operation
        /// </summary>
        public string BaseName
        {
            get { return baseName; }
            set { baseName = value; }
        }
        /// <summary>
        /// Constructs an ExportProgressEventArgs object
        /// </summary>
        /// <param name="baseName">The base name of the resource export operation</param>
        /// <param name="position">The current position of the progress</param>
        /// <param name="maximum">The maximum size of the operation</param>
        public ExportProgressEventArgs(string baseName, int position, int maximum)
        {
            this.baseName = baseName;
            this.maximum = maximum;
            this.position = position;
        }
    }
}
