// File           : ResourceGovernor.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;
using System.Resources;
using System.Reflection;
using System.Globalization;
using System.Text;

namespace Internationalization.ResourceAdministration
{
	/// <summary>
	/// ResourceGovernor is an abstract class for managing a resource
	/// </summary>
	public abstract class ResourceGovernor
	{
		private string baseName;
		private CultureInfo cultureInfo;
		
        /// <summary>
        /// Constructs a ResourceGovernor object
        /// </summary>
		public ResourceGovernor()
		{
		}
        /// <summary>
        /// Initialize initializes the resource governor
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="cultureInfo">The CultureInfo identifying the culture of the resources</param>
        protected void Initialize(string baseName, CultureInfo cultureInfo)
		{
			this.baseName = baseName;
			this.cultureInfo = cultureInfo;
		}
        /// <summary>
        /// BaseName is the qualified base name which the resources represent
        /// </summary>
		public string BaseName
		{
			get {return baseName;}
		}
        /// <summary>
        /// CultureInfo is the CultureInfo identifying the culture of the resources
        /// </summary>
		public CultureInfo CultureInfo
		{
			get {return cultureInfo;}
		}

        /// <summary>
        /// ReadResourceSet reads the resource entries from the source into a ResourceSet
        /// </summary>
        /// <returns>A ResourceSet containing the resource entries read from the source</returns>
		public abstract ResourceSet ReadResourceSet();

        /// <summary>
        /// ReadResourceSet reads the resource entries from the source into a ResourceSet
        /// </summary>
        /// <param name="resourceReader">An IResourceReader to use for reading the resource entries</param>
        /// <returns>A ResourceSet containing the resource entries read from the source</returns>
		protected virtual ResourceSet ReadResourceSet(IResourceReader resourceReader)
		{
			return new ResourceSet(resourceReader);
		}
        /// <summary>
        /// WriteResourceSet writes resource entries in a ResourceSet back to the source
        /// </summary>
        /// <param name="resourceWriter">An IResourceWriter to use to write the resource entries</param>
        /// <param name="resourceSet">The ResourceSet containing the resource entries to write</param>
		protected virtual void WriteResourceSet(IResourceWriter resourceWriter, ResourceSet resourceSet)
		{
			Hashtable table = GetResourceSetTable(resourceSet);
			foreach(DictionaryEntry dictionaryEntry in table)
			{
				resourceWriter.AddResource(dictionaryEntry.Key.ToString(), dictionaryEntry.Value);
			}
			resourceWriter.Generate();
			resourceWriter.Close();
		}
        /// <summary>
        /// GetResourceSetTable gets the Hashtable which contains the resource entries in the ResourceSet
        /// </summary>
        /// <param name="resourceSet">The ResourceSet to get the Hashtable from</param>
        /// <returns>The Hashtable which contains the resource entries in the ResourceSet</returns>
		protected virtual Hashtable GetResourceSetTable(ResourceSet resourceSet)
		{
			FieldInfo fieldInfo = 
				resourceSet.GetType().GetField("Table", BindingFlags.Instance | BindingFlags.NonPublic);
			if (fieldInfo != null)
			{
				object tableObject = fieldInfo.GetValue(resourceSet);
				if (tableObject != null)
					return (Hashtable) tableObject;
			}
			return null;
		}
        /// <summary>
        /// ResourceKeyExists returns true if the key exists in the resource entries
        /// </summary>
        /// <param name="key">The key to use to test the resource entries for</param>
        /// <returns>True if the key exists in the resource entries</returns>
		public bool ResourceKeyExists(string key)
		{
			ResourceSet resourceSet = ReadResourceSet();
			Hashtable hashtable = GetResourceSetTable(resourceSet);
			return hashtable.ContainsKey(key);
		}
        /// <summary>
        /// AddResourceSet adds resource entries from one ResourceSet to another
        /// </summary>
        /// <param name="resourceSet1">The ResourceSet into which resource entries from the second 
        /// ResourceSet are added</param>
        /// <param name="resourceSet2">The ResourceSet containing resource entries to add to the first ResourceSet</param>
        /// <returns>True if any resource entries were added to the first ResourceSet</returns>
        /// <remarks>AddResourceSet adds all of the entries in resourceSet2 which are not already in
        /// resourceSet1 to resourceSet1. So resourceSet1 might be for "Form1.fr.resx" and resourceSet2
        /// might be for "Form1.resx". The result is a flattened (Single File Mode) ResourceSet in resourceSet1.
        /// </remarks>
        public bool AddResourceSet(ResourceSet resourceSet1, ResourceSet resourceSet2)
        {
            bool changesMade = false;
            Hashtable resourceTable1 = GetResourceSetTable(resourceSet1);
            IDictionaryEnumerator resourceSet2Enumerator = resourceSet2.GetEnumerator();
            while (resourceSet2Enumerator.MoveNext())
            {
                DictionaryEntry entry = (DictionaryEntry) resourceSet2Enumerator.Current;
                if (! resourceTable1.ContainsKey(entry.Key))
                {
                    resourceTable1.Add(entry.Key, entry.Value);
                    changesMade = true;
                }
            }
            return changesMade;
        }
        /// <summary>
        /// ReintegrateResourceSet reintegrates resource entries from an incoming ResourceSet to a master ResourceSet
        /// </summary>
        /// <param name="flattenedMasterResourceSet">A ResourceSet containing the complete set of resource entries
        /// from parent cultures</param>
        /// <param name="masterResourceSet">The original (unflattened) ResourceSet to which incoming resource entries
        /// are added or modified</param>
        /// <param name="incomingResourceSet">The incoming (unflattened) ResourceSet containing added or modified resource entries</param>
        /// <param name="acceptResourceEntry">An AcceptResourceEntry delegate which can be used to filter incoming
        /// resource entries. acceptResourceEntry can be null to ignore this filtering process</param>
        /// <returns>True if changes were made to the master ResourceSet</returns>
        public bool ReintegrateResourceSet(ResourceSet flattenedMasterResourceSet, ResourceSet masterResourceSet, ResourceSet incomingResourceSet,
            AcceptResourceEntry acceptResourceEntry)
        {
            bool changesMade = false;
            Hashtable flattenedMasterResourceTable = GetResourceSetTable(flattenedMasterResourceSet);
            Hashtable masterResourceTable = GetResourceSetTable(masterResourceSet);
            IDictionaryEnumerator incomingResourceSetEnumerator = incomingResourceSet.GetEnumerator();
            while (incomingResourceSetEnumerator.MoveNext())
            {
                DictionaryEntry incomingEntry = (DictionaryEntry) incomingResourceSetEnumerator.Current;
                ResourcesDataNodeInterpreter incomingDataNodeInterpreter = new ResourcesDataNodeInterpreter(incomingEntry.Key.ToString(), incomingEntry.Value);
                if (incomingDataNodeInterpreter.Value != null &&
                    flattenedMasterResourceTable.ContainsKey(incomingDataNodeInterpreter.Name))
                {
                    ResourcesDataNodeInterpreter masterDataNodeInterpreter = new ResourcesDataNodeInterpreter(incomingDataNodeInterpreter.Name, flattenedMasterResourceTable[incomingDataNodeInterpreter.Name]);
                    if (!masterDataNodeInterpreter.Value.Equals(incomingDataNodeInterpreter.Value))
                    {
                        // the incoming entry exists somewhere in the hierarchy and is different to the master copy of the same entry
                        if (acceptResourceEntry == null || acceptResourceEntry(incomingEntry, flattenedMasterResourceTable[incomingEntry.Key]))
                        {
                            if (masterResourceTable.ContainsKey(incomingEntry.Key))
                                masterResourceTable[incomingEntry.Key] = incomingEntry.Value;
                            else
                                masterResourceTable.Add(incomingEntry.Key, incomingEntry.Value);

                            changesMade = true;
                        }
                    }
                }
            }
            return changesMade;
        }
        /// <summary>
        /// CreateDataNode creates a data node given a name and a value
        /// </summary>
        /// <param name="name">The name of the data node</param>
        /// <param name="value">The value of the data node</param>
        /// <returns>A data node representing the supplied data</returns>
        /// <remarks>The exact implementation of the data node is dependant upon the functionality supported by
        /// the ResourceGovernor. A .NET Framework 2.0 ResxResourceGovernor, for example, creates ResXDataNode
        /// objects whereas a ResourcesResourceGovernor simply returns the data value given (because it lacks
        /// support for complex data nodes).</remarks>
        public virtual object CreateDataNode(string name, object value)
        {
            return value;
        }
        /// <summary>
        /// CreateFileRef creates a file reference object given a file name and data type
        /// </summary>
        /// <param name="fileName">The file name to create the file reference for</param>
        /// <param name="typeName">The data type of the data stored in the file</param>
        /// <returns>This implementation throws a NotImplementedException</returns>
        /// <remarks>The implementation of this method is dependant upon the functionality supported by
        /// the ResourceGovernor. A .NET Framework 2.0 ResxResourceGovernor, for example, creates ResXFileRef
        /// objects whereas a ResourcesResourceGovernor throws a NotImplementedException (because it lacks
        /// support for file references). Check the corresponding ResourcesGovernor.SupportsFileReferences
        /// property before calling CreateFileRef to determine if file references are supported.</remarks>
        public virtual object CreateFileRef(string fileName, string typeName)
        {
            throw new NotImplementedException();
        }
        /// <summary>
        /// CreateFileRef creates a file reference object given a file name and data type
        /// </summary>
        /// <param name="fileName">The file name to create the file reference for</param>
        /// <param name="typeName">The data type of the data stored in the file</param>
        /// <param name="textFileEncoding">The Encoding mechanism of the file reference (if the file is a text file)</param>
        /// <returns>This implementation throws a NotImplementedException</returns>
        /// <remarks>The implementation of this method is dependant upon the functionality supported by
        /// the ResourceGovernor. A .NET Framework 2.0 ResxResourceGovernor, for example, creates ResXFileRef
        /// objects whereas a ResourcesResourceGovernor throws a NotImplementedException (because it lacks
        /// support for file references). Check the corresponding ResourcesGovernor.SupportsFileReferences
        /// property before calling CreateFileRef to determine if file references are supported.</remarks>
        public virtual object CreateFileRef(string fileName, string typeName, Encoding textFileEncoding)
        {
            throw new NotImplementedException();
        }
    }
}
