// File           : ResourcesGovernor.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;
using System.Collections.Generic;

namespace Internationalization.ResourceAdministration
{
    /// <summary>
    /// ResourcesGovernor is an abstract base class for managing a complete set of resources
    /// </summary>
    /// <remarks>Take care to notice the distinction between two similarly named classes:
    /// ResourcesGovernor and ResourceGovernor. The former managers a set of resources whereas the latter manages
    /// a single resource. Further note the potential terminology confusion between a set of resources, a resource
    /// and a resource entry. A set of resources would be, for example, the complete set of resx files in a
    /// folder. A resource would be a single resx file and a resource entry would be one entry within a single
    /// resx file.</remarks>
    public abstract class ResourcesGovernor
	{
        /// <summary>
        /// supportsComments is true if the resource governor allows comments to be stored with resource entries
        /// </summary>
        protected bool supportsComments;
        /// <summary>
        /// supportsFileReferences is true if the resource governor allows resource entry values to be file
        /// references
        /// </summary>
        protected bool supportsFileReferences;

        /// <summary>
        /// Constructs a ResourcesGovernor object
        /// </summary>
		public ResourcesGovernor()
		{
            supportsComments = false;
            supportsFileReferences = false;
		}
        /// <summary>
        /// SupportsComments is true if the resource governor allows comments to be stored with resource entries
        /// </summary>
        public bool SupportsComments
        {
            get {return supportsComments;}
            set
            {
                if (value != supportsComments)
                {
                    if (CommentsAreOptional)
                        supportsComments = value;
                    else
                        throw new ArgumentException();
                }
            }
        }
        /// <summary>
        /// SupportsFileReferences is true if the resource governor allows resource entry values to be file
        /// references
        /// </summary>
        public bool SupportsFileReferences
        {
            get {return supportsFileReferences;}
            set
            {
                if (value != supportsFileReferences)
                {
                    if (FileReferencesAreOptional)
                        supportsFileReferences = value;
                    else
                        throw new ArgumentException();
                }
            }
        }
        /// <summary>
        /// CommentsAreOptional is true if the resource governor can optionally work with or without comment support
        /// </summary>
        public bool CommentsAreOptional
        {
            get { return false; }
        }
        /// <summary>
        /// FileReferencesAreOptional is true if the resource governor can optionally work with or without
        /// file reference support
        /// </summary>
        public bool FileReferencesAreOptional
        {
            get { return false; }
        }
        /// <summary>
        /// The absolute path to resource files (if the resources are filed based)
        /// </summary>
        public string ResourceFileAbsolutePath
        {
            get { throw new NotImplementedException(); }
        }
        /// <summary>
        /// The relative path to resource files (if the resources are filed based)
        /// </summary>
        public string ResourceFileRelativePath
        {
            get { throw new NotImplementedException(); }
        }
        /// <summary>
        /// SupportedTypes is an array of all of the Types supported by the resource format
        /// </summary>
        /// <remarks>If SupportedTypes is null there are no restrictions on Types.</remarks>
        public Type[] SupportedTypes
        {
            get { return null; }
        }
        /// <summary>
        /// GetExistingCultures gets an array of CultureInfo objects for resources which exist for the given
        /// base name
        /// </summary>
        /// <param name="baseName">The base name of the resource</param>
        /// <returns>An array of CultureInfo objects for resources which exist for the given base name</returns>
        /// <remarks>For a resx resources governor for a folder containing Form1.resx, Form1.fr.resx,
        /// and Form1.fr-FR.resx files this method returns {invariant CultureInfo, French CultureInfo,
        /// French (France) CultureInfo} when the base name given is "Form1".</remarks>
        public abstract CultureInfo[] GetExistingCultures(string baseName);
        /// <summary>
        /// GetExistingCultures gets an array of CultureInfo objects for resources which exist for the given
        /// base name
        /// </summary>
        /// <param name="baseName">The base name of the resource</param>
        /// <param name="cultureInfo">The CultureInfo for which to restrict the cultures</param>
        /// <returns>An array of CultureInfo objects for resources which exist for the given base name</returns>
        /// <remarks>For a resx resources governor for a folder containing Form1.resx, Form1.fr.resx,
        /// and Form1.fr-FR.resx files this method returns {invariant CultureInfo, French CultureInfo,
        /// French (France) CultureInfo} when the base name given is "Form1".</remarks>
        public CultureInfo[] GetExistingCultures(string baseName, CultureInfo cultureInfo)
        {
            CultureInfo[] allCultures = GetExistingCultures(baseName);
            if (cultureInfo != null && !cultureInfo.Equals(CultureInfo.InvariantCulture))
            {
                List<CultureInfo> matchingCultures = new List<CultureInfo>();
                foreach (CultureInfo allCulture in allCultures)
                {
                    if (!allCulture.Equals(CultureInfo.InvariantCulture) && IsCultureInfoParentOf(cultureInfo, allCulture))
                    {
                        // this culture is or shares the same neutral culture as the culture we are looking for
                        matchingCultures.Add(allCulture);
                    }
                }

                return matchingCultures.ToArray();
            }
            else
            {
                return allCultures;
            }
        }

        /// <summary>
        /// IsCultureInfoParentOf returns true if the parent CultureInfo is equal to or is a parent of the child CultureInfo
        /// </summary>
        /// <param name="parentCultureInfo">The parent CultureInfo</param>
        /// <param name="childCultureInfo">The child CultureInfo</param>
        /// <returns>True if the parent CultureInfo is equal to or is a parent of the child CultureInfo</returns>
        private bool IsCultureInfoParentOf(CultureInfo parentCultureInfo, CultureInfo childCultureInfo)
        {
            while (childCultureInfo != null && !childCultureInfo.Equals(CultureInfo.InvariantCulture))
            {
                if (parentCultureInfo.Name == childCultureInfo.Name)
                {
                    return true;
                }
                childCultureInfo = childCultureInfo.Parent;
            }

            return false;
        }
    }
}
