// File           : ResXResourcesGovernor.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;
using System.Resources;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Reflection;
using System.Text;

namespace Internationalization.ResourceAdministration
{
    /// <summary>
    /// ResXResourcesGovernor manages a complete set of resx file resources
    /// </summary>
    /// <remarks>Take care to notice the distinction between two similarly named classes:
    /// ResXResourcesGovernor and ResXResourceGovernor. The former managers a set of resources whereas the
    /// latter manages a single resource. Further note the potential terminology confusion between a set of
    /// resources, a resource and a resource entry. A set of resources would be the complete set of
    /// resx files in a folder. A resource would be a single resx file and a resource entry would be one entry
    /// within a single resx file.</remarks>
    public class ResXResourcesGovernor : FileBasedResourcesGovernor, IResourcesGovernor
	{
		private string path;

        /// <summary>
        /// Constructs a ResXResourcesGovernor object
        /// </summary>
        /// <param name="path">The path to the resource files</param>
        public ResXResourcesGovernor(string path)
            : base(path, "resx")
		{
            this.path = path;
            supportsComments = true;
            supportsFileReferences = true;
        }
        /// <summary>
        /// GetResourceGovernor gets an IResourceGovernor for the given base name and culture
        /// </summary>
        /// <param name="baseName">The base name of the resource</param>
        /// <param name="cultureInfo">The CultureInfo of the resource</param>
        /// <returns>An IResourceGovernor for the given base name and culture</returns>
        /// <remarks>The IResourceGovernor is implemented by the ResXResourceGovernor class</remarks>
		public virtual IResourceGovernor GetResourceGovernor(string baseName, CultureInfo cultureInfo)
		{
			return new ResXResourceGovernor(path, baseName, cultureInfo);
		}
        /// <summary>
        /// The absolute path to resource files
        /// </summary>
        public new string ResourceFileAbsolutePath
        {
            get
            {
                string relativePath = ResourceFileRelativePath;
                if (relativePath == String.Empty)
                    return path;
                else if (path.EndsWith(Path.DirectorySeparatorChar.ToString()))
                    return path + relativePath;
                else
                    return path + Path.DirectorySeparatorChar + relativePath;
            }
        }
        /// <summary>
        /// The relative path to resource files
        /// </summary>
        public new string ResourceFileRelativePath
        {
            get
            {
                string tempPath = path;
                if (tempPath.EndsWith(Path.DirectorySeparatorChar.ToString()))
                    // strip off the trailing directory separator character
                    tempPath = tempPath.Substring(0, tempPath.Length - 1);
                int lastDirectorySeparatorIndex = tempPath.LastIndexOf(Path.DirectorySeparatorChar);
                if (lastDirectorySeparatorIndex == -1)
                    return String.Empty;

                string lastDirectory = tempPath.Substring(lastDirectorySeparatorIndex + 1);
                if (String.Compare(lastDirectory, "App_GlobalResources", true, CultureInfo.InvariantCulture) == 0)
                    // the path ends with App_GlobalResources so files are placed in the same folder
                    return String.Empty;
                else if (String.Compare(lastDirectory, "App_LocalResources", true, CultureInfo.InvariantCulture) == 0)
                    // the path ends with App_LocalResources so files are placed in the same folder
                    return String.Empty;
                else
                    // the path is a normal path so resx files are placed in a regular folder and therefore
                    // the application is probably a Windows Forms application so resource files are
                    // placed in a Resources folder
                    return "Resources";
            }
        }
    }

    /// <summary>
    /// ResXResourceGovernor manages a resx file resource
    /// </summary>
    public class ResXResourceGovernor : FileBasedResourceGovernor, IResourceGovernor
	{
        /// <summary>
        /// Constructs a ResXResourceGovernor object
        /// </summary>
        /// <param name="path">The path to the resource files</param>
        /// <param name="baseName">The base name of the resource (e.g. "Form1")</param>
        /// <param name="cultureInfo">The CultureInfo which represents the culture of the resource</param>
        public ResXResourceGovernor(string path, string baseName, CultureInfo cultureInfo)
            : base(path, baseName, cultureInfo, "resx")
        {
		}
        /// <summary>
        /// ReadResourceSet reads the resource entries from the source into a ResourceSet
        /// </summary>
        /// <returns>A ResourceSet containing the resource entries read from the source</returns>
        public override ResourceSet ReadResourceSet()
		{
            ResXResourceReader reader = new ResXResourceReader(Filename);
            reader.BasePath = Path;
            reader.UseResXDataNodes = true;
			return ReadResourceSet(reader);
		}
        /// <summary>
        /// WriteResourceSet writes resource entries in a ResourceSet back to the source
        /// </summary>
        /// <param name="resourceSet">The ResourceSet containing the resource entries to write</param>
        public virtual void WriteResourceSet(ResourceSet resourceSet)
		{
			WriteResourceSet(new ResXResourceWriter(Filename), resourceSet);
		}
        /// <summary>
        /// CreateDataNode creates a ResXDataNode given a name and a value
        /// </summary>
        /// <param name="name">The name of the data node</param>
        /// <param name="value">The value of the data node</param>
        /// <returns>A ResXDataNode representing the supplied data</returns>
        public override object CreateDataNode(string name, object value)
        {
            if (value is ResXFileRef)
                return new ResXDataNode(name, (ResXFileRef) value);
            else
                return new ResXDataNode(name, value);
        }
        /// <summary>
        /// CreateFileRef creates a ResXFileRef object for the given a file name and data type
        /// </summary>
        /// <param name="fileName">The file name to create the file reference for</param>
        /// <param name="typeName">The data type of the data stored in the file</param>
        /// <returns>A ResXFileRef object for the given a file name and data type</returns>
        public override object CreateFileRef(string fileName, string typeName)
        {
            return new ResXFileRef(fileName, typeName);
        }
        /// <summary>
        /// CreateFileRef creates a ResXFileRef object for the given a file name and data type
        /// </summary>
        /// <param name="fileName">The file name to create the file reference for</param>
        /// <param name="typeName">The data type of the data stored in the file</param>
        /// <param name="textFileEncoding">The Encoding mechanism of the file reference (if the file is a text file)</param>
        /// <returns>A ResXFileRef object for the given a file name and data type</returns>
        public override object CreateFileRef(string fileName, string typeName, Encoding textFileEncoding)
        {
            return new ResXFileRef(fileName, typeName, textFileEncoding);
        }
    }
}
