// File           : SetOfResourceSets.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;
using System.Globalization;
using System.Resources;

namespace Internationalization.ResourceAdministration
{
	/// <summary>
	/// A SetOfResourceSets is a set of ResourceSet objects for the same basename
	/// </summary>
	/// <remarks>A SetOfResourceSets might include the Form1 ResourceSets for the invariant culture, 
	/// French culture and Spanish culture, for example.</remarks>
	public class SetOfResourceSets
	{
		private string baseName;
		private ArrayList resourceGovernors;
		private ArrayList resourceSets;

        /// <summary>
        /// Constructs a SetOfResourceSets object
        /// </summary>
        /// <param name="baseName">The base name of all of the ResourceSets in the set</param>
		public SetOfResourceSets(string baseName)
		{
			this.baseName = baseName;
			this.resourceGovernors = new ArrayList();
			this.resourceSets = new ArrayList();
		}

		/// <summary>
		/// BaseName is the base name for this set of ResourceSets (e.g. "Form1")
		/// </summary>
		public string BaseName
		{
			get {return baseName;}
			set {baseName = value;}
		}

		/// <summary>
		/// AddResourceSet adds an IResourceGoverner/ResourceSet combination where the ResourceSet is read from the IResourceGovernor
		/// </summary>
		/// <param name="resourceGovernor">The IResourceGovernor for which a ResourceSet should be added</param>
        /// <remarks>The ResourceSet is read using the given IResourceGovernor and then added to the
        /// set of ResourceSets</remarks>
		public void AddResourceSet(IResourceGovernor resourceGovernor)
		{
			ResourceSet resourceSet = resourceGovernor.ReadResourceSet();
			AddResourceSet(resourceGovernor, resourceSet);
		}

        /// <summary>
        /// AddResourceSet adds an IResourceGoverner/ResourceSet combination
        /// </summary>
        /// <param name="resourceGovernor">The IResourceGovernor to which corresponds to the ResourceSet</param>
        /// <param name="resourceSet">The ResourceSet to add to the set of ResourceSets</param>
		public void AddResourceSet(IResourceGovernor resourceGovernor, ResourceSet resourceSet)
		{
			if (resourceGovernors.IndexOf(resourceGovernor) > -1)
				throw new ArgumentException("IResourceGovernor already exists in set", resourceGovernor.BaseName);

			resourceGovernors.Add(resourceGovernor);
			resourceSets.Add(resourceSet);
		}

		/// <summary>
		/// GetResourceGovernor gets the IResourceGovernor for the given CultureInfo
		/// </summary>
		/// <param name="cultureInfo"></param>
		/// <returns></returns>
		public IResourceGovernor GetResourceGovernor(CultureInfo cultureInfo)
		{
			foreach(IResourceGovernor resourceGovernor in resourceGovernors)
			{
				if (resourceGovernor.CultureInfo.Name == cultureInfo.Name)
					return resourceGovernor;
			}
			return null;
		}

		/// <summary>
		/// GetResourceSet gets the ResourceSet which corresponds to the given IResourceGovernor
		/// </summary>
		/// <param name="resourceGovernor"></param>
		/// <returns></returns>
		public ResourceSet GetResourceSet(IResourceGovernor resourceGovernor)
		{
			int index = resourceGovernors.IndexOf(resourceGovernor);
			if (index == -1)
				return null;

			return (ResourceSet) resourceSets[index];
		}

		/// <summary>
		/// GetResourceSet gets the ResourceSet which corresponds to the given CultureInfo
		/// </summary>
		/// <param name="cultureInfo"></param>
		/// <returns></returns>
		public ResourceSet GetResourceSet(CultureInfo cultureInfo)
		{
			IResourceGovernor resourceGovernor = GetResourceGovernor(cultureInfo);
			if (resourceGovernor == null)
				return null;

			int index = resourceGovernors.IndexOf(resourceGovernor);
			if (index == -1)
				return null;

			return (ResourceSet) resourceSets[index];
		}
        /// <summary>
        /// Refresh re-reads all resource set from their original source
        /// </summary>
        public void Refresh()
        {
            for (int resourceGovernorNumber = 0; resourceGovernorNumber < resourceGovernors.Count - 1; resourceGovernorNumber++)
            {
                IResourceGovernor resourceGovernor = (IResourceGovernor) resourceGovernors[resourceGovernorNumber];
                resourceSets[resourceGovernorNumber] = resourceGovernor.ReadResourceSet();
            }
        }
        /// <summary>
        /// AnyExist returns true if any of the resource sets exist
        /// </summary>
        /// <returns>True if any of the resource sets exist</returns>
        public bool AnyExist()
        {
            foreach (ResourceSet resourceSet in resourceSets)
            {
                if (resourceSet != null)
                    // at least one resource set exists
                    return true;
            }
            return false;
        }
    }
}
