﻿// File           : DoNotConcatenateLocalizableStrings.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : February 2012
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using Microsoft.FxCop.Sdk;

namespace GlobalizationRules
{
    /// <summary>
    /// DoNotConcatenateLocalizableStrings is a rule to identify when resource strings are concatenated with other strings
    /// </summary>
    public class DoNotConcatenateLocalizableStrings : BaseGlobalizationRule
    {
        /// <summary>
        /// Constructs an DoNotConcatenateLocalizableStrings object
        /// </summary>
        public DoNotConcatenateLocalizableStrings() :
            base("DoNotConcatenateLocalizableStrings")
        {
        }

        /// <summary>
        /// Check checks members for adherence to the rule
        /// </summary>
        /// <param name="member">The Member to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection Check(Member member)
        {
            Method method = member as Method;
            if (method != null)
            {
                VisitMethod(method);
            }

            return this.Problems;
        }

        /// <summary>
        /// VisitExpression is called by the FxCop engine to visit every expression in scope
        /// </summary>
        /// <param name="expression">The Expression that is being visited</param>
        public override void VisitExpression(Expression expression)
        {
            if (expression is MethodCall)
            {
                MethodCall methodCall = (MethodCall)expression;
                if (methodCall.Operands.Count > 1 && methodCall.Callee is MemberBinding)
                {
                    MemberBinding calleeMemberBinding = (MemberBinding)methodCall.Callee;
                    if (calleeMemberBinding.BoundMember.FullName == "System.String.Concat(System.String,System.String)")
                    {
                        // this expression is a string concatenation
                        foreach (Expression operand in methodCall.Operands)
                        {
                            if (operand is MethodCall)
                            {
                                MethodCall operandMethodCall = (MethodCall)operand;
                                MemberBinding memberBinding = (MemberBinding)operandMethodCall.Callee;
                                if (memberBinding != null)
                                {
                                    if (IsStronglyTypedResourceClass(memberBinding.BoundMember.DeclaringType))
                                    {
                                        // one of the operands in this expression is a StronglyTypedResourceClass
                                        Resolution resolution = GetResolution(new string[] { memberBinding.BoundMember.DeclaringType.FullName });
                                        Problems.Add(new Problem(resolution));
                                    }
                                }
                            }
                        }
                    }
                }
            }

            base.VisitExpression(expression);
        }
    }
}
