﻿// File           : ResourceEntryIsNotUsed.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : February 2012
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using Microsoft.FxCop.Sdk;

namespace GlobalizationRules
{
    /// <summary>
    /// ResourceEntryIsNotUsed is an FxCop rule to identify  resource entries that are not used
    /// </summary>
    /// <remarks>
    /// FxCop is unwilling to reveal types that are decorated with the CompilerGeneratedAttribute. StronglyTypedResourceClasses are decorated with this attribute.
    /// In order for this rule to identify unused resource entries you must manually comment out this attribute from every StronglyTypedResourceClass (by editing
    /// each .Designer.cs file).
    /// 
    /// An alternative way to remove the CompilerGeneratedAttribute is to replace the Visual Studio ResXFileCodeGenerator with a new code generator that does not
    /// include this attribute.
    /// 
    /// Also note that this rule will produce false positives under either of the following conditions:-
    ///    (1) The resource entry is used using ResourceManager.GetString (or GetObject) instead of the strongly type property
    ///    (2) The resource class is public and is used outside of the assembly in which it is declared
    /// 
    /// </remarks>
    public class ResourceEntryIsNotUsed : BaseGlobalizationRule
    {
        /// <summary>
        /// Constructs an ResourceEntryIsNotUsed object
        /// </summary>
        public ResourceEntryIsNotUsed() :
            base("ResourceEntryIsNotUsed")
        {
        }

        /// <summary>
        /// Check checks types for adherence to the rule
        /// </summary>
        /// <param name="type">The TypeNode to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection Check(TypeNode type)
        {
            base.Check(type);

            if (IsStronglyTypedResourceClass(type))
            {
                foreach (Member member in type.Members)
                {
                    if (member is Method && (member.Name.ToString().StartsWith("get_") && 
                        member.Name.ToString() != "get_Culture" && member.Name.ToString() != "get_ResourceManager"))
                    {
                        // the member is a get method of a property
                        Method method = (Method)member;
                        if (CallGraph.CallersFor(method).Count == 0)
                        {
                            // the property's get method has no callers
                            string propertyName = method.Name.ToString().Substring(4);
                            Resolution resolution = GetResolution(new string[] { String.Format("{0}.{1}", type.FullName, propertyName) });
                            Problems.Add(new Problem(resolution));
                        }
                    }
                }
            }

            return this.Problems;
        }
    }
}
