// File           : ResourceStringMissingFromFallbackAssembly.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Resources;
using System.Collections;
using System.IO;
using Microsoft.FxCop.Sdk;

namespace GlobalizationRules
{
	/// <summary>
	/// ResourceStringMissingFromFallbackAssembly checks that all resource entry names referenced in the code
    /// have corresponding entries in the fallback assembly
	/// </summary>
	public class ResourceStringMissingFromFallbackAssembly: BaseGlobalizationRule
	{
        /// <summary>
        /// Constructs a ResourceStringMissingFromFallbackAssembly object
        /// </summary>
		public ResourceStringMissingFromFallbackAssembly(): base("ResourceStringMissingFromFallbackAssembly")
		{
		}
        /// <summary>
        /// Check checks members for adherence to the rule
        /// </summary>
        /// <param name="member">The Member to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection Check(Member member)
		{
			// Don't bother checking Form.InitializeComponent because this code is automatically generated
			// and the resource which is associated with it is also automatically generated and the two
			// are always kept in sync.
			Method method = member as Method;
			if (method == null)
				return null;

			if ( ! (method.Name.Name == "InitializeComponent" && 
				TypeIsSubClassOf(method.DeclaringType, "System.Windows.Forms.Form")))
			{
				// This method looks for the following pattern in the IL:-
				// IL_0011:  ldstr      "Hello"
				// IL_0016:  callvirt   instance string [mscorlib]System.Resources.ResourceManager::GetString(string)
				// It then checks to see if the string name ("Hello") is present in the fallback assembly.
				for(int instructionNumber = 0; instructionNumber < method.Instructions.Count; instructionNumber++)
				{
					Instruction instruction = method.Instructions[instructionNumber];
					if (instruction.OpCode == OpCode.Ldstr && instructionNumber + 1 < method.Instructions.Count)
					{
						// This instruction is a "load string" instruction and it isn't the last instruction 
						// in the method.
						string loadString = instruction.Value.ToString();
						Instruction nextInstruction = method.Instructions[instructionNumber + 1];
						if (nextInstruction.OpCode == OpCode.Callvirt && nextInstruction.Value is Method)
						{
							// The load string instruction is followed by a call to a virtual method.
							Method nextInstructionMethod = (Method) nextInstruction.Value;
							if (nextInstructionMethod.FullName == 
								"System.Resources.ResourceManager.GetString(System.String)")
							{
								// We have found a call to ResourceManager.GetString. Now let's look for it
								// in the fallback assembly.
								Resource resource;
								// Big assumption here:- The resource file associated with the class
								// has been given the same name as the class but with a "Resources" suffix.
								// So "WindowsApplication1.Form1.cs" has an associated resource file called
								// "WindowsApplication1.Form1Resources.resx".
								if (GetResource(method.DeclaringType, out resource, "Resources"))
								{
									if (! KeyExistsInResource(resource, loadString))
									{
										Resolution resolution = GetResolution(new string[] {method.Name.Name, loadString});
										Problems.Add(new Problem(resolution));
										return Problems;
									}
								}
							}
						}
					}
				}
			}
			return base.Check(member);
		}
        /// <summary>
        /// KeyExistsInResource returns true if the resource entry name exists in the resource
        /// </summary>
        /// <param name="resource">The Resource to check the name for</param>
        /// <param name="keyName">The resource entry name</param>
        /// <returns>True if the resource entry name exists in the resource</returns>
		protected virtual bool KeyExistsInResource(Resource resource, string keyName)
		{
			ResourceReader reader = new ResourceReader(new System.IO.MemoryStream(resource.Data));
			try
			{
				IDictionaryEnumerator enumerator = reader.GetEnumerator();
				while (enumerator.MoveNext())
				{
					if (enumerator.Key.ToString() == keyName)
						return true;
				}
			}
			finally
			{
				reader.Close();
			}
			return false;
		}
	}
}
