// File           : BaseGlobalizationRuleForASPGeneratedAssemblies.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : March 2008
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using Microsoft.FxCop.Sdk;
using System.Collections.Generic;

namespace GlobalizationRulesForASPGeneratedAssemblies
{
	/// <summary>
	/// BaseGlobalizationRuleForASPGeneratedAssemblies is an FxCop rule base class for rules that analyze
    /// assemblies generated from .aspx files
	/// </summary>
	public abstract class BaseGlobalizationRuleForASPGeneratedAssemblies: BaseIntrospectionRule
	{
        private bool processingRule = true;
        private List<Method> buildMethods;

        /// <summary>
        /// Constructs a BaseGlobalizationRuleForASPGeneratedAssemblies object
        /// </summary>
        /// <param name="name">The name of the rule</param>
		public BaseGlobalizationRuleForASPGeneratedAssemblies(string name)
            : base(name, "GlobalizationRulesForASPGeneratedAssemblies.RuleData", typeof(BaseGlobalizationRuleForASPGeneratedAssemblies).Assembly)
		{
		}
        /// <summary>
        /// ProcessingRule is a flag indicating whether the rule is being processed
        /// </summary>
        /// <remarks>The GetBuildMethods method has a side effect of firing Visit methods and to protect
        /// against this sub-classes must check the ProcessingRule flag to see whether they can perform
        /// their own processing yet</remarks>
        public bool ProcessingRule
        {
            get { return processingRule; }
        }
        /// <summary>
        /// GetBuildMethods get a List of Methods that begin with "__BuildControl" i.e. the methods generated from a .aspx page
        /// </summary>
        /// <param name="type">The type for which build methods should be returned</param>
        /// <returns>A List of Methods that begin with "__BuildControl"</returns>
        protected List<Method> GetBuildMethods(TypeNode type)
        {
            processingRule = false;
            buildMethods = new List<Method>();
            Identifier identifier = Identifier.For("__BuildControlTree");
            Method method = type.GetMethod(identifier, type);
            if (method != null)
            {
                // this type has a "__BuildControlTree" method
                // now open the __BuildControlTree method and see what other "__BuildControl" methods it calls
                VisitMethod(method);
            }
            processingRule = true;
            return buildMethods;
        }
        /// <summary>
        /// VisitMethodCall is called for each call to a method
        /// </summary>
        /// <param name="call">The MethodCall of the method that is being called</param>
        public override void VisitMethodCall(MethodCall call)
        {
            if (!processingRule)
            {
                if (call != null)
                {
                    MemberBinding memberBinding = call.Callee as MemberBinding;
                    if (memberBinding != null)
                    {
                        Method method = memberBinding.BoundMember as Method;
                        if (method != null &&
                            method.Name.Name.StartsWith("__BuildControl"))
                        {
                            buildMethods.Add(method);
                            VisitMethod(method);
                        }
                    }
                }
            }
            base.VisitMethodCall(call);
        }
        /// <summary>
        /// GetWebFormName gets the name of the WebForm from a type
        /// </summary>
        /// <param name="typeNode">The type to get the WebForm name from</param>
        /// <returns>The name of the WebForm</returns>
        protected string GetWebFormName(TypeNode typeNode)
        {
            string formName = typeNode.FullName;
            if (formName.StartsWith("ASP."))
                formName = formName.Substring(4);

            if (formName.EndsWith("_aspx"))
                formName = formName.Substring(0, formName.Length - 5) + ".aspx";
            else if (formName.EndsWith("_ascx"))
                formName = formName.Substring(0, formName.Length - 5) + ".ascx";

            return formName;
        }
        /// <summary>
        /// GetControlName gets the name of the control given the name of a build method
        /// </summary>
        /// <param name="buildMethodName">The name of the buid method e.g. __BuildControlLabel1</param>
        /// <returns>The name of the control e.g. Label1</returns>
        protected string GetControlName(string buildMethodName)
        {
            if (buildMethodName.StartsWith("__BuildControl"))
                return buildMethodName.Substring(14);

            return buildMethodName;
        }
        /// <summary>
        /// IsInstructionSpecificMethodCall returns true if the instruction is a call to a specific method
        /// </summary>
        /// <param name="instruction">The instruction to test</param>
        /// <param name="methodName">The method to test the instruction for</param>
        /// <returns>True if the instruction is a call to the given method</returns>
        protected bool IsInstructionSpecificMethodCall(Instruction instruction, string methodName)
        {
            return IsInstructionSpecificMethodCall(instruction, new string[] { methodName });
        }
        /// <summary>
        /// IsInstructionSpecificMethodCall returns true if the instruction is a call to a specific method
        /// </summary>
        /// <param name="instruction">The instruction to test</param>
        /// <param name="methodNames">An array of methods to test the instruction for</param>
        /// <returns>True if the instruction is a call to one of the given methods</returns>
        protected bool IsInstructionSpecificMethodCall(Instruction instruction, string[] methodNames)
        {
            if ((instruction.OpCode == OpCode.Callvirt || instruction.OpCode == OpCode.Call)
                && instruction.Value is Method)
            {
                Method instructionMethod = (Method)instruction.Value;
                foreach (string methodName in methodNames)
                {
                    if (instructionMethod.FullName.StartsWith(methodName + "("))
                        return true;
                }
                return false;
            }
            return false;
        }
        /// <summary>
        /// IsInstructionNewObject returns true if the instruction creates a new object of the given class name
        /// </summary>
        /// <param name="instruction">The instruction to test</param>
        /// <param name="className">The name of the class to compare the instruction with</param>
        /// <returns>True if the instruction creates a new object of the given class name</returns>
        protected bool IsInstructionNewObject(Instruction instruction, string className)
        {
            return IsInstructionNewObject(instruction, new string[] { className });
        }
        /// <summary>
        /// IsInstructionNewObject returns true if the instruction creates a new object of one of the given class names
        /// </summary>
        /// <param name="instruction">The instruction to test</param>
        /// <param name="classNames">An array of the names of the classes to compare the instruction with</param>
        /// <returns>True if the instruction creates a new object of one of the given class names</returns>
        protected bool IsInstructionNewObject(Instruction instruction, string[] classNames)
        {
            if (instruction.OpCode == OpCode.Newobj && instruction.Value is InstanceInitializer)
            {
                InstanceInitializer instructionInstanceInitializer = (InstanceInitializer)instruction.Value;
                foreach (string className in classNames)
                {
                    if (instructionInstanceInitializer.FullName.StartsWith(className + ".#ctor"))
                        return true;
                }
                return false;
            }
            return false;
        }
    }
}
