// File           : ColumnTextShouldBeLocalizable.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : March 2008
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Specialized;
using Microsoft.FxCop.Sdk;
using System.Collections.Generic;

namespace GlobalizationRulesForASPGeneratedAssemblies
{
    /// <summary>
    /// ColumnTextShouldBeLocalizable checks that the text in a BoundColumn or TemplateColumn is not hard wired
    /// </summary>
    public class ColumnTextShouldBeLocalizable : BaseGlobalizationRuleForASPGeneratedAssemblies
    {
        /// <summary>
        /// Constructs a ColumnTextShouldBeLocalizable object
        /// </summary>
        public ColumnTextShouldBeLocalizable()
            : base("ColumnTextShouldBeLocalizable")
        {
        }
        /// <summary>
        /// Check checks types in an assembly
        /// </summary>
        /// <param name="type">The TypeNode being checked</param>
        /// <returns>A ProblemCollection of Problems</returns>
        public override ProblemCollection Check(TypeNode type)
        {
            List<Method> methods = GetBuildMethods(type);
            if (methods.Count > 0)
            {
                foreach (Method method in methods)
                {
                    for (int instructionNumber = 0; instructionNumber < method.Instructions.Count; instructionNumber++)
                    {
                        Instruction instruction = method.Instructions[instructionNumber];
                        if (instruction.OpCode == OpCode.Ldstr && instructionNumber + 1 < method.Instructions.Count)
                        {
                            // This instruction is a "load string" instruction and it isn't the last instruction 
                            // in the method.
                            Instruction nextInstruction = method.Instructions[instructionNumber + 1];
                            if (IsInstructionSpecificMethodCall(nextInstruction, new string[]
                                {
                                    "System.Web.UI.WebControls.DataGridColumn.set_HeaderText",
                                    "System.Web.UI.WebControls.DataGridColumn.set_FooterText"
                                    }))
                            {
                                string loadString = instruction.Value.ToString();
                                if (IsStringLiteral(loadString))
                                {
                                    // The string is a literal string and it is suspiciously like a localizable
                                    // string and it is not already in the list of ignored strings and it is
                                    // not passed to one of the exempt constructors or methods.
                                    Resolution resolution = GetResolution(new string[] { loadString, method.Name.Name, GetWebFormName(method.DeclaringType) });
                                    Problems.Add(new Problem(resolution));
                                }
                            }
                        }
                    }
                }
                return Problems;
            }
            return base.Check(type);
        }

        private bool IsStringLiteral(string stringLiteral)
        {
            // the string is considered to be a string literal if it doesn't start with "<%="
            // because "<%= GetLocalResourceObject("ResourceName").ToString() %>" starts with "<%="
            return !stringLiteral.StartsWith("<%=");
        }
        //public override void VisitMemberBinding(MemberBinding memberBinding)
        //{
        //    if (ProcessingRule)
        //    {
        //        if (memberBinding.BoundMember.FullName == 
        //            "System.Web.UI.WebControls.DataGridColumn.set_HeaderText(System.String)" && 
        //            memberBinding.TargetObject is Local)
        //        {
        //            Local local = (Local)memberBinding.TargetObject;
        //            if (local.Type.FullName == "System.Web.UI.WebControls.BoundColumn")
        //            {
        //                // but what is being assigned to this member ?
        //            }
        //        }
        //    }
        //    base.VisitMemberBinding(memberBinding);
        //}
    }
}
