// File           : DoNotUseAbsolutePositioning.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : March 2008
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Specialized;
using Microsoft.FxCop.Sdk;
using System.Collections.Generic;

namespace GlobalizationRulesForASPGeneratedAssemblies
{
    /// <summary>
    /// DoNotUseAbsolutePositioning checks that controls do not use absolute positioning
    /// </summary>
    public class DoNotUseAbsolutePositioning: BaseGlobalizationRuleForASPGeneratedAssemblies
    {
        private bool controlHasAbsolutePositioning;

        /// <summary>
        /// Constructs a DoNotUseAbsolutePositioning object
        /// </summary>
        public DoNotUseAbsolutePositioning(): base("DoNotUseAbsolutePositioning")
        {
        }
        /// <summary>
        /// Check checks types in an assembly
        /// </summary>
        /// <param name="type">The TypeNode being checked</param>
        /// <returns>A ProblemCollection of Problems</returns>
        public override ProblemCollection Check(TypeNode type)
        {
            List<Method> methods = GetBuildMethods(type);
            if (methods.Count > 0)
            {
                foreach (Method method in methods)
                {
                    controlHasAbsolutePositioning = false;
                    VisitMethod(method);
                    if (controlHasAbsolutePositioning)
                    {
                        Resolution resolution = GetResolution(new string[] { GetWebFormName(method.DeclaringType), GetControlName(method.Name.Name) });
                        Problems.Add(new Problem(resolution));
                    }
                }
                return Problems;
            }
            return base.Check(type);
        }
        /// <summary>
        /// VisitMethodCall is called for each call to a method
        /// </summary>
        /// <param name="call">The MethodCall of the method that is being called</param>
        public override void VisitMethodCall(MethodCall call)
        {
            if (ProcessingRule)
            {
                if (call != null)
                {
                    MemberBinding memberBinding = call.Callee as MemberBinding;
                    if (memberBinding != null)
                    {
                        Method method = memberBinding.BoundMember as Method;
                        if (method != null &&
                            method.DeclaringType.FullName == "System.Web.UI.IAttributeAccessor" &&
                            method.Name.Name == "SetAttribute" &&
                            call.Operands.Count > 1)
                        {
                            Literal literalFirstParameter = call.Operands[0] as Literal;
                            Literal literalSecondParameter = call.Operands[1] as Literal;
                            if (literalFirstParameter != null && literalSecondParameter != null &&
                                literalFirstParameter.Value.ToString() == "style")
                            {
                                // this is a call to Control.SetAttribute which passes "style" for the first parameter and
                                // a literal string value as the second parameter e.g. 
                                // button1.SetAttribute("style", "Z-INDEX: 101; LEFT: 24px; POSITION: absolute; TOP: 24px");
                                string style = literalSecondParameter.Value.ToString().ToUpper();
                                if (style.IndexOf("POSITION: ABSOLUTE") > -1)
                                    controlHasAbsolutePositioning = true;
                            }
                        }
                    }
                }
            }
            base.VisitMethodCall(call);
        }
        /// <summary>
        /// TypeIsSubClassOf returns true if a type is a subclass of a given type
        /// </summary>
        /// <param name="type">The TypeNode of the given type</param>
        /// <param name="typeName">The name of the type to compare with</param>
        /// <returns>True if a type is a subclass of a given type</returns>
        protected virtual bool TypeIsSubClassOf(TypeNode type, string typeName)
        {
            if (type.FullName == typeName)
                return true;
            else if (type.BaseType == null)
                return false;
            else
                return TypeIsSubClassOf(type.BaseType, typeName);
        }
    }
}
