// File           : DoNotUseHtmlControls.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : March 2008
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Specialized;
using Microsoft.FxCop.Sdk;
using System.Collections.Generic;

namespace GlobalizationRulesForASPGeneratedAssemblies
{
    /// <summary>
    /// DoNotUseHtmlControls checks that HTML controls are not used on a page (because they are not localizable)
    /// </summary>
    public class DoNotUseHtmlControls : BaseGlobalizationRuleForASPGeneratedAssemblies
    {
        private StringCollection literalStrings;

        /// <summary>
        /// Constructs a DoNotUseHtmlControls object
        /// </summary>
        public DoNotUseHtmlControls()
            : base("DoNotUseHtmlControls")
        {
        }
        /// <summary>
        /// Check checks types in an assembly
        /// </summary>
        /// <param name="type">The TypeNode being checked</param>
        /// <returns>A ProblemCollection of Problems</returns>
        public override ProblemCollection Check(TypeNode type)
        {
            List<Method> methods = GetBuildMethods(type);
            if (methods.Count > 0)
            {
                Method method = GetFirstHtmlFormMethod(methods);
                if (method != null)
                {
                    literalStrings = new StringCollection();
                    VisitMethod(method);
                    if (literalStrings.Count > 0)
                    {
                        foreach (string literalString in literalStrings)
                        {
                            List<string> htmlControls = GetHtmlControls(literalString);
                            foreach (string htmlControl in htmlControls)
                            {
                                Resolution resolution = GetResolution(
                                    new string[] { GetWebFormName(method.DeclaringType), htmlControl });
                                Problems.Add(new Problem(resolution));
                            }
                        }
                    }
                }
                return Problems;
            }
            return base.Check(type);
        }
        private Method GetFirstHtmlFormMethod(List<Method> methods)
        {
            foreach (Method method in methods)
            {
                if (method.ReturnType.FullName == "System.Web.UI.HtmlControls.HtmlForm")
                    return method;
            }
            return null;
        }
        /// <summary>
        /// VisitConstruct is called for each construction of a new object
        /// </summary>
        /// <param name="cons">A Construct object</param>
        public override void VisitConstruct(Construct cons)
        {
            if (ProcessingRule)
            {
                if (cons != null)
                {
                    MemberBinding memberBinding = cons.Constructor as MemberBinding;
                    if (memberBinding != null)
                    {
                        InstanceInitializer instanceInitializer =
                            memberBinding.BoundMember as InstanceInitializer;
                        if (instanceInitializer != null &&
                            instanceInitializer.DeclaringType.FullName == "System.Web.UI.LiteralControl" &&
                            cons.Operands.Count > 0 &&
                            cons.Operands[0].Type.FullName == "System.String")
                        {
                            Literal literal = cons.Operands[0] as Literal;
                            if (literal != null)
                            {
                                // the LiteralControl constructor has been called with a string parameter
                                string literalText = ExtractControlCharacters(literal.Value.ToString());
                                if (literalText.Length > 0)
                                    literalStrings.Add(literalText);
                            }
                        }
                    }
                }
            }
        }
        /// <summary>
        /// ExtractControlCharacters extracts control characters from a string
        /// </summary>
        /// <param name="text">A string containing control characters</param>
        /// <returns>A string that does not contain control characters</returns>
        protected virtual string ExtractControlCharacters(string text)
        {
            return text.Replace("\t", String.Empty).Replace("\n", String.Empty).
                Replace("\r", String.Empty).Replace("&nbsp;", String.Empty).Trim();
        }
        /// <summary>
        /// GetHtmlControls gets a list of strings of HTML controls in a string
        /// </summary>
        /// <param name="text">A string containing HTML controls</param>
        /// <returns>A List of strings of HTML controls</returns>
        protected virtual List<string> GetHtmlControls(string text)
        {
            // TODO: Rewrite this method to be more efficient and more intelligent
            List<string> htmlControls = new List<string>();
            int indexOpeningTag = text.IndexOf("<");
            while (indexOpeningTag > -1)
            {
                int indexClosingTag = text.IndexOf(">", indexOpeningTag);
                if (indexClosingTag == -1)
                    break;

                string htmlControl = text.Substring(indexOpeningTag + 1);
                if (htmlControl.Length > 0 && htmlControl[0] != (char)47)
                {
                    // htmlControl does not start with a forward slash
                    int spaceOrBracketIndex = IndexOfArray(htmlControl, new char[] { ' ', '>' });
                    if (spaceOrBracketIndex > -1)
                    {
                        htmlControl = htmlControl.Substring(0, spaceOrBracketIndex);
                        if (!String.IsNullOrEmpty(htmlControl))
                        {
                            string upperHtmlControl = htmlControl.ToUpper();
                            // there is no ASP.NET Server control equivalent to a reset input
                            // field so don't count it as an error because it cannot be fixed so easily
                            bool isResetInput = (upperHtmlControl.StartsWith("INPUT") && 
                                text.ToUpper().IndexOf(@"TYPE=""RESET""") > -1);

                            List<string> htmlControlsWithNoServerSideEquivalents = new List<string>(
                                new string[] {"DIV", "BR", "P", "SPAN", "!--", "H1", "H2", "H3", "FIELDSET",
                                "LEGEND"});

                            if (htmlControlsWithNoServerSideEquivalents.IndexOf(upperHtmlControl) == -1
                                && ! isResetInput)
                                htmlControls.Add(htmlControl);
                        }
                    }
                }

                text = text.Substring(indexClosingTag + 1);

                indexOpeningTag = text.IndexOf("<");
            }
            return htmlControls;
        }
        private int IndexOfArray(string text, char[] searchChars)
        {
            int index = -1;
            foreach (char searchChar in searchChars)
            {
                int charIndex = text.IndexOf(searchChar);
                if (charIndex != -1)
                {
                    if (index == -1)
                        index = charIndex;
                    else if (charIndex < index)
                        index = charIndex;
                }
            }
            return index;
        }
    }
}
