// File           : OutputCacheMustBeCultureSensitive.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : March 2008
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Specialized;
using Microsoft.FxCop.Sdk;

namespace GlobalizationRulesForASPGeneratedAssemblies
{
    /// <summary>
    /// OutputCacheMustBeCultureSensitive checks that the Page's OutputCache is culture-sensitive
    /// </summary>
    public class OutputCacheMustBeCultureSensitive: BaseGlobalizationRuleForASPGeneratedAssemblies
    {
        /// <summary>
        /// Constructs an OutputCacheMustBeCultureSensitive object
        /// </summary>
        public OutputCacheMustBeCultureSensitive(): base("OutputCacheMustBeCultureSensitive")
        {
        }
        /// <summary>
        /// Check checks types in an assembly
        /// </summary>
        /// <param name="type">The TypeNode being checked</param>
        /// <returns>A ProblemCollection of Problems</returns>
        public override ProblemCollection Check(TypeNode type)
        {
            // Given the following outputcache declaration in a .aspx file:-
            //
            // <% @Outputcache Duration=20 VaryByParam=none VaryByCustom="CurrentUICulture"%>
            //
            // the resulting C# generated in the page's constructor is:-
            //
            // OutputCacheParameters outputCacheSettings = new OutputCacheParameters();
            // outputCacheSettings.Duration = 20;
            // outputCacheSettings.VaryByCustom = "CurrentUICulture";
            // outputCacheSettings.VaryByParam = null;
            //
            // the resulting IL generated in the page's constructor is:-
            //
            // IL_0066:  nop
            // IL_0067:  newobj     instance void [System.Web]System.Web.UI.OutputCacheParameters::.ctor()
            // IL_006c:  stloc.2
            // IL_006d:  ldloc.2
            // IL_006e:  ldc.i4.s   20
            // IL_0070:  callvirt   instance void [System.Web]System.Web.UI.OutputCacheParameters::set_Duration(int32)
            // IL_0075:  nop
            // IL_0076:  ldloc.2
            // IL_0077:  ldstr      "CurrentUICulture"
            // IL_007c:  callvirt   instance void [System.Web]System.Web.UI.OutputCacheParameters::set_VaryByCustom(string)
            // IL_0081:  nop
            // IL_0082:  ldloc.2
            // IL_0083:  ldnull
            // IL_0084:  callvirt   instance void [System.Web]System.Web.UI.OutputCacheParameters::set_VaryByParam(string)
            //
            InstanceInitializer constructor = type.GetConstructor();
            if (constructor != null)
            {
                bool outputCacheParametersConstructorDiscovered = false;
                bool varyByCustomPropertySet = false;
                for (int instructionNumber = 0; instructionNumber < constructor.Instructions.Count; instructionNumber++)
                {
                    Instruction instruction = constructor.Instructions[instructionNumber];
                    if (outputCacheParametersConstructorDiscovered)
                    {
                        if (instruction.OpCode == OpCode.Callvirt && instruction.Value is Method)
                        {
                            Method method = (Method)instruction.Value;
                            if (method.FullName == "System.Web.UI.OutputCacheParameters.set_VaryByCustom(System.String)")
                            {
                                // we have found what we are looking for - stop looking
                                varyByCustomPropertySet = true;
                                break;
                            }
                        }
                    }
                    else if (IsInstructionNewObject(instruction, "System.Web.UI.OutputCacheParameters"))
                    {
                        outputCacheParametersConstructorDiscovered = true;
                    }
                }
                if (outputCacheParametersConstructorDiscovered && !varyByCustomPropertySet)
                {
                    // An OutputCacheParameters object was created but there was no setting of the VaryByCustom property
                    Resolution resolution = GetResolution(new string[] { GetWebFormName(constructor.DeclaringType) });
                    Problems.Add(new Problem(resolution));
                    return Problems;
                }
            }
            return base.Check(type);
        }
    }
}
