// File           : ControlCharactersEmbeddedInResourceString.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Resources;
using System.Collections;
using System.IO;
using Microsoft.Cci;
using Microsoft.FxCop.Sdk;
using Microsoft.FxCop.Sdk.Introspection;

namespace GlobalizationRules
{
	/// <summary>
	/// ControlCharactersEmbeddedInResourceString checks that control characters are not embedded in resource
    /// strings
	/// </summary>
	public class ControlCharactersEmbeddedInResourceString: BaseGlobalizationRule
	{
        /// <summary>
        /// Constructs a ControlCharactersEmbeddedInResourceString object
        /// </summary>
		public ControlCharactersEmbeddedInResourceString(): base("ControlCharactersEmbeddedInResourceString")
		{
		}
        /// <summary>
        /// Check checks resources for adherence to the rule
        /// </summary>
        /// <param name="resource">The Resource to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection Check(Resource resource)
		{
			ResourceReader reader = new ResourceReader(new System.IO.MemoryStream(resource.Data));
			try
			{
				IDictionaryEnumerator enumerator = reader.GetEnumerator();
				while (enumerator.MoveNext())
				{
					if (ResourceStringHasEmbeddedControlCharacters(enumerator.Value))
					{
						Resolution resolution = GetResolution(new string[] {resource.Name.ToString(), enumerator.Value.ToString()});
						Problems.Add(new Problem(resolution));
						return Problems;
					}
				}
			}
			finally
			{
				reader.Close();
			}
			return base.Check(resource);
		}
        /// <summary>
        /// ResourceStringHasEmbeddedControlCharacters returns true if the resource string has embedded control
        /// characters in it
        /// </summary>
        /// <param name="resourceObject">The resource entry</param>
        /// <returns>True if the resource string has embedded control characters in it</returns>
        /// <example>"You have 5 credits\nPress any key" has a "\n" character in it.</example>
		protected virtual bool ResourceStringHasEmbeddedControlCharacters(object resourceObject)
		{
			if (resourceObject is string)
			{
				string resourceString = (string) resourceObject;
				int startIndex = 0;
				int foundCharacter;
				while ((foundCharacter = resourceString.IndexOf('\\', startIndex)) > -1)
				{
					if (foundCharacter < resourceString.Length && resourceString[foundCharacter + 1] != '\\')
						// The '\' character that was found was not the last character and was not followed by 
						// another '\'. Therefore the '\' character is the start of a control character.
						return true;
                    startIndex = foundCharacter + 1;
				}
			}
			return false;
		}
	}
}
