// File           : DateTimeToStringShouldNotUseCultureSpecificFormat.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Specialized;
using System.Text;
using Microsoft.Cci;
using Microsoft.FxCop.Sdk;
using Microsoft.FxCop.Sdk.Introspection;

namespace GlobalizationRules
{
	/// <summary>
	/// DateTimeToStringShouldNotUseCultureSpecificFormat checks against the use of a culture-specific date/time
    /// format in DateTime.ToString()
	/// </summary>
	public class DateTimeToStringShouldNotUseCultureSpecificFormat: BaseGlobalizationRule
	{
        /// <summary>
        /// Constructs a DateTimeToStringShouldNotUseCultureSpecificFormat object
        /// </summary>
		public DateTimeToStringShouldNotUseCultureSpecificFormat(): 
			base("DateTimeToStringShouldNotUseCultureSpecificFormat")
		{
		}
        /// <summary>
        /// Check checks members for adherence to the rule
        /// </summary>
        /// <param name="member">The Member to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection Check(Member member)
		{
			// This method looks for the following pattern in the IL:-
			// IL_0011:  ldstr      "MM/dd/yy"
			// IL_0016:  call       instance string [mscorlib]System.DateTime::ToString(string)
			// The search it performed "back to front". In other words we search firstly for the call
			// to DateTime.ToString() and then we check the preceeding ldstr instruction.
			// Start checking instructions from the *second* instruction (not the first). This is
			// because if the DateTime.ToString() is not preceeded by a ldstr instruction then we are not
			// interested in it.
			Method method = member as Method;
			if (method == null)
				return null;

			for(int instructionNumber = 1; instructionNumber < method.Instructions.Length; instructionNumber++)
			{
				Microsoft.Cci.Instruction instruction = method.Instructions[instructionNumber];
				if (instruction.OpCode == OpCode.Call && instruction.Value is Microsoft.Cci.Method)
				{
					Microsoft.Cci.Method instructionMethod = (Microsoft.Cci.Method) instruction.Value;
					if (instructionMethod.FullName == "System.DateTime.ToString(System.String)")
					{
						// This is a call to DateTime.ToString.
						// Check to see if the previous instruction is a ldstr.
						Microsoft.Cci.Instruction previousInstruction = method.Instructions[instructionNumber - 1];
						if (previousInstruction.OpCode == OpCode.Ldstr)
						{
							// This instruction is a "load string" instruction.
							string loadString = previousInstruction.Value.ToString();
							if (DateTimeFormatStringIsCultureSpecific(loadString))
							{
								Resolution resolution = GetResolution(new string[] {loadString});
								Problems.Add(new Problem(resolution));
								return Problems;
							}
						}
					}
				}
			}
			return base.Check(member);
		}
        /// <summary>
        /// DateTimeFormatStringIsCultureSpecific returns true if the date/time format string is culture-specific
        /// </summary>
        /// <param name="format">The string format to test</param>
        /// <returns>True if the date/time format string is culture-specific</returns>
        /// <remarks>A format is determined to be culture specific if it is not one of the culture-sensitive
        /// formats</remarks>
		protected virtual bool DateTimeFormatStringIsCultureSpecific(string format)
		{
			if (format.Length == 1)
			{
				string[] cultureSensitiveFormats = new string[]
				{"d", "D", "f", "F", "g", "G", "m", "M", "r", "R", "s", "t", "T", "u", "U", "y", "Y"};
				foreach(string cultureSensitiveFormat in cultureSensitiveFormats)
				{
					if (format == cultureSensitiveFormat)
						return false;
				}
			}
			return true;
		}
	}
}
