// File           : NewObjectRule.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Specialized;
using System.Text;
using Microsoft.Cci;
using Microsoft.FxCop.Sdk;
using Microsoft.FxCop.Sdk.Introspection;

namespace GlobalizationRules
{
	/// <summary>
	/// NewObjectRule is a rule which checks for the creation of a specific type of object
	/// </summary>
	public abstract class NewObjectRule: BaseGlobalizationRule
	{
		private string[] searchClassNames;
        private StringCollection foundClassNames;
        /// <summary>
        /// Constructs a NewObjectRule object
        /// </summary>
        /// <param name="name">The name of the rule</param>
        /// <param name="searchClassNames">A string array of class names to search for</param>
		public NewObjectRule(string name, string[] searchClassNames): base(name)
		{
			this.searchClassNames = searchClassNames;
		}
        /// <summary>
        /// Check checks members for adherence to the rule
        /// </summary>
        /// <param name="member">The Member to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection Check(Member member)
        {
            Method method = member as Method;
            if (method == null)
                return null;

            foundClassNames = new StringCollection();
            VisitMethod(method);
			if (foundClassNames.Count == 0)
				return base.Check(member);
			else
			{
				StringBuilder classNames = new StringBuilder(foundClassNames[0]);
				for(int classNameNumber = 1; classNameNumber < foundClassNames.Count; classNameNumber++)
				{
					classNames.Append(", ");
					classNames.Append(foundClassNames[classNameNumber]);
				}
                foundClassNames = null;
				Resolution resolution = GetResolution(new string[] {method.Name.Name, classNames.ToString()});
				Problems.Add(new Problem(resolution));
				return Problems;
			}
		}
        /// <summary>
        /// VisitConstruct is called to visit instance initializers
        /// </summary>
        /// <param name="cons">The Construct object to search</param>
        /// <returns>An Expression</returns>
        public override Expression VisitConstruct(Construct cons)
        {
            if (cons != null)
            {
                MemberBinding memberBinding = cons.Constructor as MemberBinding;
                if (memberBinding != null)
                {
                    InstanceInitializer instanceInitializer = memberBinding.BoundMember as InstanceInitializer;
                    if (instanceInitializer != null)
                    {
                        foreach(string searchClassName in searchClassNames)
                        {
                            if (instanceInitializer.DeclaringType.FullName == searchClassName && foundClassNames.IndexOf(searchClassName) == -1)
                                foundClassNames.Add(searchClassName);
                        }
                    }
                }
            }
            return base.VisitConstruct (cons);
        }
    }
}
