// File           : ObjectNotProvidedByProvider.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using Microsoft.Cci;
using Microsoft.FxCop.Sdk;
using Microsoft.FxCop.Sdk.Introspection;

namespace GlobalizationRules
{
    /// <summary>
    /// ObjectNotProvidedByProvider is an abstract rule which checks for the creation of objects which should be
    /// provided by a provider
    /// </summary>
    public abstract class ObjectNotProvidedByProvider: BaseGlobalizationRule
    {
        private string className;
        private string providerClassName;
        private bool classFound;

        /// <summary>
        /// Constructs an ObjectNotProvidedByProvider object
        /// </summary>
        /// <param name="name">The name of the rule</param>
        /// <param name="className">The name of the class to look for</param>
        /// <param name="providerClassName">The name of the class which should provide objects of this class</param>
        public ObjectNotProvidedByProvider(string name, string className, string providerClassName): base(name)
        {
            this.className = className;
            this.providerClassName = providerClassName;
        }
        /// <summary>
        /// Check checks members for adherence to the rule
        /// </summary>
        /// <param name="member">The Member to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection Check(Member member)
        {
            Method method = member as Method;
            if (method == null)
                return null;

            if (! TypeIsSubClassOf(method.DeclaringType, providerClassName))
            {
                // The method is not a method of the provider class.
                // There is no point in checking the provider class itself because the purpose of the
                // provider is to create new objects of the original class)
                classFound = false;
                VisitMethod(method);
                if (classFound)
                {
                    Resolution resolution = GetResolution(new string[] {method.Name.Name});
                    Problems.Add(new Problem(resolution));
                    return Problems;
                }
            }
            return base.Check(member);
        }
        /// <summary>
        /// VisitConstruct is called to visit instance initializers
        /// </summary>
        /// <param name="cons">The Construct object to search</param>
        /// <returns>An Expression</returns>
        public override Expression VisitConstruct(Construct cons)
        {
            if (cons != null)
            {
                MemberBinding memberBinding = cons.Constructor as MemberBinding;
                if (memberBinding != null)
                {
                    InstanceInitializer instanceInitializer = memberBinding.BoundMember as InstanceInitializer;
                    if (instanceInitializer != null && instanceInitializer.DeclaringType.FullName == className)
                        classFound = true;
                }
            }
            return base.VisitConstruct (cons);
        }
    }
}
