// File           : RegexShouldNotUseCultureSpecificFormat.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : April 2008
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Specialized;
using System.Text;
using Microsoft.Cci;
using Microsoft.FxCop.Sdk;
using Microsoft.FxCop.Sdk.Introspection;

namespace GlobalizationRules
{
    /// <summary>
    /// RegexShouldNotUseCultureSpecificFormat is a rule which checks for a regular expression that does not work for all cultures
    /// </summary>
    public class RegexShouldNotUseCultureSpecificFormat : BaseGlobalizationRule
    {
        private Method method;
        /// <summary>
        /// Constructs a RegexShouldNotUseCultureSpecificFormat object
        /// </summary>
        public RegexShouldNotUseCultureSpecificFormat()
            : base("RegexShouldNotUseCultureSpecificFormat")
        {
        }
        /// <summary>
        /// Check checks members for adherence to the rule
        /// </summary>
        /// <param name="member">The Member to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection Check(Member member)
        {
            method = member as Method;
            if (method == null)
                return null;

            for (int instructionNumber = 0; instructionNumber < method.Instructions.Length; instructionNumber++)
            {
                Instruction instruction = method.Instructions[instructionNumber];
                if (instruction.OpCode == OpCode.Ldstr && instructionNumber + 1 < method.Instructions.Length)
                {
                    // This instruction is a "load string" instruction and it isn't the last instruction 
                    // in the method.
                    Instruction nextInstruction = method.Instructions[instructionNumber + 1];
                    if (IsInstructionNewObject(nextInstruction, "System.Text.RegularExpressions.Regex"))
                    {
                        string loadString = instruction.Value.ToString();
                        if (IsCultureSpecificRegex(loadString))
                        {
                            // The string is a literal string and it is suspiciously like a localizable
                            // string and it is not already in the list of ignored strings and it is
                            // not passed to one of the exempt constructors or methods.
                            Resolution resolution = GetResolution(new string[] { loadString, method.Name.Name });
                            Problems.Add(new Problem(resolution));
                        }
                    }
                }
            }
            return Problems;
        }

        private bool IsCultureSpecificRegex(string regex)
        {
            return regex.IndexOf("[a-z]") > -1 || regex.IndexOf("[A-Z]") > -1;
        }
    }
}
