// File           : CommandLineParameters.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;

namespace Internationalization.Common
{
	/// <summary>
	/// CommandLineParameters handles command line parameters
	/// </summary>
	public class CommandLineParameters
	{
		/// <summary>
        /// Constructs a CommandLineParameters object
		/// </summary>
		/// <param name="args"></param>
		public CommandLineParameters(string[] args)
		{
			ProcessParameters(args);
		}
		private string[] parameters;
		/// <summary>
		/// Parameters is an array of command line parameters
		/// </summary>
		public string[] Parameters
		{
			get {return parameters;}
		}
		private string[] values;
		/// <summary>
		/// Values is an array of values of command line parameters
		/// </summary>
		public string[] Values
		{
			get {return values;}
		}
		private char[] parameterSeparators = new char[] {'/', '-'};
		/// <summary>
		/// ProcessParameters processes the incoming parameters and sets the Parameter list to those parameters
		/// </summary>
		/// <param name="args">An array of command line parameters</param>
		protected virtual void ProcessParameters(string[] args)
		{
			ArrayList parameterList = new ArrayList();
			ArrayList valueList = new ArrayList();
			foreach(string arg in args)
			{
				string parameter;
				if (Array.IndexOf(parameterSeparators, arg[0]) > -1)
					// strip off the leading parameter separator
					parameter = arg.Substring(1);
				else
					parameter = arg;

				int equalsIndex = parameter.IndexOf("=");
				if (equalsIndex > -1)
				{
					// this parameter has an associated value
					parameterList.Add(parameter.Substring(0, equalsIndex));
					valueList    .Add(parameter.Substring(equalsIndex + 1));
				}
				else
				{
					parameterList.Add(parameter);
					valueList.Add(String.Empty);
				}
			}
			parameters = (string[]) parameterList.ToArray(typeof(string));
			values     = (string[]) valueList    .ToArray(typeof(string));
		}
		/// <summary>
		/// IndexOf searches for a value in the list of parameters
		/// </summary>
		/// <param name="parameter">A command line parameter</param>
		/// <returns>The index of the command line parameter</returns>
		public virtual int IndexOf(string parameter)
		{
			return Array.IndexOf(parameters, parameter);
		}
		/// <summary>
		/// IndexOf searches for a value in the list of parameters
		/// </summary>
        /// <param name="parameter">A command line parameter</param>
		/// <param name="caseSensitive">True if the search should be case sensitive</param>
        /// <returns>The index of the command line parameter</returns>
		public virtual int IndexOf(string parameter, bool caseSensitive)
		{
			if (caseSensitive)
				return Array.IndexOf(parameters, parameter);
			else
			{
				// perform a case insensitive search
				int index = 0;
				foreach(string param in parameters)
				{
					if (param.ToUpper() == parameter.ToUpper())
						return index;
					index++;
				}
				return -1;
			}
		}
	}
}
