// File           : MainForm.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;
using System.Xml;
using Internationalization.Common;

namespace LiteralStringManager
{
	/// <summary>
	/// MainForm is the main form of the Literal String Manager
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
		private string ignoredLiteralStringsFile;
		private DataTable ignoredLiteralStringsDataTable;
		private DataTable newLiteralStringsDataTable;

		private System.Windows.Forms.TabControl tabControl;
		private System.Windows.Forms.TabPage tabPage1;
        private System.Windows.Forms.TabPage tabPage2;
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuItem2;
        private System.Windows.Forms.MenuItem menuItem3;
        private DataGridView dataGridViewNewLiteralStrings;
        private DataGridView dataGridViewIgnoredLiteralStrings;
        private IContainer components;

		/// <summary>
		/// Constructs a MainForm object
		/// </summary>
		public MainForm()
		{
			InitializeComponent();
		}

		/// <summary>
		/// Constructor with arguments
		/// </summary>
		/// <param name="args">A string array of command line parameters</param>
		public MainForm(string[] args)
		{
			InitializeComponent();

			string literalStringsFile;
			CommandLineParameters parameters = new CommandLineParameters(args);
			int literalStringsFileIndex = parameters.IndexOf("LiteralStringsFile", false);
			if (literalStringsFileIndex != -1)
				literalStringsFile = parameters.Values[literalStringsFileIndex];
			else
				literalStringsFile = FileFinder.Find("LiteralStrings.xml");

			if (literalStringsFile == String.Empty || ! File.Exists(literalStringsFile))
			{
				if (literalStringsFileIndex == -1)
					MessageBox.Show(MainFormResources.UnableToFindLiteralStringsXml);
				else
					MessageBox.Show(String.Format(
						MainFormResources.LiteralStringsFileDoesNotExist, literalStringsFile));

				Application.Exit();
			}
			else
			{
				DataSet dataSet = LoadLiteralStringsFile(literalStringsFile);
				newLiteralStringsDataTable = dataSet.Tables["NewLiteralStrings"];
                dataGridViewNewLiteralStrings.DataSource = dataSet;
                dataGridViewNewLiteralStrings.DataMember = "NewLiteralStrings";

				int ignoredLiteralStringsFileIndex = parameters.IndexOf("IgnoredLiteralStringsFile");
				if (ignoredLiteralStringsFileIndex != -1)
					ignoredLiteralStringsFile = parameters.Values[ignoredLiteralStringsFileIndex];
				else
					ignoredLiteralStringsFile = FileFinder.Find("IgnoredLiteralStrings.xml");

				DataSet ignoredLiteralStringsDataSet;
				if (ignoredLiteralStringsFile == String.Empty || ! File.Exists(ignoredLiteralStringsFile))
				{
					// put the ignored literal strings file back in the same place that we found the literal strings
					// file
					ignoredLiteralStringsFile = Path.GetDirectoryName(literalStringsFile) +
						Path.DirectorySeparatorChar + "IgnoredLiteralStrings.xml";

					ignoredLiteralStringsDataSet = CreateIgnoredLiteralStringsDataSet();
				}
				else
				{
					ignoredLiteralStringsDataSet = new DataSet();
					ignoredLiteralStringsDataSet.ReadXml(ignoredLiteralStringsFile);
				}

				ignoredLiteralStringsDataTable = ignoredLiteralStringsDataSet.Tables["IgnoredLiteralStrings"];

                dataGridViewIgnoredLiteralStrings.DataSource = ignoredLiteralStringsDataSet;
                dataGridViewIgnoredLiteralStrings.DataMember = "IgnoredLiteralStrings";
			}
		}
		private DataSet CreateIgnoredLiteralStringsDataSet()
		{
			DataColumn dataColumn;

			DataSet dataSet = new DataSet("IgnoredLiteralStrings");
			DataTable dataTable = new DataTable("IgnoredLiteralStrings");
			dataColumn = dataTable.Columns.Add("MethodName");
			dataColumn.ReadOnly = true;
			dataColumn = dataTable.Columns.Add("LiteralString");
			dataColumn.ReadOnly = true;
			dataSet.Tables.Add(dataTable);
			return dataSet;
		}
		private DataSet LoadLiteralStringsFile(string literalStringsFile)
		{
			DataSet dataSet = new DataSet("LiteralStrings");
			DataTable dataTable = new DataTable("NewLiteralStrings");
			dataTable.Columns.Add("Accept", typeof(Boolean));
			dataTable.Columns.Add("MethodName");
			dataTable.Columns.Add("LiteralString");
			dataSet.Tables.Add(dataTable);

			XmlTextReader reader = new XmlTextReader(literalStringsFile);
			try
			{
				while (reader.Read())
				{
					if (reader.NodeType == XmlNodeType.Element && reader.Name == "LiteralString")
					{
						string methodName = String.Empty;
						string literalString = String.Empty;
						for(int attributeNo = 0; attributeNo < reader.AttributeCount; attributeNo++)
						{
							reader.MoveToAttribute(attributeNo);
							if (reader.Name == "Method")
								methodName = reader.Value;
							else if (reader.Name == "String")
								literalString = reader.Value;
						}
						if (methodName != String.Empty && literalString != String.Empty)
							AddLiteralStringsRow(dataTable, methodName, literalString);
					}
				}
			}
			finally
			{
				reader.Close();
			}
			dataTable.Columns["MethodName"   ].ReadOnly = true;
			dataTable.Columns["LiteralString"].ReadOnly = true;
			return dataSet;
		}
		private void AddLiteralStringsRow(DataTable dataTable, string methodName, string literalString)
		{
			literalString = StripControlChars(literalString);
			DataRow dataRow = dataTable.NewRow();
			dataRow["Accept"] = false;
			dataRow["MethodName"] = methodName;
			dataRow["LiteralString"] = literalString.ToString();
			dataTable.Rows.Add(dataRow);
		}
		private string StripControlChars(string literalString)
		{
			literalString = literalString.Replace(((char) 13).ToString(), "");
			literalString = literalString.Replace(((char) 10).ToString(), "");
			return literalString;
		}
		private void AddIgnoredLiteralStringsRow(DataTable dataTable, string methodName, string literalString)
		{
			DataRow dataRow = dataTable.NewRow();
			dataRow["MethodName"] = methodName;
			dataRow["LiteralString"] = literalString;
			dataTable.Rows.Add(dataRow);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(MainForm));
            this.tabControl = new System.Windows.Forms.TabControl();
            this.tabPage1 = new System.Windows.Forms.TabPage();
            this.dataGridViewNewLiteralStrings = new System.Windows.Forms.DataGridView();
            this.tabPage2 = new System.Windows.Forms.TabPage();
            this.dataGridViewIgnoredLiteralStrings = new System.Windows.Forms.DataGridView();
            this.mainMenu = new System.Windows.Forms.MainMenu(this.components);
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.menuItem2 = new System.Windows.Forms.MenuItem();
            this.menuItem3 = new System.Windows.Forms.MenuItem();
            this.tabControl.SuspendLayout();
            this.tabPage1.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.dataGridViewNewLiteralStrings)).BeginInit();
            this.tabPage2.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.dataGridViewIgnoredLiteralStrings)).BeginInit();
            this.SuspendLayout();
            // 
            // tabControl
            // 
            this.tabControl.Controls.Add(this.tabPage1);
            this.tabControl.Controls.Add(this.tabPage2);
            resources.ApplyResources(this.tabControl, "tabControl");
            this.tabControl.Name = "tabControl";
            this.tabControl.SelectedIndex = 0;
            this.tabControl.SelectedIndexChanged += new System.EventHandler(this.tabControl_SelectedIndexChanged);
            // 
            // tabPage1
            // 
            this.tabPage1.Controls.Add(this.dataGridViewNewLiteralStrings);
            resources.ApplyResources(this.tabPage1, "tabPage1");
            this.tabPage1.Name = "tabPage1";
            // 
            // dataGridViewNewLiteralStrings
            // 
            this.dataGridViewNewLiteralStrings.ColumnHeadersHeightSizeMode = System.Windows.Forms.DataGridViewColumnHeadersHeightSizeMode.AutoSize;
            resources.ApplyResources(this.dataGridViewNewLiteralStrings, "dataGridViewNewLiteralStrings");
            this.dataGridViewNewLiteralStrings.Name = "dataGridViewNewLiteralStrings";
            // 
            // tabPage2
            // 
            this.tabPage2.Controls.Add(this.dataGridViewIgnoredLiteralStrings);
            resources.ApplyResources(this.tabPage2, "tabPage2");
            this.tabPage2.Name = "tabPage2";
            // 
            // dataGridViewIgnoredLiteralStrings
            // 
            this.dataGridViewIgnoredLiteralStrings.AllowUserToAddRows = false;
            this.dataGridViewIgnoredLiteralStrings.AllowUserToDeleteRows = false;
            this.dataGridViewIgnoredLiteralStrings.ColumnHeadersHeightSizeMode = System.Windows.Forms.DataGridViewColumnHeadersHeightSizeMode.AutoSize;
            resources.ApplyResources(this.dataGridViewIgnoredLiteralStrings, "dataGridViewIgnoredLiteralStrings");
            this.dataGridViewIgnoredLiteralStrings.Name = "dataGridViewIgnoredLiteralStrings";
            this.dataGridViewIgnoredLiteralStrings.ReadOnly = true;
            // 
            // mainMenu
            // 
            this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem1});
            // 
            // menuItem1
            // 
            this.menuItem1.Index = 0;
            this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem2,
            this.menuItem3});
            resources.ApplyResources(this.menuItem1, "menuItem1");
            // 
            // menuItem2
            // 
            this.menuItem2.Index = 0;
            resources.ApplyResources(this.menuItem2, "menuItem2");
            this.menuItem2.Click += new System.EventHandler(this.menuItem2_Click);
            // 
            // menuItem3
            // 
            this.menuItem3.Index = 1;
            resources.ApplyResources(this.menuItem3, "menuItem3");
            this.menuItem3.Click += new System.EventHandler(this.menuItem3_Click);
            // 
            // MainForm
            // 
            resources.ApplyResources(this, "$this");
            this.Controls.Add(this.tabControl);
            this.Menu = this.mainMenu;
            this.Name = "MainForm";
            this.Closing += new System.ComponentModel.CancelEventHandler(this.MainForm_Closing);
            this.Load += new System.EventHandler(this.MainForm_Load);
            this.tabControl.ResumeLayout(false);
            this.tabPage1.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.dataGridViewNewLiteralStrings)).EndInit();
            this.tabPage2.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.dataGridViewIgnoredLiteralStrings)).EndInit();
            this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args) 
		{
			Application.Run(new MainForm(args));
		}

		private void menuItem2_Click(object sender, System.EventArgs e)
		{
			Save();
		}
		private void Save()
		{
			DataRow[] dataRows = newLiteralStringsDataTable.Select("Accept='true'");
			if (dataRows.GetLength(0) > 0)
			{
				foreach(DataRow dataRow in dataRows)
				{
					AddIgnoredLiteralStringsRow(ignoredLiteralStringsDataTable, 
						dataRow["MethodName"].ToString(), dataRow["LiteralString"].ToString());
				}

				// now that the DataTable has been updated write it back to the file
				ignoredLiteralStringsDataTable.DataSet.WriteXml(ignoredLiteralStringsFile);

				// finally remove the "Accepted" rows
				foreach(DataRow dataRow in dataRows)
				{
					dataRow.Delete();
				}
				newLiteralStringsDataTable.AcceptChanges();
			}
		}

		private void menuItem3_Click(object sender, System.EventArgs e)
		{
			Close();
		}

		private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if (newLiteralStringsDataTable != null)
			{
				DataRow[] dataRows = newLiteralStringsDataTable.Select("Accept='true'");
				if (dataRows.GetLength(0) > 0)
				{
					DialogResult result = MessageBox.Show(MainFormResources.ChangesNotSaved +
                        System.Environment.NewLine + System.Environment.NewLine + MainFormResources.SaveChanges,
						"Save", MessageBoxButtons.YesNoCancel);

					if (result == DialogResult.Yes)
						Save();
					else if (result == DialogResult.Cancel)
						e.Cancel = true;
				}
			}
		}

        private void MainForm_Load(object sender, EventArgs e)
        {
            dataGridViewNewLiteralStrings.Columns["MethodName"].Width = 400;
            dataGridViewNewLiteralStrings.Columns["LiteralString"].Width = 400;
        }

        private bool dataGridViewIgnoredLiteralStringsColumnWidthsSet = false;
        private void tabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!dataGridViewIgnoredLiteralStringsColumnWidthsSet)
            {
                dataGridViewIgnoredLiteralStrings.Columns["MethodName"].Width = 400;
                dataGridViewIgnoredLiteralStrings.Columns["LiteralString"].Width = 400;
                dataGridViewIgnoredLiteralStringsColumnWidthsSet = true;
            }
        }
	}
}
