﻿// File           : Program.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : February 2012
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;
using System.Drawing;
using System.Globalization;
using System.Resources;
using Internationalization.ResourceAdministration;
using System.IO;
using System.ComponentModel.Design;
using System.Collections.Generic;

namespace ReintegrateResources
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("ReintegrateResources");
            Console.WriteLine("ReintegrateResources is a utility for reintegrating resources returned from the translator into the master copy of resources.");
            Console.WriteLine(String.Empty);
            string masterResourcesPath = GetParameter(args, "MasterResourcesPath");
            string incomingResourcesPath = GetParameter(args, "IncomingResourcesPath");

            if (String.IsNullOrEmpty(masterResourcesPath) || String.IsNullOrEmpty(incomingResourcesPath))
            {
                ShowSyntax();
            }
            else
            {
                bool noWrite = (GetParameter(args, "NoWrite") != null);
                Reintegrate(masterResourcesPath, incomingResourcesPath, noWrite);

                Console.WriteLine("Done.");
            }
        }

        public static void ShowSyntax()
        {
            Console.WriteLine("Syntax:");
            Console.WriteLine("ReintegrateResources /MasterResourcesPath=<MasterResourcesPath> /IncomingResourcesPath=<IncomingResourcesPath> [/NoWrite]");
            Console.WriteLine("where:-");
            Console.WriteLine("       MasterResourcesPath is the file path to the master copy of resources");
            Console.WriteLine("       IncomingResourcesPath is the file path to the incoming resources being returned from the translator");
            Console.WriteLine("       NoWrite specifies that the incoming changes should not be written back to the master copy of the resources");
            Console.WriteLine(String.Empty);
            Console.WriteLine("Example:");
            Console.WriteLine(@"ReintegrateResources /MasterResourcesPath=c:\MyProjects\CustomerCareApplication /IncomingResourcesPath=c:\temp");
            Console.WriteLine(String.Empty);
        }

        static void Reintegrate(string masterResourcesPath, string incomingResourcesPath, bool noWrite)
        {
            // reintegrate this folder
            ReintegrateFolder(masterResourcesPath, incomingResourcesPath, noWrite);

            // reintegrate all subfolders (and their subfolders etc.)
            foreach (string incomingSubFolder in Directory.GetDirectories(incomingResourcesPath))
            {
                string relativeSubFolder = incomingSubFolder.Substring(incomingResourcesPath.Length + 1);
                Reintegrate(Path.Combine(masterResourcesPath, relativeSubFolder), incomingSubFolder, noWrite);
            }
        }

        static void ReintegrateFolder(string masterResourcesPath, string incomingResourcesPath, bool noWrite)
        {
            // this is hard wired to use a ResXResourcesGovernor but it would be possible to pass this type as a parameter
            IResourcesGovernor incomingResourcesGovernor = new ResXResourcesGovernor(incomingResourcesPath);

            // this is hard wired to use a ResXResourcesGovernor but it would be possible to pass this type as a parameter
            IResourcesGovernor masterResourcesGovernor = new ResXResourcesGovernor(masterResourcesPath);

            foreach (string baseName in incomingResourcesGovernor.GetInvariantCultureBaseNames())
            {
                foreach (CultureInfo cultureInfo in incomingResourcesGovernor.GetExistingCultures(baseName))
                {
                    if (!cultureInfo.Equals(CultureInfo.InvariantCulture))
                    {
                        ResourceSet flattenedMasterResourceSet = GetFlattenedResourceSet(masterResourcesGovernor, baseName, cultureInfo);

                        IResourceGovernor incomingResourceGovernor = incomingResourcesGovernor.GetResourceGovernor(baseName, cultureInfo);

                        IResourceGovernor masterResourceGovernor = masterResourcesGovernor.GetResourceGovernor(baseName, cultureInfo);

                        if (incomingResourceGovernor.ResourceExists() && masterResourceGovernor.ResourceExists())
                        {
                            string incomingFilename = GetFilename(incomingResourcesPath, ((FileBasedResourcesGovernor)incomingResourcesGovernor).Extension, incomingResourceGovernor);
                            Console.Write(String.Format("Reading {0}...", incomingFilename));

                            ResourceSet incomingResourceSet = incomingResourceGovernor.ReadResourceSet();

                            ResourceSet masterResourceSet = masterResourceGovernor.ReadResourceSet();

                            if (masterResourceGovernor.ReintegrateResourceSet(flattenedMasterResourceSet, masterResourceSet, incomingResourceSet, new AcceptResourceEntry(AcceptStringAndImageResourceEntry)))
                            {
                                if (!noWrite)
                                {
                                    masterResourceGovernor.WriteResourceSet(masterResourceSet);
                                    Console.WriteLine(" Changes applied");
                                }
                                else
                                {
                                    Console.WriteLine(" Changes required but no changes made (/NoWrite specified)");
                                }
                            }
                            else
                            {
                                Console.WriteLine(" No changes required");
                            }
                        }
                    }
                }
            }
        }

        private static string GetFilename(string path, string extension, IResourceGovernor resourceGovernor)
        {
            return GetFilename(path, resourceGovernor.BaseName, resourceGovernor.CultureInfo, extension);
        }

        private static string GetFilename(string path, string baseName, CultureInfo cultureInfo, string extension)
        {
            string fileName;
            if (cultureInfo.Equals(CultureInfo.InvariantCulture))
                fileName = baseName + "." + extension;
            else
                fileName = baseName + "." + cultureInfo.Name + "." + extension;

            return System.IO.Path.Combine(path, fileName);
        }

        private static bool AcceptStringAndImageResourceEntry(DictionaryEntry entry, object originalValue)
        {
            ResourcesDataNodeInterpreter resourcesDataNodeInterpreter = new ResourcesDataNodeInterpreter(entry.Key.ToString(), entry.Value);
            if (resourcesDataNodeInterpreter.Value is string || resourcesDataNodeInterpreter.Value is Image)
            {
                // the incoming resource entry is a string or an Image
                if (resourcesDataNodeInterpreter.Value is string)
                {
                    // the incoming value is a string so check that it has the same number of parameters
                    ResourcesDataNodeInterpreter originalResourcesDataNodeInterpreter = new ResourcesDataNodeInterpreter(entry.Key.ToString(), originalValue);
                    if (GetHighestParameterNumber(resourcesDataNodeInterpreter.Value.ToString()) != 
                        GetHighestParameterNumber(originalResourcesDataNodeInterpreter.Value.ToString()))
                    {
                        // the number of parameters in the incoming string is not the same as the number of parameters in the original string
                        return false;
                    }
                }

                return true;
            }
            else
            {
                return false;
            }
        }

        private static int GetHighestParameterNumber(string text)
        {
            string[] parameters = GetParameters(text);
            int highestParameterNumber = 0;

            foreach (string parameter in parameters)
            {
                int parameterNumber = GetParameterNumber(parameter);
                if (parameterNumber > highestParameterNumber)
                {
                    highestParameterNumber = parameterNumber;
                }
            }

            return highestParameterNumber;
        }

        private static int GetParameterNumber(string parameter)
        {
            int colonIndex = parameter.IndexOf(":");
            if (colonIndex > -1)
            {
                // the parameter has a data type so strip it out
                parameter = parameter.Substring(0, colonIndex);
            }

            return int.Parse(parameter);
        }

        private static string[] GetParameters(string text)
        {
            List<string> parameters = new List<string>();

            int openingBraceIndex = text.IndexOf("{");
            while (openingBraceIndex > -1)
            {
                text = text.Substring(openingBraceIndex + 1);
                int closingBraceIndex = text.IndexOf("}");
                if (closingBraceIndex == -1)
                {
                    break;
                }

                string parameter = text.Substring(0, closingBraceIndex);
                parameters.Add(parameter);

                openingBraceIndex = text.IndexOf("{");
            }

            return parameters.ToArray();
        }

        private static ResourceSet GetFlattenedResourceSet(IResourcesGovernor masterResourcesGovernor, string baseName, CultureInfo flattenedCultureInfo)
        {
            CultureInfo[] masterCultureInfos = masterResourcesGovernor.GetExistingCultures(baseName, flattenedCultureInfo);

            if (masterCultureInfos.GetLength(0) == 0)
                return null;

            Array.Sort(masterCultureInfos, new CultureInfoParentComparer());

            IResourceGovernor masterResourceGovernor = masterResourcesGovernor.GetResourceGovernor(baseName, masterCultureInfos[masterCultureInfos.GetLength(0) - 1]);

            ResourceSet flattenedMasterResourceSet = masterResourceGovernor.ReadResourceSet();

            for (int cultureInfoNumber = masterCultureInfos.GetLength(0) - 2; cultureInfoNumber >= 0; cultureInfoNumber--)
            {
                CultureInfo cultureInfo = masterCultureInfos[cultureInfoNumber];

                masterResourceGovernor = masterResourcesGovernor.GetResourceGovernor(baseName, cultureInfo);

                masterResourceGovernor.AddResourceSet(flattenedMasterResourceSet, masterResourceGovernor.ReadResourceSet());
            }

            return flattenedMasterResourceSet;
        }

        private static string GetParameter(string[] args, string parameterName)
        {
            foreach (string arg in args)
            {
                if (arg.StartsWith("/" + parameterName, StringComparison.OrdinalIgnoreCase))
                {
                    int equalsIndex = arg.IndexOf("=");
                    if (equalsIndex > -1)
                    {
                        return arg.Substring(equalsIndex + 1);
                    }

                    return String.Empty;
                }
            }

            return null;
        }

        public class CultureInfoParentComparer : IComparer
        {
            int IComparer.Compare(Object x, Object y)
            {
                CultureInfo cultureInfoX = ((CultureInfo)x);
                CultureInfo cultureInfoY = ((CultureInfo)y);
                if (cultureInfoX.Equals(CultureInfo.InvariantCulture) && cultureInfoY.Equals(CultureInfo.InvariantCulture))
                    return 0;
                else if (cultureInfoX.Equals(CultureInfo.InvariantCulture))
                    return -1;
                else if (cultureInfoY.Equals(CultureInfo.InvariantCulture))
                    return 1;
                else if (cultureInfoX.IsNeutralCulture &&
                    cultureInfoY.IsNeutralCulture)
                    return 0;
                else if (cultureInfoX.IsNeutralCulture)
                    return -1;
                else if (cultureInfoY.IsNeutralCulture)
                    return 1;
                else if (!cultureInfoX.IsNeutralCulture &&
                    !cultureInfoY.IsNeutralCulture)
                    return 0;
                else if (!cultureInfoX.IsNeutralCulture)
                    return -1;
                else
                    return 1;
            }
        }
    }
}
