﻿// File           : Program.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : February 2012
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.IO;

namespace ResourcePackager
{
    class Program
    {
        // Example uses:-
        // ResourcePackager /TargetZipFilename=c:\temp\Resources.zip /MasterResourcesPath=c:\temp\WindowsFormsSpecifics /RecurseSubFolders
        // ResourcePackager /SourceZipFilename=c:\temp\Resources.zip /TargetRootPath=c:\temp\IncomingResources
        static void Main(string[] args)
        {
            Console.WriteLine("ResourcePackager");
            Console.WriteLine("ResourcePackager is a utility for packaging and unpackaging resources for transfer to a translator.");
            Console.WriteLine(String.Empty);
            string masterResourcesPath = GetParameter(args, "MasterResourcesPath");
            string targetZipFilename = GetParameter(args, "TargetZipFilename");
            string sourceZipFilename = GetParameter(args, "SourceZipFilename");

            if (!String.IsNullOrEmpty(sourceZipFilename) &&
                !(String.IsNullOrEmpty(masterResourcesPath) || String.IsNullOrEmpty(targetZipFilename)))
            {
                Console.WriteLine("SourceZipFilename and MasterResourcesPath/TargetZipFilename are mutually exclusive");
                Console.WriteLine(String.Empty);
                ShowSyntax();
            }
            else if (String.IsNullOrEmpty(sourceZipFilename) &&
                (String.IsNullOrEmpty(masterResourcesPath) || String.IsNullOrEmpty(targetZipFilename)))
            {
                ShowSyntax();
            }
            else if (! String.IsNullOrEmpty(masterResourcesPath) && ! Directory.Exists(masterResourcesPath))
            {
                Console.WriteLine(String.Format("MasterResourcesPath ({0}) does not exist", masterResourcesPath));
            }
            else if (!String.IsNullOrEmpty(sourceZipFilename))
            {
                // unzip package
                bool overwrite = (GetParameter(args, "Overwrite") != null);

                ResourcePackager.Unpackage(
                    sourceZipFilename, 
                    GetParameter(args, "TargetRootPath"), 
                    new ResourcePackagerProgressEventHandler(UnpackagingProgress),
                    overwrite);
            }
            else
            {
                // create package
                bool recurseSubFolders = (GetParameter(args, "RecurseSubFolders") != null);
                bool saveAbsoluteMasterResourcesPath = (GetParameter(args, "SaveAbsoluteMasterResourcesPath") != null);

                ResourcePackager.Package(masterResourcesPath, targetZipFilename, new ResourcePackagerProgressEventHandler(PackagingProgress), recurseSubFolders, saveAbsoluteMasterResourcesPath);
            }
        }

        public static void ShowSyntax()
        {
            Console.WriteLine("Syntax To Create A Package:");
            Console.WriteLine("ResourcePackager /MasterResourcesPath=<MasterResourcesPath> /TargetZipFilename=<TargetZipFilename> [/RecurseSubFolders] [/SaveAbsoluteMasterResourcesPath]");
            Console.WriteLine("where:-");
            Console.WriteLine("       MasterResourcesPath is the file path to the resources");
            Console.WriteLine("       TargetZipFilename is the filename of the ZIP file to create");
            Console.WriteLine("       RecurseSubFolders specifies that ResourcePackager should iterate through the subfolders of MasterResourcesPath");
            Console.WriteLine("       SaveAbsoluteMasterResourcesPath specifies that the absolute path of the resource files should be saved in the ZIP file");
            Console.WriteLine(String.Empty);
            Console.WriteLine("Example To Create A Package:");
            Console.WriteLine(@"ResourcePackager /MasterResourcesPath=c:\MyProjects\CustomerCareApplication /TargetZipFilename=c:\MyProjects\CustomerCareResources.zip");
            Console.WriteLine(String.Empty);

            Console.WriteLine("Syntax To Unpackage A ZIP:");
            Console.WriteLine("ResourcePackager /SourceZipFilename=<TargetZipFilename> [/TargetRootPath=<TargetRootPath>] [/Overwrite]");
            Console.WriteLine("where:-");
            Console.WriteLine("       SourceZipFilename is the filename of the ZIP file to extract the resources from");
            Console.WriteLine("       TargetRootPath is the path into which the files are extracted to");
            Console.WriteLine("       Overwrite specifies that existing resources files should be overwritten");
            Console.WriteLine(String.Empty);
            Console.WriteLine("Example To Unpackage A ZIP:");
            Console.WriteLine(@"ResourcePackager /SourceZipFilename=c:\MyProjects\CustomerCareResources.zip /TargetRootPath=c:\IncomingResources /Overwrite");
            Console.WriteLine(String.Empty);
        }

        public static void PackagingProgress(object sender, ResourcePackagerProgressEventArgs args)
        {
            Console.WriteLine(String.Format("    Adding {0}...", args.Filename));
        }

        public static void UnpackagingProgress(object sender, ResourcePackagerProgressEventArgs args)
        {
            if (args.CannotOverwrite)
            {
                Console.WriteLine(String.Format("    Cannot overwrite {0}.", args.Filename));
            }
            else
            {
                Console.WriteLine(String.Format("    Extracting {0}...", args.Filename));
            }
        }

        private static string GetParameter(string[] args, string parameterName)
        {
            foreach (string arg in args)
            {
                if (arg.StartsWith("/" + parameterName, StringComparison.OrdinalIgnoreCase))
                {
                    int equalsIndex = arg.IndexOf("=");
                    if (equalsIndex > -1)
                    {
                        return arg.Substring(equalsIndex + 1);
                    }

                    return String.Empty;
                }
            }

            return null;
        }
    }
}
