﻿// File           : ResourcePackager.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : February 2012
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;
using System.IO;
using Internationalization.ResourceAdministration;

namespace ResourcePackager
{
    public class ResourcePackagerProgressEventArgs: EventArgs
    {
        private bool include = true;
        public string Filename { get; set; }

        public bool Include
        {
            get
            {
                return this.include;
            }

            set
            {
                this.include = value;
            }
        }

        public bool CannotOverwrite { get; set; }
    }

    public delegate void ResourcePackagerProgressEventHandler(object sender, ResourcePackagerProgressEventArgs args);

    public class ResourcePackager
    {
        public static void Package(string masterResourcesPath, string targetZipFilename)
        {
            Package(masterResourcesPath, targetZipFilename, null, true, true);
        }

        public static void Package(string masterResourcesPath, string targetZipFilename, ResourcePackagerProgressEventHandler eventHandler, bool recurseSubFolders, bool saveAbsoluteMasterResourcesPath)
        {
            if (File.Exists(targetZipFilename))
            {
                File.Delete(targetZipFilename);
            }

            ZipFile zipFile = new ZipFile(targetZipFilename);
            zipFile.Create();
            try
            {
                Package(masterResourcesPath, masterResourcesPath, zipFile, eventHandler, recurseSubFolders, saveAbsoluteMasterResourcesPath);
            }
            finally
            {
                zipFile.Close();
            }
        }

        private static void Package(
            string masterResourcesPath, 
            string resourcesPath, 
            ZipFile zipFile, 
            ResourcePackagerProgressEventHandler eventHandler, 
            bool recurseSubFolders, 
            bool saveAbsoluteMasterResourcesPath)
        {
            // this is hard wired to use a ResXResourcesGovernor but it would be possible to pass this type as a parameter
            ResXResourcesGovernor resourcesGovernor = new ResXResourcesGovernor(resourcesPath);
            foreach (string invariantCultureBaseName in resourcesGovernor.GetInvariantCultureBaseNames())
            {
                foreach (CultureInfo cultureInfo in resourcesGovernor.GetExistingCultures(invariantCultureBaseName))
                {
                    string filename = GetFilename(resourcesPath, invariantCultureBaseName, cultureInfo, resourcesGovernor.Extension);
                    if (Progress(eventHandler, filename))
                    {
                        // the entry name should not include the drive
                        string entryName = CreateEntryName(filename, masterResourcesPath, saveAbsoluteMasterResourcesPath);
                        zipFile.AddFile(filename, entryName);
                    }
                }
            }

            if (recurseSubFolders)
            {
                foreach (string subFolder in Directory.GetDirectories(resourcesPath))
                {
                    Package(masterResourcesPath, subFolder, zipFile, eventHandler, recurseSubFolders, saveAbsoluteMasterResourcesPath);
                }
            }
        }

        private static string CreateEntryName(string filename, string masterResourcesPath, bool saveAbsoluteMasterResourcesPath)
        {
            // remove the drive letter and colon
            filename = RemoveDrive(filename);

            if (!saveAbsoluteMasterResourcesPath)
            {
                // the entry should be saved as a relative path and not an absolute path
                string masterResourcesPathWithoutDrive = RemoveDrive(masterResourcesPath);
                if (filename.StartsWith(masterResourcesPathWithoutDrive, StringComparison.OrdinalIgnoreCase))
                {
                    // strip the path from the start of filename
                    filename = filename.Substring(masterResourcesPathWithoutDrive.Length);
                }
            }

            return filename;
        }

        private static string RemoveDrive(string filename)
        {
            if (filename.Length > 1 && filename[1] == ':')
            {
                // the second character of the filename is a colon so the first character is a drive letter
                filename = filename.Substring(2);
            }

            return filename;
        }

        private static bool Progress(ResourcePackagerProgressEventHandler eventHandler, string filename)
        {
            return Progress(eventHandler, filename, false);
        }

        private static bool Progress(ResourcePackagerProgressEventHandler eventHandler, string filename, bool cannotOverwrite)
        {
            if (eventHandler != null)
            {
                ResourcePackagerProgressEventArgs args = new ResourcePackagerProgressEventArgs();
                args.Filename = filename;
                args.CannotOverwrite = cannotOverwrite;
                eventHandler(null, args);
                return args.Include;
            }

            return true;
        }

        private static string GetFilename(string path, string baseName, CultureInfo cultureInfo, string extension)
        {
            string fileName;
            if (cultureInfo.Equals(CultureInfo.InvariantCulture))
                fileName = baseName + "." + extension;
            else
                fileName = baseName + "." + cultureInfo.Name + "." + extension;

            return System.IO.Path.Combine(path, fileName);
        }

        public static void Unpackage(string sourceZipFilename, string targetRootPath, ResourcePackagerProgressEventHandler resourcePackagerProgressEventHandler, bool overwrite)
        {
            ZipFile zipFile = new ZipFile(sourceZipFilename);
            zipFile.Open();
            try
            {
                foreach (string filename in zipFile.Files)
                {
                    string targetFilename = filename;
                    if (targetRootPath != null)
                    {
                        targetFilename = Path.Combine(targetRootPath, filename.Substring(1));
                    }

                    if (overwrite || !File.Exists(targetFilename))
                    {
                        // either we can overwrite all files or else this file does not exist
                        if (Progress(resourcePackagerProgressEventHandler, filename))
                        {
                            zipFile.ExtractFile(filename, targetFilename);
                        }
                    }
                    else
                    {
                        // the file exists and we are not allowed to overwrite it
                        Progress(resourcePackagerProgressEventHandler, filename, true);
                    }
                }
            }
            finally
            {
                zipFile.Close();
            }
        }
    }
}
