﻿// File           : ZipFile.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : February 2012
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Packaging;
using System.Net.Mime;

namespace ResourcePackager
{
    // The downside of using the Package class is that it creates an additional file
    // called "[Content_Types].xml". It is not possible to prevent this file from
    // being added.
    public class ZipFile
    {
        private Package package;

        public ZipFile(string filename)
        {
            this.Filename = filename;
        }

        public string Filename { get; set; }

        public void Create()
        {
            this.package = Package.Open(this.Filename, FileMode.Create);
        }

        public void AddEntry(string entryName, byte[] entry)
        {
            using (MemoryStream stream = new MemoryStream())
            {
                stream.Write(entry, 0, entry.Length);
                stream.Position = 0;
                AddStream(entryName, stream);
            }
        }

        public void AddFile(string file, string entryName)
        {
            using (FileStream fileStream =
                new FileStream(file, FileMode.Open, FileAccess.Read))
            {
                AddStream(entryName, fileStream);
            }
        }

        private void AddStream(string path, Stream stream)
        {
            Uri uri = PackUriHelper.CreatePartUri(new Uri(path, UriKind.Relative));

            PackagePart packagePart = package.CreatePart(uri, MediaTypeNames.Text.Plain);

            CopyStream(stream, packagePart.GetStream());
        }

        private static void CopyStream(Stream source, Stream target)
        {
            const int bufSize = 0x1000;
            byte[] bytes = new byte[bufSize];
            int bytesRead = 0;
            while ((bytesRead = source.Read(bytes, 0, bufSize)) > 0)
            {
                target.Write(bytes, 0, bytesRead);
            }
        }

        public void Close()
        {
            package.Close();
        }

        public void Open()
        {
            this.package = Package.Open(this.Filename, FileMode.Open);
        }

        public string[] Files
        {
            get
            {
                List<string> files = new List<string>();
                PackagePartCollection parts = this.package.GetParts();
                foreach (PackagePart part in parts)
                {
                    files.Add(part.Uri.ToString().Replace("/", @"\"));
                }

                return files.ToArray();
            }
        }

        public void ExtractFile(string filename, string targetFilename)
        {
            string targetPath = Path.GetDirectoryName(targetFilename);
            if (!Directory.Exists(targetPath))
            {
                // the target directory does not exist
                Directory.CreateDirectory(targetPath);
            }

            string uriString = filename.Replace(@"\", "/");
            Uri uri = new Uri(uriString, UriKind.Relative);
            PackagePart part = package.GetPart(uri);
            Stream stream = part.GetStream();
            using (FileStream fileStream =
                new FileStream(targetFilename, FileMode.Create, FileAccess.Write))
            {
                CopyStream(stream, fileStream);
            }
        }
    }
}
