// File           : PublishStateHelper.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : October 2007
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Collections;
using System.IO;
using System.Globalization;

namespace Internationalization.ClickOnce
{
    /// <summary>
    /// PublishStateHelper is a class for modifying .csproj and .vbproj files for internationalized applications deployed using ClickOnce
    /// </summary>
    /// <remarks>
    /// PublishStateHelper's SetPublishState method sets the PublishState of localized resources according to parameters passed.
    /// Only those culture-neutral and culture-specific resources that exist in the project file are changed. If a resource
    /// file is set to "Auto" then it is not included in the project file and therefore not changed.
    /// </remarks>
    public class PublishStateHelper
    {
        private class CultureInfoHelper
        {
            public static bool IsValidCultureName(string cultureName)
            {
                try
                {
                    CultureInfo cultureInfo = new CultureInfo(cultureName);
                    return true;
                }
                catch
                {
                    return false;
                }
            }
        }
        /// <summary>
        /// PublishStateChange represents a desired change to the PublishState of a localized file to be deployed by ClickOnce
        /// </summary>
        public class PublishStateChange
        {
            private string cultureName;
            private bool include;

            /// <summary>
            /// CultureName is the name of the culture for which the PublishState should be changed
            /// </summary>
            public string CultureName
            {
                get { return cultureName; }
                set { cultureName = value; }
            }
            /// <summary>
            /// Include is true if the PublishState should be set to "Include" or false if it should be set to "Exclude"
            /// </summary>
            public bool Include
            {
                get { return include; }
                set { include = value; }
            }
            /// <summary>
            /// Constructs a PublishStateChange object
            /// </summary>
            /// <param name="cultureName">The name of the culture</param>
            /// <param name="include">true if the PublishState should be set to "Include" or false if it should be set to "Exclude"</param>
            public PublishStateChange(string cultureName, bool include)
            {
                this.cultureName = cultureName;
                this.include = include;
            }
        }
        /// <summary>
        /// PublishStateChanges is a representation of all of the PublishState changes that should be applied to a project file
        /// </summary>
        public class PublishStateChanges
        {
            private bool allParameterFound;
            private bool allInclude;
            private List<PublishStateChange> changes = new List<PublishStateChange>();

            /// <summary>
            /// AllParameterFound is true is an "ALL" parameter was found in the list of parameters passed to the constructor
            /// </summary>
            public bool AllParameterFound
            {
                get { return allParameterFound; }
                set { allParameterFound = value; }
            }
            /// <summary>
            /// AllInclude is true if all parameters other than those specifically listed should be set to "Include". false indicates that they should be set to "Exclude".
            /// </summary>
            /// <remarks>This property is only relevant if AllParameterFound is true</remarks>
            public bool AllInclude
            {
                get { return allInclude; }
                set { allInclude = value; }
            }
            /// <summary>
            /// Changes is a list of PublishStateChange objects indicating the changes that should be made to named cultures
            /// </summary>
            public List<PublishStateChange> Changes
            {
                get { return changes; }
                set { changes = value; }
            }
            /// <summary>
            /// Constructs a PublishStateChanges object
            /// </summary>
            /// <param name="changeStrings">An array of changes to make to the PublishStates in a project file</param>
            public PublishStateChanges(string[] changeStrings)
            {
                foreach (string changeString in changeStrings)
                {
                    string[] changeBits = changeString.Split(':');
                    if (changeBits.GetLength(0) != 2)
                        throw new ArgumentException(PublishStateHelperResources.BadChangeParameter);

                    if (String.Compare(changeBits[0], "ALL", StringComparison.OrdinalIgnoreCase) == 0)
                    {
                        allParameterFound = true;
                        allInclude = GetIncludeExcludeValue(changeBits[1]);
                    }
                    else if (CultureInfoHelper.IsValidCultureName(changeBits[0]))
                    {
                        changes.Add(new PublishStateChange(
                            changeBits[0], GetIncludeExcludeValue(changeBits[1])));
                    }
                    else
                        throw new ArgumentException(PublishStateHelperResources.BadChangeParameter);
                }
            }
            private bool GetIncludeExcludeValue(string value)
            {
                if (String.Compare(value, "INCLUDE", StringComparison.OrdinalIgnoreCase) == 0)
                    return true;
                else if (String.Compare(value, "EXCLUDE", StringComparison.OrdinalIgnoreCase) == 0)
                    return false;
                else
                    throw new ArgumentException(PublishStateHelperResources.BadChangeParameter);
            }
            /// <summary>
            /// Find finds a PublishStateChange that corresponds to the culture name identified in the given path
            /// </summary>
            /// <param name="path">The path to the resource file</param>
            /// <returns>A PublishStateChange that corresponds to the culture name in the given path or null if no PublishStateChange is found</returns>
            public PublishStateChange Find(string path)
            {
                foreach (PublishStateChange change in changes)
                {
                    if (path.StartsWith(change.CultureName + @"\"))
                        return change;
                }
                return null;
            }
        }
        /// <summary>
        /// SetPublishState applies a set of changes to a project file
        /// </summary>
        /// <param name="projectFilename">The filename of the project file to change</param>
        /// <param name="changes">A comma delimited list of changes to apply to the project file</param>
        /// <returns>true if changes were made to the project file. Otherwise false.</returns>
        /// <example>The list of changes could be:
        /// fr:Include, fr-FR:Include, fr-CA:Include, All:Exclude
        /// </example>
        public static bool SetPublishState(string projectFilename, string changes)
        {
            string[] changeArray = changes.Split(',');
            // remove leading spaces
            for (int change = 0; change < changeArray.GetLength(0); change++)
            {
                changeArray[change] = changeArray[change].TrimStart(' ');
            }
            return SetPublishState(projectFilename, changeArray);
        }
        /// <summary>
        /// SetPublishState applies a set of changes to a project file
        /// </summary>
        /// <param name="projectFilename">The filename of the project file to change</param>
        /// <param name="changes">An array of changes to apply to the project file</param>
        /// <returns>true if changes were made to the project file. Otherwise false.</returns>
        public static bool SetPublishState(string projectFilename, string[] changes)
        {
            return SetPublishState(projectFilename, new PublishStateChanges(changes));
        }
        /// <summary>
        /// SetPublishState applies a set of changes to a project file
        /// </summary>
        /// <param name="projectFilename">The filename of the project file to change</param>
        /// <param name="publishStateChanges">A PublishStateChanges that represent the changes to apply to the project file</param>
        /// <returns>true if changes were made to the project file. Otherwise false.</returns>
        public static bool SetPublishState(string projectFilename, PublishStateChanges publishStateChanges)
        {
            bool changed = false;
            XmlDocument xmlDocument = new XmlDocument();
            using (FileStream fileStream = new FileStream(projectFilename, FileMode.Open))
            {
                xmlDocument.Load(fileStream);

                XmlNamespaceManager xmlNamespaceManager = new XmlNamespaceManager(xmlDocument.NameTable);
                xmlNamespaceManager.AddNamespace("msbuild", "http://schemas.microsoft.com/developer/msbuild/2003");

                XmlNodeList xmlNodeList = xmlDocument.SelectNodes("/msbuild:Project/msbuild:ItemGroup/msbuild:PublishFile", xmlNamespaceManager);
                IEnumerator enumerator = xmlNodeList.GetEnumerator();
                while (enumerator.MoveNext())
                {
                    XmlNode xmlNode = (XmlNode)enumerator.Current;
                    if (UpdatePublishFile(xmlNode, publishStateChanges))
                        changed = true;
                }
            }
            if (changed)
            {
                xmlDocument.Save(projectFilename);
            }
            return changed;
        }

        private static bool UpdatePublishFile(XmlNode xmlNode, PublishStateChanges changes)
        {
            bool changed = false;
            PublishStateChange change = null;
            bool isIncludeAttributeValidCultureName = false;
            XmlAttributeCollection xmlAttributeCollection = xmlNode.Attributes;
            foreach (XmlAttribute xmlAttribute in xmlAttributeCollection)
            {
                if (xmlAttribute.Name == "Include")
                {
                    isIncludeAttributeValidCultureName = StartsWithCultureName(xmlAttribute.Value);
                    change = changes.Find(xmlAttribute.Value);
                    if (change != null)
                    {
                        break;
                    }
                }
            }
            if (change == null)
            {
                if (changes.AllParameterFound && isIncludeAttributeValidCultureName)
                {
                    changed = SetPublishState(xmlNode, changes.AllInclude);
                }
            }
            else
            {
                changed = SetPublishState(xmlNode, change.Include);
            }
            return changed;
        }
        private static bool StartsWithCultureName(string value)
        {
            int index = value.IndexOf(@"\");
            if (index == -1)
                return false;
            string cultureName = value.Substring(0, index);
            return CultureInfoHelper.IsValidCultureName(cultureName);
        }
        private static bool SetPublishState(XmlNode xmlNode, bool include)
        {
            bool changed = false;
            IEnumerator enumerator = xmlNode.ChildNodes.GetEnumerator();
            while (enumerator.MoveNext())
            {
                XmlNode xmlNodeChild = (XmlNode)enumerator.Current;
                if (xmlNodeChild.Name == "PublishState")
                {
                    if (include)
                    {
                        if (xmlNodeChild.InnerText != "Include")
                        {
                            xmlNodeChild.InnerText = "Include";
                            changed = true;
                        }
                    }
                    else if (xmlNodeChild.InnerText != "Exclude")
                    {
                        xmlNodeChild.InnerText = "Exclude";
                        changed = true;
                    }

                    break;
                }
            }
            return changed;
        }
    }
}
